/******/ (() => { // webpackBootstrap
/******/ 	var __webpack_modules__ = ([
/* 0 */,
/* 1 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "Renderer": () => (/* binding */ Renderer),
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__)
/* harmony export */ });
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(3);
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(7);
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(8);
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(9);
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(10);
/* harmony import */ var snabbdom__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(11);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(2);
/** @module @hat-open/renderer
 */






// patched version of snabbdom's modules/attributes.js
const snabbdomAttributesModule = (() => {
    function updateAttrs(oldVnode, vnode) {
        let key;
        const elm = vnode.elm;
        let oldAttrs = oldVnode.data.attrs;
        let attrs = vnode.data.attrs;
        if (!oldAttrs && !attrs)
            return;
        if (oldAttrs === attrs)
            return;
        oldAttrs = oldAttrs || {};
        attrs = attrs || {};
        for (key in attrs) {
            const cur = attrs[key];
            const old = oldAttrs[key];
            if (old !== cur) {
                if (cur === true) {
                    elm.setAttribute(key, "");
                }
                else if (cur === false) {
                    elm.removeAttribute(key);
                }
                else {
                    elm.setAttribute(key, cur);
                }
            }
        }
        for (key in oldAttrs) {
            if (!(key in attrs)) {
                elm.removeAttribute(key);
            }
        }
    }
    return { create: updateAttrs, update: updateAttrs };
})();


// patched version of snabbdom's modules/props.js
const snabbdomPropsModule = (() => {
    function updateProps(oldVnode, vnode) {
        let key;
        let cur;
        let old;
        const elm = vnode.elm;
        let oldProps = oldVnode.data.props;
        let props = vnode.data.props;
        if (!oldProps && !props)
            return;
        if (oldProps === props)
            return;
        oldProps = oldProps || {};
        props = props || {};
        for (key in oldProps) {
            if (!props[key]) {
                if (key === 'style') {
                    elm[key] = '';
                } else {
                    delete elm[key];
                }
            }
        }
        for (key in props) {
            cur = props[key];
            old = oldProps[key];
            if (old !== cur && (key !== 'value' || elm[key] !== cur)) {
                elm[key] = cur;
            }
        }
    }
    return { create: updateProps, update: updateProps };
})();


const patch = snabbdom__WEBPACK_IMPORTED_MODULE_1__.init([
    snabbdomAttributesModule,
    snabbdom__WEBPACK_IMPORTED_MODULE_2__.classModule,
    snabbdom__WEBPACK_IMPORTED_MODULE_3__.datasetModule,
    snabbdom__WEBPACK_IMPORTED_MODULE_4__.eventListenersModule,
    snabbdomPropsModule,
    snabbdom__WEBPACK_IMPORTED_MODULE_5__.styleModule
]);


function vhFromArray(node) {
    if (!node)
        return [];
    if (_hat_open_util__WEBPACK_IMPORTED_MODULE_0__.isString(node))
        return node;
    if (!_hat_open_util__WEBPACK_IMPORTED_MODULE_0__.isArray(node))
        throw 'Invalid node structure';
    if (node.length < 1)
        return [];
    if (typeof node[0] != 'string')
        return node.map(vhFromArray);
    const hasData = node.length > 1 && _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.isObject(node[1]);
    const children = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.pipe(
        _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.map(vhFromArray),
        _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.flatten,
        Array.from
    )(node.slice(hasData ? 2 : 1));
    const result = hasData ?
        snabbdom__WEBPACK_IMPORTED_MODULE_6__.h(node[0], node[1], children) :
        snabbdom__WEBPACK_IMPORTED_MODULE_6__.h(node[0], children);
    return result;
}

/**
 * Virtual DOM renderer
 */
class Renderer extends EventTarget {

    /**
     * Calls `init` method
     * @param {HTMLElement} [el=document.body]
     * @param {Any} [initState=null]
     * @param {Function} [vtCb=null]
     * @param {Number} [maxFps=30]
     */
    constructor(el, initState, vtCb, maxFps) {
        super();
        this.init(el, initState, vtCb, maxFps);
    }

    /**
     * Initialize renderer
     * @param {HTMLElement} [el=document.body]
     * @param {Any} [initState=null]
     * @param {Function} [vtCb=null]
     * @param {Number} [maxFps=30]
     * @return {Promise}
     */
    init(el, initState, vtCb, maxFps) {
        this._state = null;
        this._changes = [];
        this._promise = null;
        this._timeout = null;
        this._lastRender = null;
        this._vtCb = vtCb;
        this._maxFps = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.isNumber(maxFps) ? maxFps : 30;
        this._vNode = el || document.querySelector('body');
        if (initState)
            return this.change(_ => initState);
        return new Promise(resolve => { resolve(); });
    }

    /**
      * Render
      */
    render() {
        if (!this._vtCb)
            return;
        this._lastRender = performance.now();
        const vNode = vhFromArray(this._vtCb(this));
        patch(this._vNode, vNode);
        this._vNode = vNode;
        this.dispatchEvent(new CustomEvent('render', {detail: this._state}));
    }

    /**
     * Get current state value referenced by `paths`
     * @param {...Path} paths
     * @return {Any}
     */
    get(...paths) {
        return _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.get(paths, this._state);
    }

    /**
     * Change current state value referenced by `path`
     * @param {Path} path
     * @param {Any} value
     * @return {Promise}
     */
    set(path, value) {
        if (arguments.length < 2) {
            value = path;
            path = [];
        }
        return this.change(path, _ => value);
    }

    /**
     * Change current state value referenced by `path`
     * @param {Path} path
     * @param {Function} cb
     * @return {Promise}
     */
    change(path, cb) {
        if (arguments.length < 2) {
            cb = path;
            path = [];
        }
        this._changes.push([path, cb]);
        if (this._promise)
            return this._promise;
        this._promise = new Promise((resolve, reject) => {
            setTimeout(() => {
                try {
                    this._change();
                } catch(e) {
                    this._promise = null;
                    reject(e);
                    throw e;
                }
                this._promise = null;
                resolve();
            }, 0);
        });
        return this._promise;
    }

    _change() {
        while (this._changes.length > 0) {
            let change = false;
            while (this._changes.length > 0) {
                const [path, cb] = this._changes.shift();
                const view = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.get(path);
                const oldState = this._state;
                this._state = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.change(path, cb, this._state);
                if (this._state && _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.equals(view(oldState),
                                            view(this._state)))
                    continue;
                change = true;
                if (!this._vtCb || this._timeout)
                    continue;
                const delay = (!this._lastRender || !this._maxFps ?
                    0 :
                    (1000 / this._maxFps) -
                    (performance.now() - this._lastRender));
                this._timeout = setTimeout(() => {
                    this._timeout = null;
                    this.render();
                }, (delay > 0 ? delay : 0));
            }
            if (change)
                this.dispatchEvent(
                    new CustomEvent('change', {detail: this._state}));
        }
    }
}
// Renderer.prototype.set = u.curry(Renderer.prototype.set);
// Renderer.prototype.change = u.curry(Renderer.prototype.change);


/**
 * Default renderer
 * @static
 * @type {Renderer}
 */
const defaultRenderer = (() => {
    const r = (window && window.__hat_default_renderer) || new Renderer();
    if (window)
        window.__hat_default_renderer = r;
    return r;
})();
/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (defaultRenderer);


/***/ }),
/* 2 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "identity": () => (/* binding */ identity),
/* harmony export */   "isNil": () => (/* binding */ isNil),
/* harmony export */   "isBoolean": () => (/* binding */ isBoolean),
/* harmony export */   "isInteger": () => (/* binding */ isInteger),
/* harmony export */   "isNumber": () => (/* binding */ isNumber),
/* harmony export */   "isString": () => (/* binding */ isString),
/* harmony export */   "isArray": () => (/* binding */ isArray),
/* harmony export */   "isObject": () => (/* binding */ isObject),
/* harmony export */   "strictParseInt": () => (/* binding */ strictParseInt),
/* harmony export */   "strictParseFloat": () => (/* binding */ strictParseFloat),
/* harmony export */   "clone": () => (/* binding */ clone),
/* harmony export */   "zip": () => (/* binding */ zip),
/* harmony export */   "toPairs": () => (/* binding */ toPairs),
/* harmony export */   "fromPairs": () => (/* binding */ fromPairs),
/* harmony export */   "flatten": () => (/* binding */ flatten),
/* harmony export */   "pipe": () => (/* binding */ pipe),
/* harmony export */   "flap": () => (/* binding */ flap),
/* harmony export */   "curry": () => (/* binding */ curry),
/* harmony export */   "equals": () => (/* binding */ equals),
/* harmony export */   "repeat": () => (/* binding */ repeat),
/* harmony export */   "get": () => (/* binding */ get),
/* harmony export */   "change": () => (/* binding */ change),
/* harmony export */   "set": () => (/* binding */ set),
/* harmony export */   "omit": () => (/* binding */ omit),
/* harmony export */   "move": () => (/* binding */ move),
/* harmony export */   "sort": () => (/* binding */ sort),
/* harmony export */   "sortBy": () => (/* binding */ sortBy),
/* harmony export */   "pick": () => (/* binding */ pick),
/* harmony export */   "map": () => (/* binding */ map),
/* harmony export */   "filter": () => (/* binding */ filter),
/* harmony export */   "append": () => (/* binding */ append),
/* harmony export */   "reduce": () => (/* binding */ reduce),
/* harmony export */   "merge": () => (/* binding */ merge),
/* harmony export */   "mergeAll": () => (/* binding */ mergeAll),
/* harmony export */   "find": () => (/* binding */ find),
/* harmony export */   "findIndex": () => (/* binding */ findIndex),
/* harmony export */   "concat": () => (/* binding */ concat),
/* harmony export */   "union": () => (/* binding */ union),
/* harmony export */   "contains": () => (/* binding */ contains),
/* harmony export */   "insert": () => (/* binding */ insert),
/* harmony export */   "slice": () => (/* binding */ slice),
/* harmony export */   "reverse": () => (/* binding */ reverse),
/* harmony export */   "length": () => (/* binding */ length),
/* harmony export */   "inc": () => (/* binding */ inc),
/* harmony export */   "dec": () => (/* binding */ dec),
/* harmony export */   "not": () => (/* binding */ not),
/* harmony export */   "sleep": () => (/* binding */ sleep),
/* harmony export */   "delay": () => (/* binding */ delay)
/* harmony export */ });
/**
 * Utility library for manipulation of JSON data.
 *
 * Main characteristics:
 *   - input/output data types are limited to JSON data, functions and
 *     `undefined` (sparse arrays and complex objects with prototype chain are
 *     not supported)
 *   - functional API with curried functions (similar to ramdajs)
 *   - implementation based on natively supported browser JS API
 *   - scope limited to most used functions in hat projects
 *   - usage of `paths` instead of `lenses`
 *
 * TODO: define convetion for naming arguments based on their type and
 *       semantics
 *
 * @module @hat-open/util
 */

/**
 * Path can be an object property name, array index, or array of Paths
 *
 * TODO: explain paths and path compositions (include examples)
 *
 * @typedef {(String|Number|Path[])} module:@hat-open/util.Path
 */

/**
 * Identity function returning same value provided as argument.
 *
 * @function
 * @sig a -> a
 * @param {*} x input value
 * @return {*} same value as input
 */
const identity = x => x;

/**
 * Check if value is `null` or `undefined`.
 *
 * For same argument, if this function returns `true`, functions `isBoolean`,
 * `isInteger`, `isNumber`, `isString`, `isArray` and `isObject` will return
 * `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @return {Boolean}
 */
const isNil = x => x == null;

/**
 * Check if value is Boolean.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isInteger`, `isNumber`, `isString`, `isArray` and `isObject` will return
 * `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @return {Boolean}
 */
const isBoolean = x => typeof(x) == 'boolean';

/**
 * Check if value is Integer.
 *
 * For same argument, if this function returns `true`, function `isNumber` will
 * also return `true`.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isBoolean`, `isString`, `isArray` and `isObject` will return `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @type {Boolean}
 */
const isInteger = Number.isInteger;

/**
 * Check if value is Number.
 *
 * For same argument, if this function returns `true`, function `isInteger` may
 * also return `true` if argument is integer number.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isBoolean`, `isString`, `isArray` and `isObject` will return `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @return {Boolean}
 */
const isNumber = x => typeof(x) == 'number';

/**
 * Check if value is String.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isBoolean`, `isInteger`, `isNumber`, `isArray`, and `isObject` will return
 * `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {Any} x input value
 * @type {Boolean}
 */
const isString = x => typeof(x) == 'string';

/**
 * Check if value is Array.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isBoolean`, `isInteger`, `isNumber`, `isString`, and `isObject` will return
 * `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @return {Boolean}
 */
const isArray = Array.isArray;

/**
 * Check if value is Object.
 *
 * For same argument, if this function returns `true`, functions `isNil`,
 * `isBoolean`, `isInteger`, `isNumber`, `isString`, and `isArray` will return
 * `false`.
 *
 * @function
 * @sig * -> Boolean
 * @param {*} x input value
 * @return {Boolean}
 */
const isObject = x => typeof(x) == 'object' &&
                             !isArray(x) &&
                             !isNil(x);

/**
 * Strictly parse integer from string
 *
 * If provided string doesn't represent integer value, `NaN` is returned.
 *
 * @function
 * @sig String -> Number
 * @param {String} value
 * @return {Number}
 */
function strictParseInt(value) {
    if (/^(-|\+)?([0-9]+)$/.test(value))
        return Number(value);
    return NaN;
}

/**
 * Strictly parse floating point number from string
 *
 * If provided string doesn't represent valid number, `NaN` is returned.
 *
 * @function
 * @sig String -> Number
 * @param {String} value
 * @return {Number}
 */
function strictParseFloat(value) {
    if (/^(-|\+)?([0-9]+(\.[0-9]+)?)$/.test(value))
        return Number(value);
    return NaN;
}

/**
 * Create new deep copy of input value.
 *
 * In case of Objects or Arrays, new instances are created with elements
 * obtained by recursivly calling `clone` in input argument values.
 *
 * @function
 * @sig * -> *
 * @param {*} x value
 * @return {*} copy of value
 */
function clone(x) {
    if (isArray(x))
        return Array.from(x, clone);
    if (isObject(x)) {
        let ret = {};
        for (let i in x)
            ret[i] = clone(x[i]);
        return ret;
    }
    return x;
}

/**
 * Combine two arrays in single array of pairs
 *
 * The returned array is truncated to the length of the shorter of the two
 * input arrays.
 *
 * @function
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} arr1
 * @param {Array} arr2
 * @return {Array}
 */
function zip(arr1, arr2) {
    return Array.from((function*() {
        for (let i = 0; i < arr1.length || i < arr2.length; ++i)
            yield [arr1[i], arr2[i]];
    })());
}

/**
 * Convert object to array of key, value pairs
 *
 * @function
 * @sig Object -> [[String,*]]
 * @param {Object} obj
 * @return {Array}
 */
function toPairs(obj) {
    return Object.entries(obj);
}

/**
 * Convert array of key, value pairs to object
 *
 * @function
 * @sig [[String,*]] -> Object
 * @param {Array} arr
 * @return {Object}
 */
function fromPairs(arr) {
    let ret = {};
    for (let [k, v] of arr)
        ret[k] = v;
    return ret;
}

/**
 * Flatten nested arrays.
 *
 * Create array with same elements as in input array where all elements which
 * are also arrays are replaced with elements of resulting recursive
 * application of flatten function.
 *
 * If argument is not an array, function returns the argument encapsulated in
 * an array.
 *
 * @function
 * @sig [a] -> [b]
 * @param {*} arr
 * @return {Array}
 */
function flatten(arr) {
    return isArray(arr) ? arr.flat(Infinity) : [arr];
}

/**
 * Pipe function calls
 *
 * Pipe provides functional composition with reversed order. First function
 * may have any arity and all other functions are called with only single
 * argument (result from previous function application).
 *
 * In case when no function is provided, pipe returns identity function.
 *
 * @function
 * @sig (((a1, a2, ..., an) -> b1), (b1 -> b2), ..., (bm1 -> bm)) -> ((a1, a2, ..., an) -> bm)
 * @param {...Function} fns functions
 * @return {Function}
 */
function pipe(...fns) {
    if (fns.length < 1)
        return identity;
    return function (...args) {
        let ret = fns[0].apply(this, args);
        for (let fn of fns.slice(1))
            ret = fn(ret);
        return ret;
    };
}

/**
 * Apply list of functions to same arguments and return list of results
 *
 * @function
 * @sig ((a1 -> ... -> an -> b1), ..., (a1 -> ... -> an -> bm)) -> (a1 -> ... -> an -> [b1,...,bm])
 * @param {...Function} fns functions
 * @return {Function}
 */
function flap(...fns) {
    return (...args) => fns.map(fn => fn.apply(this, args));
}

/**
 * Curry function with fixed arguments lenth
 *
 * Function arity is determined based on function's length property.
 *
 * @function
 * @sig (* -> a) -> (* -> a)
 * @param {Function} fn
 * @return {Function}
 */
function curry(fn) {
    let wrapper = function(oldArgs) {
        return function(...args) {
            args = oldArgs.concat(args);
            if (args.length >= fn.length)
                return fn(...args);
            return wrapper(args);
        };
    };
    return wrapper([]);
}

/**
 * Deep object equality
 * (curried function)
 *
 * @function
 * @sig a -> b -> Boolean
 * @param {*} x
 * @param {*} y
 * @return {Boolean}
 */
const equals = curry((x, y) => {
    if (x === y)
        return true;
    if (typeof(x) != 'object' ||
        typeof(y) != 'object' ||
        x === null ||
        y === null)
        return false;
    if (Array.isArray(x) && Array.isArray(y)) {
        if (x.length != y.length)
            return false;
        for (let [a, b] of zip(x, y)) {
            if (!equals(a, b))
                return false;
        }
        return true;
    } else if (!Array.isArray(x) && !Array.isArray(y)) {
        if (Object.keys(x).length != Object.keys(y).length)
            return false;
        for (let key in x) {
            if (!(key in y))
                return false;
        }
        for (let key in x) {
            if (!equals(x[key], y[key]))
                return false;
        }
        return true;
    }
    return false;
});


/**
 * Create array by repeating same value
 * (curried function)
 *
 * @function
 * @sig a -> Number -> [a]
 * @param {*} x
 * @param {Number} n
 * @return {Array}
 */
const repeat = curry((x, n) => Array.from({length: n}, _ => x));

/**
 * Get value referenced by path
 * (curried function)
 *
 * If input value doesn't contain provided path value, `undefined` is returned.
 *
 * @function
 * @sig Path -> a -> b
 * @param {Path} path
 * @param {*} x
 * @return {*}
 */
const get = curry((path, x) => {
    let ret = x;
    for (let i of flatten(path)) {
        if (ret === null || typeof(ret) != 'object')
            return undefined;
        ret = ret[i];
    }
    return ret;
});

/**
 * Change value referenced with path by appling function
 * (curried function)
 *
 * @function
 * @sig Path -> (a -> b) -> c -> c
 * @param {Path} path
 * @param {Function} fn
 * @param {*} x
 * @return {*}
 */
const change = curry((path, fn, x) => {
    return (function change(path, x) {
        if (path.length < 1)
            return fn(x);
        const [first, ...rest] = path;
        if (isInteger(first)) {
            x = (isArray(x) ? Array.from(x) : repeat(undefined, first));
        } else if (isString(first)) {
            x = (isObject(x) ? Object.assign({}, x) : {});
        } else {
            throw 'invalid path';
        }
        x[first] = change(rest, x[first]);
        return x;
    })(flatten(path), x);
});

/**
 * Replace value referenced with path with another value
 * (curried function)
 *
 * @function
 * @sig Path -> (a -> b) -> c -> c
 * @param {Path} path
 * @param {*} val
 * @param {*} x
 * @return {*}
 */
const set = curry((path, val, x) => change(path, _ => val, x));

/**
 * Omitting value referenced by path
 * (curried function)
 *
 * @function
 * @sig Path -> a -> a
 * @param {Path} path
 * @param {*} x
 * @return {*}
 */
const omit = curry((path, x) => {
    function _omit(path, x) {
        if (isInteger(path[0])) {
            x = (isArray(x) ? Array.from(x) : []);
        } else if (isString(path[0])) {
            x = (isObject(x) ? Object.assign({}, x) : {});
        } else {
            throw 'invalid path';
        }
        if (path.length > 1) {
            x[path[0]] = _omit(path.slice(1), x[path[0]]);
        } else if (isInteger(path[0])) {
            x.splice(path[0], 1);
        } else {
            delete x[path[0]];
        }
        return x;
    }
    path = flatten(path);
    if (path.length < 1)
        return undefined;
    return _omit(path, x);
});

/**
 * Change by moving value from source path to destination path
 * (curried function)
 *
 * @function
 * @sig Path -> Path -> a -> a
 * @param {Path} srcPath
 * @param {Path} dstPath
 * @param {*} x
 * @return {*}
 */
const move = curry((srcPath, dstPath, x) => pipe(
    set(dstPath, get(srcPath, x)),
    omit(srcPath)
)(x));

/**
 * Sort array
 * (curried function)
 *
 * Comparison function receives two arguments representing array elements and
 * should return:
 *   - negative number in case first argument is more significant then second
 *   - zero in case first argument is equaly significant as second
 *   - positive number in case first argument is less significant then second
 *
 * @function
 * @sig ((a, a) -> Number) -> [a] -> [a]
 * @param {Function} fn
 * @param {Array} arr
 * @return {Array}
 */
const sort = curry((fn, arr) => Array.from(arr).sort(fn));

/**
 * Sort array based on results of appling function to it's elements
 * (curried function)
 *
 * Resulting order is determined by comparring function application results
 * with greater then and lesser then operators.
 *
 * @function
 * @sig (a -> b) -> [a] -> [a]
 * @param {Function} fn
 * @param {Array} arr
 * @return {Array}
 */
const sortBy = curry((fn, arr) => sort((x, y) => {
    const xVal = fn(x);
    const yVal = fn(y);
    if (xVal < yVal)
        return -1;
    if (xVal > yVal)
        return 1;
    return 0;
}, arr));

/**
 * Create object containing only subset of selected properties
 * (curried function)
 *
 * @function
 * @sig [String] -> a -> a
 * @param {Array} arr
 * @param {Object} obj
 * @return {Object}
 */
const pick = curry((arr, obj) => {
    const ret = {};
    for (let i of arr)
        if (i in obj)
            ret[i] = obj[i];
    return ret;
});

/**
 * Change array or object by appling function to it's elements
 * (curried function)
 *
 * For each element, provided function is called with element value,
 * index/key and original container.
 *
 * @function
 * @sig ((a, Number, [a]) -> b) -> [a] -> [b]
 * @sig ((a, String, {String: a}) -> b) -> {String: a} -> {String: b}
 * @param {Function} fn
 * @param {Array|Object} x
 * @return {Array|Object}
 */
const map = curry((fn, x) => {
    if (isArray(x))
        return x.map(fn);
    const res = {};
    for (let k in x)
        res[k] = fn(x[k], k, x);
    return res;
});

/**
 * Change array to contain only elements for which function returns `true`
 * (curried function)
 *
 * @function
 * @sig (a -> Boolean) -> [a] -> [a]
 * @param {Function} fn
 * @param {Array} arr
 * @return {Array}
 */
const filter = curry((fn, arr) => arr.filter(fn));

/**
 * Append value to end of array
 * (curried function)
 *
 * @function
 * @sig a -> [a] -> [a]
 * @param {*} val
 * @param {Array} arr
 * @return {Array}
 */
const append = curry((val, arr) => arr.concat([val]));

/**
 * Reduce array or object by appling function
 * (curried function)
 *
 * For each element, provided function is called with accumulator,
 * elements value, element index/key and original container.
 *
 * @function
 * @sig ((b, a, Number, [a]) -> b) -> b -> [a] -> b
 * @sig ((b, a, String, {String: a}) -> b) -> b -> {String: a} -> b
 * @param {Function} fn
 * @param {*} val initial accumulator value
 * @param {Array|Object} x
 * @return {*} reduced value
 */
const reduce = curry((fn, val, x) => {
    if (isArray(x))
        return x.reduce(fn, val);
    let acc = val;
    for (let k in x)
        acc = fn(acc, x[k], k, x);
    return acc;
});

/**
 * Merge two objects
 * (curried function)
 *
 * If same property exist in both arguments, second argument's value is used
 * as resulting value
 *
 * @function
 * @sig a -> a -> a
 * @param {Object} x
 * @param {Object} y
 * @return {Object}
 */
const merge = curry((x, y) => Object.assign({}, x, y));

/**
 * Merge multiple objects
 * (curried function)
 *
 * If same property exist in multiple arguments, value from the last argument
 * containing that property is used
 *
 * @function
 * @sig [a] -> a
 * @param {Object[]}
 * @return {Object}
 */
const mergeAll = reduce(merge, {});

/**
 * Find element in array or object for which provided function returns `true`
 * (curried function)
 *
 * Until element is found, provided function is called for each element with
 * arguments: current element, current index/key and initial container.
 *
 * If searched element is not found, `undefined` is returned.
 *
 * @function
 * @sig ((a, Number, [a]) -> Boolean) -> [a] -> a
 * @sig ((a, String, {String: a}) -> Boolean) -> {String: a} -> a
 * @param {Function} fn
 * @param {Array|Object} x
 * @return {*}
 */
const find = curry((fn, x) => {
    if (isArray(x))
        return x.find(fn);
    for (let k in x)
        if (fn(x[k], k, x))
            return x[k];
});

/**
 * Find element's index/key in array or object for which provided function
 * returns `true`
 * (curried function)
 *
 * Until element is found, provided function is called for each element with
 * arguments: current element, current index/key and initial container.
 *
 * If searched element is not found, `undefined` is returned.
 *
 * @function
 * @sig ((a, Number, [a]) -> Boolean) -> [a] -> a
 * @sig ((a, String, {String: a}) -> Boolean) -> {String: a} -> a
 * @param {Function} fn
 * @param {Array|Object} x
 * @return {*}
 */
const findIndex = curry((fn, x) => {
    if (isArray(x))
        return x.findIndex(fn);
    for (let k in x)
        if (fn(x[k], k, x))
            return k;
});

/**
 * Concatenate two arrays
 * (curried function)
 *
 * @function
 * @sig [a] -> [a] -> [a]
 * @param {Array} x
 * @param {Array} y
 * @return {Array}
 */
const concat = curry((x, y) => x.concat(y));

/**
 * Create union of two arrays using `equals` to check equality
 * (curried function)
 *
 * @function
 * @sig [a] -> [a] -> [a]
 * @param {Array} x
 * @param {Array} y
 * @return {Array}
 */
const union = curry((x, y) => {
    return reduce((acc, val) => {
        if (!find(equals(val), x))
            acc = append(val, acc);
        return acc;
    }, x, y);
});

/**
 * Check if array contains value
 * (curried function)
 *
 * TODO: add support for objects (should we check for keys or values?)
 *
 * @function
 * @sig a -> [a] -> Boolean
 * @param {*} val
 * @param {Array|Object} x
 * @return {Boolean}
 */
const contains = curry((val, arr) => arr.includes(val));

/**
 * Insert value into array on specified index
 * (curried function)
 *
 * @function
 * @sig Number -> a -> [a] -> [a]
 * @param {Number} idx
 * @param {*} val
 * @param {Array} arr
 * @return {Array}
 */
const insert = curry((idx, val, arr) =>
    arr.slice(0, idx).concat([val], arr.slice(idx)));

/**
 * Get array slice
 * (curried function)
 *
 * @function
 * @sig Number -> Number -> [a] -> [a]
 * @param {Number} begin
 * @param {Number} end
 * @param {Array} arr
 * @return {Array}
 */
const slice = curry((begin, end, arr) => arr.slice(begin, end));

/**
 * Reverse array
 *
 * @function
 * @sig [a] -> [a]
 * @param  {Array} arr
 * @return {Array}
 */
function reverse(arr) {
    return Array.from(arr).reverse();
}

/**
 * Array length
 *
 * @function
 * @sig [a] -> Number
 * @param  {Array} arr
 * @return {Number}
 */
function length(arr) {
    return arr.length;
}

/**
 * Increment value
 * @param  {Number} val
 * @return {Number}
 */
function inc(val) {
    return val + 1;
}

/**
 * Decrement value
 * @param  {Number} val
 * @return {Number}
 */
function dec(val) {
    return val - 1;
}

/**
 * Logical not
 * @param  {Any} val
 * @return {Boolean}
 */
function not(val) {
    return !val;
}

/**
 * Create promise that resolves in `t` milliseconds
 *
 * TODO: move to other module
 *
 * @function
 * @sig Number -> Promise
 * @param {Number} t
 * @return {Promise}
 */
function sleep(t) {
    return new Promise(resolve => {
        setTimeout(() => { resolve(); }, t);
    });
}

/**
 * Delay function call `fn(...args)` for `t` milliseconds
 *
 * TODO: move to other module
 *
 * @function
 * @sig (((a1, a2, ..., an) -> _), Number, a1, a2, ..., an) -> Promise
 * @param {Function} fn
 * @param {Number} [t=0]
 * @param {*} args
 * @return {Promise}
 */
function delay(fn, t, ...args) {
    return new Promise(resolve => {
        setTimeout(() => { resolve(fn(...args)); }, t || 0);
    });
}


/***/ }),
/* 3 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "init": () => (/* binding */ init)
/* harmony export */ });
/* harmony import */ var _vnode__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(4);
/* harmony import */ var _is__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(6);
/* harmony import */ var _htmldomapi__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(5);



function isUndef(s) {
    return s === undefined;
}
function isDef(s) {
    return s !== undefined;
}
const emptyNode = (0,_vnode__WEBPACK_IMPORTED_MODULE_0__.vnode)("", {}, [], undefined, undefined);
function sameVnode(vnode1, vnode2) {
    var _a, _b;
    const isSameKey = vnode1.key === vnode2.key;
    const isSameIs = ((_a = vnode1.data) === null || _a === void 0 ? void 0 : _a.is) === ((_b = vnode2.data) === null || _b === void 0 ? void 0 : _b.is);
    const isSameSel = vnode1.sel === vnode2.sel;
    return isSameSel && isSameKey && isSameIs;
}
function isVnode(vnode) {
    return vnode.sel !== undefined;
}
function createKeyToOldIdx(children, beginIdx, endIdx) {
    var _a;
    const map = {};
    for (let i = beginIdx; i <= endIdx; ++i) {
        const key = (_a = children[i]) === null || _a === void 0 ? void 0 : _a.key;
        if (key !== undefined) {
            map[key] = i;
        }
    }
    return map;
}
const hooks = [
    "create",
    "update",
    "remove",
    "destroy",
    "pre",
    "post",
];
function init(modules, domApi) {
    const cbs = {
        create: [],
        update: [],
        remove: [],
        destroy: [],
        pre: [],
        post: [],
    };
    const api = domApi !== undefined ? domApi : _htmldomapi__WEBPACK_IMPORTED_MODULE_1__.htmlDomApi;
    for (const hook of hooks) {
        for (const module of modules) {
            const currentHook = module[hook];
            if (currentHook !== undefined) {
                cbs[hook].push(currentHook);
            }
        }
    }
    function emptyNodeAt(elm) {
        const id = elm.id ? "#" + elm.id : "";
        // elm.className doesn't return a string when elm is an SVG element inside a shadowRoot.
        // https://stackoverflow.com/questions/29454340/detecting-classname-of-svganimatedstring
        const classes = elm.getAttribute("class");
        const c = classes ? "." + classes.split(" ").join(".") : "";
        return (0,_vnode__WEBPACK_IMPORTED_MODULE_0__.vnode)(api.tagName(elm).toLowerCase() + id + c, {}, [], undefined, elm);
    }
    function createRmCb(childElm, listeners) {
        return function rmCb() {
            if (--listeners === 0) {
                const parent = api.parentNode(childElm);
                api.removeChild(parent, childElm);
            }
        };
    }
    function createElm(vnode, insertedVnodeQueue) {
        var _a, _b;
        let i;
        let data = vnode.data;
        if (data !== undefined) {
            const init = (_a = data.hook) === null || _a === void 0 ? void 0 : _a.init;
            if (isDef(init)) {
                init(vnode);
                data = vnode.data;
            }
        }
        const children = vnode.children;
        const sel = vnode.sel;
        if (sel === "!") {
            if (isUndef(vnode.text)) {
                vnode.text = "";
            }
            vnode.elm = api.createComment(vnode.text);
        }
        else if (sel !== undefined) {
            // Parse selector
            const hashIdx = sel.indexOf("#");
            const dotIdx = sel.indexOf(".", hashIdx);
            const hash = hashIdx > 0 ? hashIdx : sel.length;
            const dot = dotIdx > 0 ? dotIdx : sel.length;
            const tag = hashIdx !== -1 || dotIdx !== -1
                ? sel.slice(0, Math.min(hash, dot))
                : sel;
            const elm = (vnode.elm =
                isDef(data) && isDef((i = data.ns))
                    ? api.createElementNS(i, tag, data)
                    : api.createElement(tag, data));
            if (hash < dot)
                elm.setAttribute("id", sel.slice(hash + 1, dot));
            if (dotIdx > 0)
                elm.setAttribute("class", sel.slice(dot + 1).replace(/\./g, " "));
            for (i = 0; i < cbs.create.length; ++i)
                cbs.create[i](emptyNode, vnode);
            if (_is__WEBPACK_IMPORTED_MODULE_2__.array(children)) {
                for (i = 0; i < children.length; ++i) {
                    const ch = children[i];
                    if (ch != null) {
                        api.appendChild(elm, createElm(ch, insertedVnodeQueue));
                    }
                }
            }
            else if (_is__WEBPACK_IMPORTED_MODULE_2__.primitive(vnode.text)) {
                api.appendChild(elm, api.createTextNode(vnode.text));
            }
            const hook = vnode.data.hook;
            if (isDef(hook)) {
                (_b = hook.create) === null || _b === void 0 ? void 0 : _b.call(hook, emptyNode, vnode);
                if (hook.insert) {
                    insertedVnodeQueue.push(vnode);
                }
            }
        }
        else {
            vnode.elm = api.createTextNode(vnode.text);
        }
        return vnode.elm;
    }
    function addVnodes(parentElm, before, vnodes, startIdx, endIdx, insertedVnodeQueue) {
        for (; startIdx <= endIdx; ++startIdx) {
            const ch = vnodes[startIdx];
            if (ch != null) {
                api.insertBefore(parentElm, createElm(ch, insertedVnodeQueue), before);
            }
        }
    }
    function invokeDestroyHook(vnode) {
        var _a, _b;
        const data = vnode.data;
        if (data !== undefined) {
            (_b = (_a = data === null || data === void 0 ? void 0 : data.hook) === null || _a === void 0 ? void 0 : _a.destroy) === null || _b === void 0 ? void 0 : _b.call(_a, vnode);
            for (let i = 0; i < cbs.destroy.length; ++i)
                cbs.destroy[i](vnode);
            if (vnode.children !== undefined) {
                for (let j = 0; j < vnode.children.length; ++j) {
                    const child = vnode.children[j];
                    if (child != null && typeof child !== "string") {
                        invokeDestroyHook(child);
                    }
                }
            }
        }
    }
    function removeVnodes(parentElm, vnodes, startIdx, endIdx) {
        var _a, _b;
        for (; startIdx <= endIdx; ++startIdx) {
            let listeners;
            let rm;
            const ch = vnodes[startIdx];
            if (ch != null) {
                if (isDef(ch.sel)) {
                    invokeDestroyHook(ch);
                    listeners = cbs.remove.length + 1;
                    rm = createRmCb(ch.elm, listeners);
                    for (let i = 0; i < cbs.remove.length; ++i)
                        cbs.remove[i](ch, rm);
                    const removeHook = (_b = (_a = ch === null || ch === void 0 ? void 0 : ch.data) === null || _a === void 0 ? void 0 : _a.hook) === null || _b === void 0 ? void 0 : _b.remove;
                    if (isDef(removeHook)) {
                        removeHook(ch, rm);
                    }
                    else {
                        rm();
                    }
                }
                else {
                    // Text node
                    api.removeChild(parentElm, ch.elm);
                }
            }
        }
    }
    function updateChildren(parentElm, oldCh, newCh, insertedVnodeQueue) {
        let oldStartIdx = 0;
        let newStartIdx = 0;
        let oldEndIdx = oldCh.length - 1;
        let oldStartVnode = oldCh[0];
        let oldEndVnode = oldCh[oldEndIdx];
        let newEndIdx = newCh.length - 1;
        let newStartVnode = newCh[0];
        let newEndVnode = newCh[newEndIdx];
        let oldKeyToIdx;
        let idxInOld;
        let elmToMove;
        let before;
        while (oldStartIdx <= oldEndIdx && newStartIdx <= newEndIdx) {
            if (oldStartVnode == null) {
                oldStartVnode = oldCh[++oldStartIdx]; // Vnode might have been moved left
            }
            else if (oldEndVnode == null) {
                oldEndVnode = oldCh[--oldEndIdx];
            }
            else if (newStartVnode == null) {
                newStartVnode = newCh[++newStartIdx];
            }
            else if (newEndVnode == null) {
                newEndVnode = newCh[--newEndIdx];
            }
            else if (sameVnode(oldStartVnode, newStartVnode)) {
                patchVnode(oldStartVnode, newStartVnode, insertedVnodeQueue);
                oldStartVnode = oldCh[++oldStartIdx];
                newStartVnode = newCh[++newStartIdx];
            }
            else if (sameVnode(oldEndVnode, newEndVnode)) {
                patchVnode(oldEndVnode, newEndVnode, insertedVnodeQueue);
                oldEndVnode = oldCh[--oldEndIdx];
                newEndVnode = newCh[--newEndIdx];
            }
            else if (sameVnode(oldStartVnode, newEndVnode)) {
                // Vnode moved right
                patchVnode(oldStartVnode, newEndVnode, insertedVnodeQueue);
                api.insertBefore(parentElm, oldStartVnode.elm, api.nextSibling(oldEndVnode.elm));
                oldStartVnode = oldCh[++oldStartIdx];
                newEndVnode = newCh[--newEndIdx];
            }
            else if (sameVnode(oldEndVnode, newStartVnode)) {
                // Vnode moved left
                patchVnode(oldEndVnode, newStartVnode, insertedVnodeQueue);
                api.insertBefore(parentElm, oldEndVnode.elm, oldStartVnode.elm);
                oldEndVnode = oldCh[--oldEndIdx];
                newStartVnode = newCh[++newStartIdx];
            }
            else {
                if (oldKeyToIdx === undefined) {
                    oldKeyToIdx = createKeyToOldIdx(oldCh, oldStartIdx, oldEndIdx);
                }
                idxInOld = oldKeyToIdx[newStartVnode.key];
                if (isUndef(idxInOld)) {
                    // New element
                    api.insertBefore(parentElm, createElm(newStartVnode, insertedVnodeQueue), oldStartVnode.elm);
                }
                else {
                    elmToMove = oldCh[idxInOld];
                    if (elmToMove.sel !== newStartVnode.sel) {
                        api.insertBefore(parentElm, createElm(newStartVnode, insertedVnodeQueue), oldStartVnode.elm);
                    }
                    else {
                        patchVnode(elmToMove, newStartVnode, insertedVnodeQueue);
                        oldCh[idxInOld] = undefined;
                        api.insertBefore(parentElm, elmToMove.elm, oldStartVnode.elm);
                    }
                }
                newStartVnode = newCh[++newStartIdx];
            }
        }
        if (oldStartIdx <= oldEndIdx || newStartIdx <= newEndIdx) {
            if (oldStartIdx > oldEndIdx) {
                before = newCh[newEndIdx + 1] == null ? null : newCh[newEndIdx + 1].elm;
                addVnodes(parentElm, before, newCh, newStartIdx, newEndIdx, insertedVnodeQueue);
            }
            else {
                removeVnodes(parentElm, oldCh, oldStartIdx, oldEndIdx);
            }
        }
    }
    function patchVnode(oldVnode, vnode, insertedVnodeQueue) {
        var _a, _b, _c, _d, _e;
        const hook = (_a = vnode.data) === null || _a === void 0 ? void 0 : _a.hook;
        (_b = hook === null || hook === void 0 ? void 0 : hook.prepatch) === null || _b === void 0 ? void 0 : _b.call(hook, oldVnode, vnode);
        const elm = (vnode.elm = oldVnode.elm);
        const oldCh = oldVnode.children;
        const ch = vnode.children;
        if (oldVnode === vnode)
            return;
        if (vnode.data !== undefined) {
            for (let i = 0; i < cbs.update.length; ++i)
                cbs.update[i](oldVnode, vnode);
            (_d = (_c = vnode.data.hook) === null || _c === void 0 ? void 0 : _c.update) === null || _d === void 0 ? void 0 : _d.call(_c, oldVnode, vnode);
        }
        if (isUndef(vnode.text)) {
            if (isDef(oldCh) && isDef(ch)) {
                if (oldCh !== ch)
                    updateChildren(elm, oldCh, ch, insertedVnodeQueue);
            }
            else if (isDef(ch)) {
                if (isDef(oldVnode.text))
                    api.setTextContent(elm, "");
                addVnodes(elm, null, ch, 0, ch.length - 1, insertedVnodeQueue);
            }
            else if (isDef(oldCh)) {
                removeVnodes(elm, oldCh, 0, oldCh.length - 1);
            }
            else if (isDef(oldVnode.text)) {
                api.setTextContent(elm, "");
            }
        }
        else if (oldVnode.text !== vnode.text) {
            if (isDef(oldCh)) {
                removeVnodes(elm, oldCh, 0, oldCh.length - 1);
            }
            api.setTextContent(elm, vnode.text);
        }
        (_e = hook === null || hook === void 0 ? void 0 : hook.postpatch) === null || _e === void 0 ? void 0 : _e.call(hook, oldVnode, vnode);
    }
    return function patch(oldVnode, vnode) {
        let i, elm, parent;
        const insertedVnodeQueue = [];
        for (i = 0; i < cbs.pre.length; ++i)
            cbs.pre[i]();
        if (!isVnode(oldVnode)) {
            oldVnode = emptyNodeAt(oldVnode);
        }
        if (sameVnode(oldVnode, vnode)) {
            patchVnode(oldVnode, vnode, insertedVnodeQueue);
        }
        else {
            elm = oldVnode.elm;
            parent = api.parentNode(elm);
            createElm(vnode, insertedVnodeQueue);
            if (parent !== null) {
                api.insertBefore(parent, vnode.elm, api.nextSibling(elm));
                removeVnodes(parent, [oldVnode], 0, 0);
            }
        }
        for (i = 0; i < insertedVnodeQueue.length; ++i) {
            insertedVnodeQueue[i].data.hook.insert(insertedVnodeQueue[i]);
        }
        for (i = 0; i < cbs.post.length; ++i)
            cbs.post[i]();
        return vnode;
    };
}
//# sourceMappingURL=init.js.map

/***/ }),
/* 4 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "vnode": () => (/* binding */ vnode)
/* harmony export */ });
function vnode(sel, data, children, text, elm) {
    const key = data === undefined ? undefined : data.key;
    return { sel, data, children, text, elm, key };
}
//# sourceMappingURL=vnode.js.map

/***/ }),
/* 5 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "htmlDomApi": () => (/* binding */ htmlDomApi)
/* harmony export */ });
function createElement(tagName, options) {
    return document.createElement(tagName, options);
}
function createElementNS(namespaceURI, qualifiedName, options) {
    return document.createElementNS(namespaceURI, qualifiedName, options);
}
function createTextNode(text) {
    return document.createTextNode(text);
}
function createComment(text) {
    return document.createComment(text);
}
function insertBefore(parentNode, newNode, referenceNode) {
    parentNode.insertBefore(newNode, referenceNode);
}
function removeChild(node, child) {
    node.removeChild(child);
}
function appendChild(node, child) {
    node.appendChild(child);
}
function parentNode(node) {
    return node.parentNode;
}
function nextSibling(node) {
    return node.nextSibling;
}
function tagName(elm) {
    return elm.tagName;
}
function setTextContent(node, text) {
    node.textContent = text;
}
function getTextContent(node) {
    return node.textContent;
}
function isElement(node) {
    return node.nodeType === 1;
}
function isText(node) {
    return node.nodeType === 3;
}
function isComment(node) {
    return node.nodeType === 8;
}
const htmlDomApi = {
    createElement,
    createElementNS,
    createTextNode,
    createComment,
    insertBefore,
    removeChild,
    appendChild,
    parentNode,
    nextSibling,
    tagName,
    setTextContent,
    getTextContent,
    isElement,
    isText,
    isComment,
};
//# sourceMappingURL=htmldomapi.js.map

/***/ }),
/* 6 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "array": () => (/* binding */ array),
/* harmony export */   "primitive": () => (/* binding */ primitive)
/* harmony export */ });
const array = Array.isArray;
function primitive(s) {
    return typeof s === "string" ||
        typeof s === "number" ||
        s instanceof String ||
        s instanceof Number;
}
//# sourceMappingURL=is.js.map

/***/ }),
/* 7 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "classModule": () => (/* binding */ classModule)
/* harmony export */ });
function updateClass(oldVnode, vnode) {
    let cur;
    let name;
    const elm = vnode.elm;
    let oldClass = oldVnode.data.class;
    let klass = vnode.data.class;
    if (!oldClass && !klass)
        return;
    if (oldClass === klass)
        return;
    oldClass = oldClass || {};
    klass = klass || {};
    for (name in oldClass) {
        if (oldClass[name] && !Object.prototype.hasOwnProperty.call(klass, name)) {
            // was `true` and now not provided
            elm.classList.remove(name);
        }
    }
    for (name in klass) {
        cur = klass[name];
        if (cur !== oldClass[name]) {
            elm.classList[cur ? "add" : "remove"](name);
        }
    }
}
const classModule = { create: updateClass, update: updateClass };
//# sourceMappingURL=class.js.map

/***/ }),
/* 8 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "datasetModule": () => (/* binding */ datasetModule)
/* harmony export */ });
const CAPS_REGEX = /[A-Z]/g;
function updateDataset(oldVnode, vnode) {
    const elm = vnode.elm;
    let oldDataset = oldVnode.data.dataset;
    let dataset = vnode.data.dataset;
    let key;
    if (!oldDataset && !dataset)
        return;
    if (oldDataset === dataset)
        return;
    oldDataset = oldDataset || {};
    dataset = dataset || {};
    const d = elm.dataset;
    for (key in oldDataset) {
        if (!dataset[key]) {
            if (d) {
                if (key in d) {
                    delete d[key];
                }
            }
            else {
                elm.removeAttribute("data-" + key.replace(CAPS_REGEX, "-$&").toLowerCase());
            }
        }
    }
    for (key in dataset) {
        if (oldDataset[key] !== dataset[key]) {
            if (d) {
                d[key] = dataset[key];
            }
            else {
                elm.setAttribute("data-" + key.replace(CAPS_REGEX, "-$&").toLowerCase(), dataset[key]);
            }
        }
    }
}
const datasetModule = {
    create: updateDataset,
    update: updateDataset,
};
//# sourceMappingURL=dataset.js.map

/***/ }),
/* 9 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "eventListenersModule": () => (/* binding */ eventListenersModule)
/* harmony export */ });
function invokeHandler(handler, vnode, event) {
    if (typeof handler === "function") {
        // call function handler
        handler.call(vnode, event, vnode);
    }
    else if (typeof handler === "object") {
        // call multiple handlers
        for (let i = 0; i < handler.length; i++) {
            invokeHandler(handler[i], vnode, event);
        }
    }
}
function handleEvent(event, vnode) {
    const name = event.type;
    const on = vnode.data.on;
    // call event handler(s) if exists
    if (on && on[name]) {
        invokeHandler(on[name], vnode, event);
    }
}
function createListener() {
    return function handler(event) {
        handleEvent(event, handler.vnode);
    };
}
function updateEventListeners(oldVnode, vnode) {
    const oldOn = oldVnode.data.on;
    const oldListener = oldVnode.listener;
    const oldElm = oldVnode.elm;
    const on = vnode && vnode.data.on;
    const elm = (vnode && vnode.elm);
    let name;
    // optimization for reused immutable handlers
    if (oldOn === on) {
        return;
    }
    // remove existing listeners which no longer used
    if (oldOn && oldListener) {
        // if element changed or deleted we remove all existing listeners unconditionally
        if (!on) {
            for (name in oldOn) {
                // remove listener if element was changed or existing listeners removed
                oldElm.removeEventListener(name, oldListener, false);
            }
        }
        else {
            for (name in oldOn) {
                // remove listener if existing listener removed
                if (!on[name]) {
                    oldElm.removeEventListener(name, oldListener, false);
                }
            }
        }
    }
    // add new listeners which has not already attached
    if (on) {
        // reuse existing listener or create new
        const listener = (vnode.listener =
            oldVnode.listener || createListener());
        // update vnode for listener
        listener.vnode = vnode;
        // if element changed or added we add all needed listeners unconditionally
        if (!oldOn) {
            for (name in on) {
                // add listener if element was changed or new listeners added
                elm.addEventListener(name, listener, false);
            }
        }
        else {
            for (name in on) {
                // add listener if new listener added
                if (!oldOn[name]) {
                    elm.addEventListener(name, listener, false);
                }
            }
        }
    }
}
const eventListenersModule = {
    create: updateEventListeners,
    update: updateEventListeners,
    destroy: updateEventListeners,
};
//# sourceMappingURL=eventlisteners.js.map

/***/ }),
/* 10 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "styleModule": () => (/* binding */ styleModule)
/* harmony export */ });
// Bindig `requestAnimationFrame` like this fixes a bug in IE/Edge. See #360 and #409.
const raf = (typeof window !== "undefined" &&
    window.requestAnimationFrame.bind(window)) ||
    setTimeout;
const nextFrame = function (fn) {
    raf(function () {
        raf(fn);
    });
};
let reflowForced = false;
function setNextFrame(obj, prop, val) {
    nextFrame(function () {
        obj[prop] = val;
    });
}
function updateStyle(oldVnode, vnode) {
    let cur;
    let name;
    const elm = vnode.elm;
    let oldStyle = oldVnode.data.style;
    let style = vnode.data.style;
    if (!oldStyle && !style)
        return;
    if (oldStyle === style)
        return;
    oldStyle = oldStyle || {};
    style = style || {};
    const oldHasDel = "delayed" in oldStyle;
    for (name in oldStyle) {
        if (!style[name]) {
            if (name[0] === "-" && name[1] === "-") {
                elm.style.removeProperty(name);
            }
            else {
                elm.style[name] = "";
            }
        }
    }
    for (name in style) {
        cur = style[name];
        if (name === "delayed" && style.delayed) {
            for (const name2 in style.delayed) {
                cur = style.delayed[name2];
                if (!oldHasDel || cur !== oldStyle.delayed[name2]) {
                    setNextFrame(elm.style, name2, cur);
                }
            }
        }
        else if (name !== "remove" && cur !== oldStyle[name]) {
            if (name[0] === "-" && name[1] === "-") {
                elm.style.setProperty(name, cur);
            }
            else {
                elm.style[name] = cur;
            }
        }
    }
}
function applyDestroyStyle(vnode) {
    let style;
    let name;
    const elm = vnode.elm;
    const s = vnode.data.style;
    if (!s || !(style = s.destroy))
        return;
    for (name in style) {
        elm.style[name] = style[name];
    }
}
function applyRemoveStyle(vnode, rm) {
    const s = vnode.data.style;
    if (!s || !s.remove) {
        rm();
        return;
    }
    if (!reflowForced) {
        // eslint-disable-next-line @typescript-eslint/no-unused-expressions
        vnode.elm.offsetLeft;
        reflowForced = true;
    }
    let name;
    const elm = vnode.elm;
    let i = 0;
    const style = s.remove;
    let amount = 0;
    const applied = [];
    for (name in style) {
        applied.push(name);
        elm.style[name] = style[name];
    }
    const compStyle = getComputedStyle(elm);
    const props = compStyle["transition-property"].split(", ");
    for (; i < props.length; ++i) {
        if (applied.indexOf(props[i]) !== -1)
            amount++;
    }
    elm.addEventListener("transitionend", function (ev) {
        if (ev.target === elm)
            --amount;
        if (amount === 0)
            rm();
    });
}
function forceReflow() {
    reflowForced = false;
}
const styleModule = {
    pre: forceReflow,
    create: updateStyle,
    update: updateStyle,
    destroy: applyDestroyStyle,
    remove: applyRemoveStyle,
};
//# sourceMappingURL=style.js.map

/***/ }),
/* 11 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "h": () => (/* binding */ h)
/* harmony export */ });
/* harmony import */ var _vnode__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(4);
/* harmony import */ var _is__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(6);


function addNS(data, children, sel) {
    data.ns = "http://www.w3.org/2000/svg";
    if (sel !== "foreignObject" && children !== undefined) {
        for (let i = 0; i < children.length; ++i) {
            const childData = children[i].data;
            if (childData !== undefined) {
                addNS(childData, children[i].children, children[i].sel);
            }
        }
    }
}
function h(sel, b, c) {
    let data = {};
    let children;
    let text;
    let i;
    if (c !== undefined) {
        if (b !== null) {
            data = b;
        }
        if (_is__WEBPACK_IMPORTED_MODULE_0__.array(c)) {
            children = c;
        }
        else if (_is__WEBPACK_IMPORTED_MODULE_0__.primitive(c)) {
            text = c.toString();
        }
        else if (c && c.sel) {
            children = [c];
        }
    }
    else if (b !== undefined && b !== null) {
        if (_is__WEBPACK_IMPORTED_MODULE_0__.array(b)) {
            children = b;
        }
        else if (_is__WEBPACK_IMPORTED_MODULE_0__.primitive(b)) {
            text = b.toString();
        }
        else if (b && b.sel) {
            children = [b];
        }
        else {
            data = b;
        }
    }
    if (children !== undefined) {
        for (i = 0; i < children.length; ++i) {
            if (_is__WEBPACK_IMPORTED_MODULE_0__.primitive(children[i]))
                children[i] = (0,_vnode__WEBPACK_IMPORTED_MODULE_1__.vnode)(undefined, undefined, undefined, children[i], undefined);
        }
    }
    if (sel[0] === "s" &&
        sel[1] === "v" &&
        sel[2] === "g" &&
        (sel.length === 3 || sel[3] === "." || sel[3] === "#")) {
        addNS(data, children, sel);
    }
    return (0,_vnode__WEBPACK_IMPORTED_MODULE_1__.vnode)(sel, data, children, text, undefined);
}
//# sourceMappingURL=h.js.map

/***/ }),
/* 12 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "defaultState": () => (/* binding */ defaultState),
/* harmony export */   "init": () => (/* binding */ init),
/* harmony export */   "showSettingsDialog": () => (/* binding */ showSettingsDialog),
/* harmony export */   "hideSettingsDialog": () => (/* binding */ hideSettingsDialog),
/* harmony export */   "setSettings": () => (/* binding */ setSettings),
/* harmony export */   "save": () => (/* binding */ save),
/* harmony export */   "showAddDialog": () => (/* binding */ showAddDialog),
/* harmony export */   "hideAddDialog": () => (/* binding */ hideAddDialog),
/* harmony export */   "add": () => (/* binding */ add),
/* harmony export */   "select": () => (/* binding */ select),
/* harmony export */   "remove": () => (/* binding */ remove),
/* harmony export */   "start": () => (/* binding */ start),
/* harmony export */   "stop": () => (/* binding */ stop),
/* harmony export */   "setName": () => (/* binding */ setName),
/* harmony export */   "setAutoStart": () => (/* binding */ setAutoStart),
/* harmony export */   "execute": () => (/* binding */ execute)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _hat_open_juggler__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(13);





let app;


const defaultState = {
    remote: null,
    local: null,
    deviceId: null,
    addDialog: {
        open: false
    },
    settingsDialog: {
        open: false
    },
    pages: {}
};


function init() {
    app = new _hat_open_juggler__WEBPACK_IMPORTED_MODULE_2__.Application('local', 'remote');
    window.app = app;
}


function showSettingsDialog() {
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('settingsDialog', _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set('open', true, defaultState.settingsDialog));
}


function hideSettingsDialog() {
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('settingsDialog', defaultState.settingsDialog);
}


function setSettings(path, value) {
    app.rpc.set_settings(path, value);
}


function save() {
    app.rpc.save();
}


function showAddDialog() {
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('addDialog', _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set('open', true, defaultState.addDialog));
}


function hideAddDialog() {
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('addDialog', defaultState.addDialog);
}


async function add(deviceType) {
    const deviceId = await app.rpc.add(deviceType);
    select(deviceId);
}


function select(deviceId) {
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('deviceId', deviceId);
}


async function remove(deviceId) {
    await app.rpc.remove(deviceId);
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].change('pages', _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.omit(deviceId));
}


function start(deviceId) {
    app.rpc.start(deviceId);
}


function stop(deviceId) {
    app.rpc.stop(deviceId);
}


function setName(deviceId, name) {
    app.rpc.set_name(deviceId, name);
}


function setAutoStart(deviceId, autoStart) {
    app.rpc.set_autostart(deviceId, autoStart);
}


async function execute(deviceId, action, ...args) {
    return await app.rpc.execute(deviceId, action, ...args);
}


/***/ }),
/* 13 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "Connection": () => (/* binding */ Connection),
/* harmony export */   "Application": () => (/* binding */ Application)
/* harmony export */ });
/* harmony import */ var jiff__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(14);
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(2);
/* harmony import */ var _hat_open_future__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(28);
/** @module @hat-open/juggler
 */








function getDefaultAddress() {
    const protocol = window.location.protocol == 'https:' ? 'wss' : 'ws';
    const hostname = window.location.hostname || 'localhost';
    const port = window.location.port;
    return `${protocol}://${hostname}` + (port ? `:${port}` : '') + '/ws';
}


/**
 * Juggler client connection
 *
 * Available events:
 *  - open - connection is opened (detail is undefined)
 *  - close - connection is closed (detail is undefined)
 *  - message - received new message (detail is received message)
 *  - change - remote data changed (detail is new remote data)
 */
class Connection extends EventTarget {

    /**
     * Create connection
     * @param {string} address Juggler server address, formatted as
     *     ``ws[s]://<host>[:<port>][/<path>]``. If not provided, hostname
     *     and port obtained from ``widow.location`` are used instead, with
     *     ``ws`` as a path.
     * @param {number} syncDelay sync delay in ms
     */
    constructor(address=getDefaultAddress(), syncDelay=100) {
        super();
        this._syncDelay = syncDelay;
        this._localData = null;
        this._remoteData = null;
        this._delayedSyncID = null;
        this._syncedLocalData = null;
        this._ws = new WebSocket(address);
        this._ws.addEventListener('open', () => this._onOpen());
        this._ws.addEventListener('close', () => this._onClose());
        this._ws.addEventListener('message', evt => this._onMessage(evt.data));
    }

    /**
     * Local data
     * @type {*}
     */
    get localData() {
        return this._localData;
    }

    /**
     * Remote data
     * @type {*}
     */
    get remoteData() {
        return this._remoteData;
    }

    /**
     * WebSocket ready state
     * @type {number}
     */
    get readyState() {
        return this._ws.readyState;
    }

    /**
     * Close connection
     */
    close() {
        this._ws.close(1000);
    }

    /**
     * Send message
     * @param {*} msg
     */
    send(msg) {
        if (this.readyState != WebSocket.OPEN) {
            throw new Error("connection not open");
        }
        this._ws.send(JSON.stringify({
            type: 'MESSAGE',
            payload: msg
        }));
    }

    /**
     * Set local data
     * @param {*} data
     */
    setLocalData(data) {
        if (this.readyState != WebSocket.OPEN) {
            throw new Error("connection not open");
        }
        this._localData = data;
        if (this._delayedSyncID != null)
            return;
        this._delayedSyncID = setTimeout(
            () => { this._onSync(); },
            this._syncDelay);
    }

    _onOpen() {
        this.dispatchEvent(new CustomEvent('open'));
    }

    _onClose() {
        if (this._delayedSyncID != null) {
            clearTimeout(this._delayedSyncID);
            this._delayedSyncID = null;
        }
        this.dispatchEvent(new CustomEvent('close'));
    }

    _onMessage(data) {
        try {
            const msg = JSON.parse(data);
            if (msg.type == 'DATA') {
                this._remoteData = jiff__WEBPACK_IMPORTED_MODULE_0__.patch(msg.payload, this._remoteData);
                this.dispatchEvent(new CustomEvent('change', {detail: this._remoteData}));
            } else if (msg.type == 'MESSAGE') {
                this.dispatchEvent(new CustomEvent('message', {detail: msg.payload}));
            } else {
                throw new Error('unsupported message type');
            }
        } catch (e) {
            this._ws.close();
            throw e;
        }
    }

    _onSync() {
        const patch = jiff__WEBPACK_IMPORTED_MODULE_0__.diff(this._syncedLocalData, this._localData);
        if (patch.length > 0) {
            this._ws.send(JSON.stringify({
                type: 'DATA',
                payload: patch
            }));
            this._syncedLocalData = this._localData;
        }
        this._delayedSyncID = null;
    }

}


/**
 * Juggler based application
 *
 * Available events:
 *  - connected - connected to server (detail is undefined)
 *  - disconnected - disconnected from server (detail is undefined)
 *  - message - received new message (detail is received message)
 */
class Application extends EventTarget {

    /**
     * Create application
     * @param {?module:@hat-open/util.Path} localPath local data state path
     * @param {?module:@hat-open/util.Path} remotePath remote data state path
     * @param {object} localRpcCbs local RPC function callbacs
     * @param {module:@hat-open/renderer.Renderer} renderer renderer
     * @param {string} address juggler server address, see
     *     {@link module:@hat-open/juggler.Connection}
     */
    constructor(
        localPath=null,
        remotePath=null,
        localRpcCbs={},
        renderer=_hat_open_renderer__WEBPACK_IMPORTED_MODULE_1__["default"],
        address=getDefaultAddress(),
        syncDelay=100,
        retryDelay=5000) {

        super();
        this._localPath = localPath;
        this._remotePath = remotePath;
        this._localRpcCbs = localRpcCbs;
        this._renderer = renderer;
        this._address = address;
        this._syncDelay = syncDelay;
        this._retryDelay = retryDelay;
        this._conn = null;
        this._lastRpcId = 0;
        this._rpcFutures = new Map();
        this._rpc = new Proxy({}, {
            get: (_, action) => (...args) => this._rpcCall(action, args)
        });

        if (localPath != null)
            renderer.addEventListener('change', () => this._onRendererChange());

        _hat_open_util__WEBPACK_IMPORTED_MODULE_2__.delay(() => this._connectLoop());
    }

    /**
     * RPC proxy
     * @type {*}
     */
    get rpc() {
        return this._rpc;
    }

    /**
     * Send message
     * @param {*} msg
     */
    send(msg) {
        if (!this._conn)
            throw new Error("connection closed");
        this._conn.send(msg);
    }

    _onOpen() {
        if (this._localPath)
            this._conn.setLocalData(this._renderer.get(this._localPath));
        this.dispatchEvent(new CustomEvent('connected'));
    }

    _onClose() {
        if (this._remotePath)
            this._renderer.set(this._remotePath, null);
        this.dispatchEvent(new CustomEvent('disconnected'));
    }

    _onMessage(msg) {
        if (_hat_open_util__WEBPACK_IMPORTED_MODULE_2__.isObject(msg) && msg.type == 'rpc') {
            this._onRpcMessage(msg);
        } else {
            this.dispatchEvent(new CustomEvent('message', {detail: msg}));
        }
    }

    _onRpcMessage(msg) {
        if (msg.direction == 'request') {
            this._onRpcReqMessage(msg);
        } else if (msg.direction == 'response') {
            this._onRpcResMessage(msg);
        }
    }

    _onRpcReqMessage(msg) {
        let success;
        let result;
        try {
            result = this._localRpcCbs[msg.action](...msg.args);
            success = true;
        } catch (e) {
            result = String(e);
            success = false;
        }
        this.send({
            type: 'rpc',
            id: msg.id,
            direction: 'response',
            success: success,
            result: result
        });
    }

    _onRpcResMessage(msg) {
        const f = this._rpcFutures.get(msg.id);
        this._rpcFutures.delete(msg.id);
        if (msg.success) {
            f.setResult(msg.result);
        } else {
            f.setError(msg.result);
        }
    }

    _onConnectionChange(data) {
        if (this._remotePath == null)
            return;
        this._renderer.set(this._remotePath, data);
    }

    _onRendererChange() {
        if (!this._conn || this._conn.readyState != WebSocket.OPEN)
            return;
        this._conn.setLocalData(this._renderer.get(this._localPath));
    }

    _rpcCall(action, args) {
        this._lastRpcId += 1;
        this.send({
            type: 'rpc',
            id: this._lastRpcId,
            direction: 'request',
            action: action,
            args: args});
        const f = _hat_open_future__WEBPACK_IMPORTED_MODULE_3__.create();
        this._rpcFutures.set(this._lastRpcId, f);
        return f;
    }

    async _connectLoop() {
        while (true) {
            this._conn = new Connection(this._address, this._syncDelay);
            this._conn.addEventListener('open', () => this._onOpen());
            this._conn.addEventListener('close', () => this._onClose());
            this._conn.addEventListener('message', evt => this._onMessage(evt.detail));
            this._conn.addEventListener('change', evt => this._onConnectionChange(evt.detail));

            const closeFuture = _hat_open_future__WEBPACK_IMPORTED_MODULE_3__.create();
            this._conn.addEventListener('close', () => closeFuture.setResult());
            await closeFuture;
            this._conn = null;
            await _hat_open_util__WEBPACK_IMPORTED_MODULE_2__.sleep(this._retryDelay);
        }
    }

}


/***/ }),
/* 14 */
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

var lcs = __webpack_require__(15);
var array = __webpack_require__(16);
var patch = __webpack_require__(17);
var inverse = __webpack_require__(27);
var jsonPointer = __webpack_require__(19);
var encodeSegment = jsonPointer.encodeSegment;

exports.diff = diff;
exports.patch = patch.apply;
exports.patchInPlace = patch.applyInPlace;
exports.inverse = inverse;
exports.clone = patch.clone;

// Errors
exports.InvalidPatchOperationError = __webpack_require__(25);
exports.TestFailedError = __webpack_require__(24);
exports.PatchNotInvertibleError = __webpack_require__(26);

var isValidObject = patch.isValidObject;
var defaultHash = patch.defaultHash;

/**
 * Compute a JSON Patch representing the differences between a and b.
 * @param {object|array|string|number|null} a
 * @param {object|array|string|number|null} b
 * @param {?function|?object} options if a function, see options.hash
 * @param {?function(x:*):String|Number} options.hash used to hash array items
 *  in order to recognize identical objects, defaults to JSON.stringify
 * @param {?function(index:Number, array:Array):object} options.makeContext
 *  used to generate patch context. If not provided, context will not be generated
 * @returns {array} JSON Patch such that patch(diff(a, b), a) ~ b
 */
function diff(a, b, options) {
	return appendChanges(a, b, '', initState(options, [])).patch;
}

/**
 * Create initial diff state from the provided options
 * @param {?function|?object} options @see diff options above
 * @param {array} patch an empty or existing JSON Patch array into which
 *  the diff should generate new patch operations
 * @returns {object} initialized diff state
 */
function initState(options, patch) {
	if(typeof options === 'object') {
		return {
			patch: patch,
			hash: orElse(isFunction, options.hash, defaultHash),
			makeContext: orElse(isFunction, options.makeContext, defaultContext),
			invertible: !(options.invertible === false)
		};
	} else {
		return {
			patch: patch,
			hash: orElse(isFunction, options, defaultHash),
			makeContext: defaultContext,
			invertible: true
		};
	}
}

/**
 * Given two JSON values (object, array, number, string, etc.), find their
 * differences and append them to the diff state
 * @param {object|array|string|number|null} a
 * @param {object|array|string|number|null} b
 * @param {string} path
 * @param {object} state
 * @returns {Object} updated diff state
 */
function appendChanges(a, b, path, state) {
	if(Array.isArray(a) && Array.isArray(b)) {
		return appendArrayChanges(a, b, path, state);
	}

	if(isValidObject(a) && isValidObject(b)) {
		return appendObjectChanges(a, b, path, state);
	}

	return appendValueChanges(a, b, path, state);
}

/**
 * Given two objects, find their differences and append them to the diff state
 * @param {object} o1
 * @param {object} o2
 * @param {string} path
 * @param {object} state
 * @returns {Object} updated diff state
 */
function appendObjectChanges(o1, o2, path, state) {
	var keys = Object.keys(o2);
	var patch = state.patch;
	var i, key;

	for(i=keys.length-1; i>=0; --i) {
		key = keys[i];
		var keyPath = path + '/' + encodeSegment(key);
		if(o1[key] !== void 0) {
			appendChanges(o1[key], o2[key], keyPath, state);
		} else {
			patch.push({ op: 'add', path: keyPath, value: o2[key] });
		}
	}

	keys = Object.keys(o1);
	for(i=keys.length-1; i>=0; --i) {
		key = keys[i];
		if(o2[key] === void 0) {
			var p = path + '/' + encodeSegment(key);
			if(state.invertible) {
				patch.push({ op: 'test', path: p, value: o1[key] });
			}
			patch.push({ op: 'remove', path: p });
		}
	}

	return state;
}

/**
 * Given two arrays, find their differences and append them to the diff state
 * @param {array} a1
 * @param {array} a2
 * @param {string} path
 * @param {object} state
 * @returns {Object} updated diff state
 */
function appendArrayChanges(a1, a2, path, state) {
	var a1hash = array.map(state.hash, a1);
	var a2hash = array.map(state.hash, a2);

	var lcsMatrix = lcs.compare(a1hash, a2hash);

	return lcsToJsonPatch(a1, a2, path, state, lcsMatrix);
}

/**
 * Transform an lcsMatrix into JSON Patch operations and append
 * them to state.patch, recursing into array elements as necessary
 * @param {array} a1
 * @param {array} a2
 * @param {string} path
 * @param {object} state
 * @param {object} lcsMatrix
 * @returns {object} new state with JSON Patch operations added based
 *  on the provided lcsMatrix
 */
function lcsToJsonPatch(a1, a2, path, state, lcsMatrix) {
	var offset = 0;
	return lcs.reduce(function(state, op, i, j) {
		var last, context;
		var patch = state.patch;
		var p = path + '/' + (j + offset);

		if (op === lcs.REMOVE) {
			// Coalesce adjacent remove + add into replace
			last = patch[patch.length-1];
			context = state.makeContext(j, a1);

			if(state.invertible) {
				patch.push({ op: 'test', path: p, value: a1[j], context: context });
			}

			if(last !== void 0 && last.op === 'add' && last.path === p) {
				last.op = 'replace';
				last.context = context;
			} else {
				patch.push({ op: 'remove', path: p, context: context });
			}

			offset -= 1;

		} else if (op === lcs.ADD) {
			// See https://tools.ietf.org/html/rfc6902#section-4.1
			// May use either index===length *or* '-' to indicate appending to array
			patch.push({ op: 'add', path: p, value: a2[i],
				context: state.makeContext(j, a1)
			});

			offset += 1;

		} else {
			appendChanges(a1[j], a2[i], p, state);
		}

		return state;

	}, state, lcsMatrix);
}

/**
 * Given two number|string|null values, if they differ, append to diff state
 * @param {string|number|null} a
 * @param {string|number|null} b
 * @param {string} path
 * @param {object} state
 * @returns {object} updated diff state
 */
function appendValueChanges(a, b, path, state) {
	if(a !== b) {
		if(state.invertible) {
			state.patch.push({ op: 'test', path: path, value: a });
		}

		state.patch.push({ op: 'replace', path: path, value: b });
	}

	return state;
}

/**
 * @param {function} predicate
 * @param {*} x
 * @param {*} y
 * @returns {*} x if predicate(x) is truthy, otherwise y
 */
function orElse(predicate, x, y) {
	return predicate(x) ? x : y;
}

/**
 * Default patch context generator
 * @returns {undefined} undefined context
 */
function defaultContext() {
	return void 0;
}

/**
 * @param {*} x
 * @returns {boolean} true if x is a function, false otherwise
 */
function isFunction(x) {
	return typeof x === 'function';
}


/***/ }),
/* 15 */
/***/ ((__unused_webpack_module, exports) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

exports.compare = compare;
exports.reduce = reduce;

var REMOVE, RIGHT, ADD, DOWN, SKIP;

exports.REMOVE = REMOVE = RIGHT = -1;
exports.ADD    = ADD    = DOWN  =  1;
exports.EQUAL  = SKIP   = 0;

/**
 * Create an lcs comparison matrix describing the differences
 * between two array-like sequences
 * @param {array} a array-like
 * @param {array} b array-like
 * @returns {object} lcs descriptor, suitable for passing to reduce()
 */
function compare(a, b) {
	var cols = a.length;
	var rows = b.length;

	var prefix = findPrefix(a, b);
	var suffix = prefix < cols && prefix < rows
		? findSuffix(a, b, prefix)
		: 0;

	var remove = suffix + prefix - 1;
	cols -= remove;
	rows -= remove;
	var matrix = createMatrix(cols, rows);

	for (var j = cols - 1; j >= 0; --j) {
		for (var i = rows - 1; i >= 0; --i) {
			matrix[i][j] = backtrack(matrix, a, b, prefix, j, i);
		}
	}

	return {
		prefix: prefix,
		matrix: matrix,
		suffix: suffix
	};
}

/**
 * Reduce a set of lcs changes previously created using compare
 * @param {function(result:*, type:number, i:number, j:number)} f
 *  reducer function, where:
 *  - result is the current reduce value,
 *  - type is the type of change: ADD, REMOVE, or SKIP
 *  - i is the index of the change location in b
 *  - j is the index of the change location in a
 * @param {*} r initial value
 * @param {object} lcs results returned by compare()
 * @returns {*} the final reduced value
 */
function reduce(f, r, lcs) {
	var i, j, k, op;

	var m = lcs.matrix;

	// Reduce shared prefix
	var l = lcs.prefix;
	for(i = 0;i < l; ++i) {
		r = f(r, SKIP, i, i);
	}

	// Reduce longest change span
	k = i;
	l = m.length;
	i = 0;
	j = 0;
	while(i < l) {
		op = m[i][j].type;
		r = f(r, op, i+k, j+k);

		switch(op) {
			case SKIP:  ++i; ++j; break;
			case RIGHT: ++j; break;
			case DOWN:  ++i; break;
		}
	}

	// Reduce shared suffix
	i += k;
	j += k;
	l = lcs.suffix;
	for(k = 0;k < l; ++k) {
		r = f(r, SKIP, i+k, j+k);
	}

	return r;
}

function findPrefix(a, b) {
	var i = 0;
	var l = Math.min(a.length, b.length);
	while(i < l && a[i] === b[i]) {
		++i;
	}
	return i;
}

function findSuffix(a, b) {
	var al = a.length - 1;
	var bl = b.length - 1;
	var l = Math.min(al, bl);
	var i = 0;
	while(i < l && a[al-i] === b[bl-i]) {
		++i;
	}
	return i;
}

function backtrack(matrix, a, b, start, j, i) {
	if (a[j+start] === b[i+start]) {
		return { value: matrix[i + 1][j + 1].value, type: SKIP };
	}
	if (matrix[i][j + 1].value < matrix[i + 1][j].value) {
		return { value: matrix[i][j + 1].value + 1, type: RIGHT };
	}

	return { value: matrix[i + 1][j].value + 1, type: DOWN };
}

function createMatrix (cols, rows) {
	var m = [], i, j, lastrow;

	// Fill the last row
	lastrow = m[rows] = [];
	for (j = 0; j<cols; ++j) {
		lastrow[j] = { value: cols - j, type: RIGHT };
	}

	// Fill the last col
	for (i = 0; i<rows; ++i) {
		m[i] = [];
		m[i][cols] = { value: rows - i, type: DOWN };
	}

	// Fill the last cell
	m[rows][cols] = { value: 0, type: SKIP };

	return m;
}


/***/ }),
/* 16 */
/***/ ((__unused_webpack_module, exports) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

exports.cons = cons;
exports.tail = tail;
exports.map = map;

/**
 * Prepend x to a, without mutating a. Faster than a.unshift(x)
 * @param {*} x
 * @param {Array} a array-like
 * @returns {Array} new Array with x prepended
 */
function cons(x, a) {
	var l = a.length;
	var b = new Array(l+1);
	b[0] = x;
	for(var i=0; i<l; ++i) {
		b[i+1] = a[i];
	}

	return b;
}

/**
 * Create a new Array containing all elements in a, except the first.
 *  Faster than a.slice(1)
 * @param {Array} a array-like
 * @returns {Array} new Array, the equivalent of a.slice(1)
 */
function tail(a) {
	var l = a.length-1;
	var b = new Array(l);
	for(var i=0; i<l; ++i) {
		b[i] = a[i+1];
	}

	return b;
}

/**
 * Map any array-like. Faster than Array.prototype.map
 * @param {function} f
 * @param {Array} a array-like
 * @returns {Array} new Array mapped by f
 */
function map(f, a) {
	var b = new Array(a.length);
	for(var i=0; i< a.length; ++i) {
		b[i] = f(a[i]);
	}
	return b;
}

/***/ }),
/* 17 */
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

var patches = __webpack_require__(18);
var clone = __webpack_require__(21);
var InvalidPatchOperationError = __webpack_require__(25);

exports.apply = patch;
exports.applyInPlace = patchInPlace;
exports.clone = clone;
exports.isValidObject = isValidObject;
exports.defaultHash = defaultHash;

var defaultOptions = {};

/**
 * Apply the supplied JSON Patch to x
 * @param {array} changes JSON Patch
 * @param {object|array|string|number} x object/array/value to patch
 * @param {object} options
 * @param {function(index:Number, array:Array, context:object):Number} options.findContext
 *  function used adjust array indexes for smarty/fuzzy patching, for
 *  patches containing context
 * @returns {object|array|string|number} patched version of x. If x is
 *  an array or object, it will be mutated and returned. Otherwise, if
 *  x is a value, the new value will be returned.
 */
function patch(changes, x, options) {
	return patchInPlace(changes, clone(x), options);
}

function patchInPlace(changes, x, options) {
	if(!options) {
		options = defaultOptions;
	}

	// TODO: Consider throwing if changes is not an array
	if(!Array.isArray(changes)) {
		return x;
	}

	var patch, p;
	for(var i=0; i<changes.length; ++i) {
		p = changes[i];
		patch = patches[p.op];

		if(patch === void 0) {
			throw new InvalidPatchOperationError('invalid op ' + JSON.stringify(p));
		}

		x = patch.apply(x, p, options);
	}

	return x;
}

function defaultHash(x) {
	return isValidObject(x) || isArray(x) ? JSON.stringify(x) : x;
}

function isValidObject (x) {
	return x !== null && Object.prototype.toString.call(x) === '[object Object]';
}

function isArray (x) {
	return Object.prototype.toString.call(x) === '[object Array]';
}


/***/ }),
/* 18 */
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

var jsonPointer = __webpack_require__(19);
var clone = __webpack_require__(21);
var deepEquals = __webpack_require__(22);
var commutePaths = __webpack_require__(23);

var array = __webpack_require__(16);

var TestFailedError = __webpack_require__(24);
var InvalidPatchOperationError = __webpack_require__(25);
var PatchNotInvertibleError = __webpack_require__(26);

var find = jsonPointer.find;
var parseArrayIndex = jsonPointer.parseArrayIndex;

exports.test = {
	apply: applyTest,
	inverse: invertTest,
	commute: commuteTest
};

exports.add = {
	apply: applyAdd,
	inverse: invertAdd,
	commute: commuteAddOrCopy
};

exports.remove = {
	apply: applyRemove,
	inverse: invertRemove,
	commute: commuteRemove
};

exports.replace = {
	apply: applyReplace,
	inverse: invertReplace,
	commute: commuteReplace
};

exports.move = {
	apply: applyMove,
	inverse: invertMove,
	commute: commuteMove
};

exports.copy = {
	apply: applyCopy,
	inverse: notInvertible,
	commute: commuteAddOrCopy
};

/**
 * Apply a test operation to x
 * @param {object|array} x
 * @param {object} test test operation
 * @throws {TestFailedError} if the test operation fails
 */

function applyTest(x, test, options) {
	var pointer = find(x, test.path, options.findContext, test.context);
	var target = pointer.target;
	var index, value;

	if(Array.isArray(target)) {
		index = parseArrayIndex(pointer.key);
		//index = findIndex(options.findContext, index, target, test.context);
		value = target[index];
	} else {
		value = pointer.key === void 0 ? pointer.target : pointer.target[pointer.key];
	}

	if(!deepEquals(value, test.value)) {
		throw new TestFailedError('test failed ' + JSON.stringify(test));
	}

	return x;
}

/**
 * Invert the provided test and add it to the inverted patch sequence
 * @param pr
 * @param test
 * @returns {number}
 */
function invertTest(pr, test) {
	pr.push(test);
	return 1;
}

function commuteTest(test, b) {
	if(test.path === b.path && b.op === 'remove') {
		throw new TypeError('Can\'t commute test,remove -> remove,test for same path');
	}

	if(b.op === 'test' || b.op === 'replace') {
		return [b, test];
	}

	return commutePaths(test, b);
}

/**
 * Apply an add operation to x
 * @param {object|array} x
 * @param {object} change add operation
 */
function applyAdd(x, change, options) {
	var pointer = find(x, change.path, options.findContext, change.context);

	if(notFound(pointer)) {
		throw new InvalidPatchOperationError('path does not exist ' + change.path);
	}

	if(change.value === void 0) {
		throw new InvalidPatchOperationError('missing value');
	}

	var val = clone(change.value);

	// If pointer refers to whole document, replace whole document
	if(pointer.key === void 0) {
		return val;
	}

	_add(pointer, val);
	return x;
}

function _add(pointer, value) {
	var target = pointer.target;

	if(Array.isArray(target)) {
		// '-' indicates 'append' to array
		if(pointer.key === '-') {
			target.push(value);
		} else if (pointer.key > target.length) {
			throw new InvalidPatchOperationError('target of add outside of array bounds')
		} else {
			target.splice(pointer.key, 0, value);
		}
	} else if(isValidObject(target)) {
		target[pointer.key] = value;
	} else {
		throw new InvalidPatchOperationError('target of add must be an object or array ' + pointer.key);
	}
}

function invertAdd(pr, add) {
	var context = add.context;
	if(context !== void 0) {
		context = {
			before: context.before,
			after: array.cons(add.value, context.after)
		}
	}
	pr.push({ op: 'test', path: add.path, value: add.value, context: context });
	pr.push({ op: 'remove', path: add.path, context: context });
	return 1;
}

function commuteAddOrCopy(add, b) {
	if(add.path === b.path && b.op === 'remove') {
		throw new TypeError('Can\'t commute add,remove -> remove,add for same path');
	}

	return commutePaths(add, b);
}

/**
 * Apply a replace operation to x
 * @param {object|array} x
 * @param {object} change replace operation
 */
function applyReplace(x, change, options) {
	var pointer = find(x, change.path, options.findContext, change.context);

	if(notFound(pointer) || missingValue(pointer)) {
		throw new InvalidPatchOperationError('path does not exist ' + change.path);
	}

	if(change.value === void 0) {
		throw new InvalidPatchOperationError('missing value');
	}

	var value = clone(change.value);

	// If pointer refers to whole document, replace whole document
	if(pointer.key === void 0) {
		return value;
	}

	var target = pointer.target;

	if(Array.isArray(target)) {
		target[parseArrayIndex(pointer.key)] = value;
	} else {
		target[pointer.key] = value;
	}

	return x;
}

function invertReplace(pr, c, i, patch) {
	var prev = patch[i-1];
	if(prev === void 0 || prev.op !== 'test' || prev.path !== c.path) {
		throw new PatchNotInvertibleError('cannot invert replace w/o test');
	}

	var context = prev.context;
	if(context !== void 0) {
		context = {
			before: context.before,
			after: array.cons(prev.value, array.tail(context.after))
		}
	}

	pr.push({ op: 'test', path: prev.path, value: c.value });
	pr.push({ op: 'replace', path: prev.path, value: prev.value });
	return 2;
}

function commuteReplace(replace, b) {
	if(replace.path === b.path && b.op === 'remove') {
		throw new TypeError('Can\'t commute replace,remove -> remove,replace for same path');
	}

	if(b.op === 'test' || b.op === 'replace') {
		return [b, replace];
	}

	return commutePaths(replace, b);
}

/**
 * Apply a remove operation to x
 * @param {object|array} x
 * @param {object} change remove operation
 */
function applyRemove(x, change, options) {
	var pointer = find(x, change.path, options.findContext, change.context);

	// key must exist for remove
	if(notFound(pointer) || pointer.target[pointer.key] === void 0) {
		throw new InvalidPatchOperationError('path does not exist ' + change.path);
	}

	_remove(pointer);
	return x;
}

function _remove (pointer) {
	var target = pointer.target;

	var removed;
	if (Array.isArray(target)) {
		removed = target.splice(parseArrayIndex(pointer.key), 1);
		return removed[0];

	} else if (isValidObject(target)) {
		removed = target[pointer.key];
		delete target[pointer.key];
		return removed;

	} else {
		throw new InvalidPatchOperationError('target of remove must be an object or array');
	}
}

function invertRemove(pr, c, i, patch) {
	var prev = patch[i-1];
	if(prev === void 0 || prev.op !== 'test' || prev.path !== c.path) {
		throw new PatchNotInvertibleError('cannot invert remove w/o test');
	}

	var context = prev.context;
	if(context !== void 0) {
		context = {
			before: context.before,
			after: array.tail(context.after)
		}
	}

	pr.push({ op: 'add', path: prev.path, value: prev.value, context: context });
	return 2;
}

function commuteRemove(remove, b) {
	if(remove.path === b.path && b.op === 'remove') {
		return [b, remove];
	}

	return commutePaths(remove, b);
}

/**
 * Apply a move operation to x
 * @param {object|array} x
 * @param {object} change move operation
 */
function applyMove(x, change, options) {
	if(jsonPointer.contains(change.path, change.from)) {
		throw new InvalidPatchOperationError('move.from cannot be ancestor of move.path');
	}

	var pto = find(x, change.path, options.findContext, change.context);
	var pfrom = find(x, change.from, options.findContext, change.fromContext);

	_add(pto, _remove(pfrom));
	return x;
}

function invertMove(pr, c) {
	pr.push({ op: 'move',
		path: c.from, context: c.fromContext,
		from: c.path, fromContext: c.context });
	return 1;
}

function commuteMove(move, b) {
	if(move.path === b.path && b.op === 'remove') {
		throw new TypeError('Can\'t commute move,remove -> move,replace for same path');
	}

	return commutePaths(move, b);
}

/**
 * Apply a copy operation to x
 * @param {object|array} x
 * @param {object} change copy operation
 */
function applyCopy(x, change, options) {
	var pto = find(x, change.path, options.findContext, change.context);
	var pfrom = find(x, change.from, options.findContext, change.fromContext);

	if(notFound(pfrom) || missingValue(pfrom)) {
		throw new InvalidPatchOperationError('copy.from must exist');
	}

	var target = pfrom.target;
	var value;

	if(Array.isArray(target)) {
		value = target[parseArrayIndex(pfrom.key)];
	} else {
		value = target[pfrom.key];
	}

	_add(pto, clone(value));
	return x;
}

// NOTE: Copy is not invertible
// See https://github.com/cujojs/jiff/issues/9
// This needs more thought. We may have to extend/amend JSON Patch.
// At first glance, this seems like it should just be a remove.
// However, that's not correct.  It violates the involution:
// invert(invert(p)) ~= p.  For example:
// invert(copy) -> remove
// invert(remove) -> add
// thus: invert(invert(copy)) -> add (DOH! this should be copy!)

function notInvertible(_, c) {
	throw new PatchNotInvertibleError('cannot invert ' + c.op);
}

function notFound (pointer) {
	return pointer === void 0 || (pointer.target == null && pointer.key !== void 0);
}

function missingValue(pointer) {
	return pointer.key !== void 0 && pointer.target[pointer.key] === void 0;
}

/**
 * Return true if x is a non-null object
 * @param {*} x
 * @returns {boolean}
 */
function isValidObject (x) {
	return x !== null && typeof x === 'object';
}


/***/ }),
/* 19 */
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

var _parse = __webpack_require__(20);

exports.find = find;
exports.join = join;
exports.absolute = absolute;
exports.parse = parse;
exports.contains = contains;
exports.encodeSegment = encodeSegment;
exports.decodeSegment = decodeSegment;
exports.parseArrayIndex = parseArrayIndex;
exports.isValidArrayIndex = isValidArrayIndex;

// http://tools.ietf.org/html/rfc6901#page-2
var separator = '/';
var separatorRx = /\//g;
var encodedSeparator = '~1';
var encodedSeparatorRx = /~1/g;

var escapeChar = '~';
var escapeRx = /~/g;
var encodedEscape = '~0';
var encodedEscapeRx = /~0/g;

/**
 * Find the parent of the specified path in x and return a descriptor
 * containing the parent and a key.  If the parent does not exist in x,
 * return undefined, instead.
 * @param {object|array} x object or array in which to search
 * @param {string} path JSON Pointer string (encoded)
 * @param {?function(index:Number, array:Array, context:object):Number} findContext
 *  optional function used adjust array indexes for smarty/fuzzy patching, for
 *  patches containing context.  If provided, context MUST also be provided.
 * @param {?{before:Array, after:Array}} context optional patch context for
 *  findContext to use to adjust array indices.  If provided, findContext MUST
 *  also be provided.
 * @returns {{target:object|array|number|string, key:string}|undefined}
 */
function find(x, path, findContext, context) {
	if(typeof path !== 'string') {
		return;
	}

	if(path === '') {
		// whole document
		return { target: x, key: void 0 };
	}

	if(path === separator) {
		return { target: x, key: '' };
	}

	var parent = x, key;
	var hasContext = context !== void 0;

	_parse(path, function(segment) {
		// hm... this seems like it should be if(typeof x === 'undefined')
		if(x == null) {
			// Signal that we prematurely hit the end of the path hierarchy.
			parent = null;
			return false;
		}

		if(Array.isArray(x)) {
			key = hasContext
				? findIndex(findContext, parseArrayIndex(segment), x, context)
				: segment === '-' ? segment : parseArrayIndex(segment);
		} else {
			key = segment;
		}

		parent = x;
		x = x[key];
	});

	return parent === null
		? void 0
		: { target: parent, key: key };
}

function absolute(path) {
	return path[0] === separator ? path : separator + path;
}

function join(segments) {
	return segments.join(separator);
}

function parse(path) {
	var segments = [];
	_parse(path, segments.push.bind(segments));
	return segments;
}

function contains(a, b) {
	return b.indexOf(a) === 0 && b[a.length] === separator;
}

/**
 * Decode a JSON Pointer path segment
 * @see http://tools.ietf.org/html/rfc6901#page-3
 * @param {string} s encoded segment
 * @returns {string} decoded segment
 */
function decodeSegment(s) {
	// See: http://tools.ietf.org/html/rfc6901#page-3
	return s.replace(encodedSeparatorRx, separator).replace(encodedEscapeRx, escapeChar);
}

/**
 * Encode a JSON Pointer path segment
 * @see http://tools.ietf.org/html/rfc6901#page-3
 * @param {string} s decoded segment
 * @returns {string} encoded segment
 */
function encodeSegment(s) {
	return s.replace(escapeRx, encodedEscape).replace(separatorRx, encodedSeparator);
}

var arrayIndexRx = /^(0|[1-9]\d*)$/;

/**
 * Return true if s is a valid JSON Pointer array index
 * @param {String} s
 * @returns {boolean}
 */
function isValidArrayIndex(s) {
	return arrayIndexRx.test(s);
}

/**
 * Safely parse a string into a number >= 0. Does not check for decimal numbers
 * @param {string} s numeric string
 * @returns {number} number >= 0
 */
function parseArrayIndex (s) {
	if(isValidArrayIndex(s)) {
		return +s;
	}

	throw new SyntaxError('invalid array index ' + s);
}

function findIndex (findContext, start, array, context) {
	var index = start;

	if(index < 0) {
		throw new Error('array index out of bounds ' + index);
	}

	if(context !== void 0 && typeof findContext === 'function') {
		index = findContext(start, array, context);
		if(index < 0) {
			throw new Error('could not find patch context ' + context);
		}
	}

	return index;
}

/***/ }),
/* 20 */
/***/ ((module) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

module.exports = jsonPointerParse;

var parseRx = /\/|~1|~0/g;
var separator = '/';
var escapeChar = '~';
var encodedSeparator = '~1';

/**
 * Parse through an encoded JSON Pointer string, decoding each path segment
 * and passing it to an onSegment callback function.
 * @see https://tools.ietf.org/html/rfc6901#section-4
 * @param {string} path encoded JSON Pointer string
 * @param {{function(segment:string):boolean}} onSegment callback function
 * @returns {string} original path
 */
function jsonPointerParse(path, onSegment) {
	var pos, accum, matches, match;

	pos = path.charAt(0) === separator ? 1 : 0;
	accum = '';
	parseRx.lastIndex = pos;

	while(matches = parseRx.exec(path)) {

		match = matches[0];
		accum += path.slice(pos, parseRx.lastIndex - match.length);
		pos = parseRx.lastIndex;

		if(match === separator) {
			if (onSegment(accum) === false) return path;
			accum = '';
		} else {
			accum += match === encodedSeparator ? separator : escapeChar;
		}
	}

	accum += path.slice(pos);
	onSegment(accum);

	return path;
}


/***/ }),
/* 21 */
/***/ ((module) => {

/** @license MIT License (c) copyright 2010-2014 original author or authors */
/** @author Brian Cavalier */
/** @author John Hann */

/**
 * Create a deep copy of x which must be a legal JSON object/array/value
 * @param {object|array|string|number|null} x object/array/value to clone
 * @returns {object|array|string|number|null} clone of x
 */
module.exports = clone;

function clone(x) {
	if(x == null || typeof x !== 'object') {
		return x;
	}

	if(Array.isArray(x)) {
		return cloneArray(x);
	}

	return cloneObject(x);
}

function cloneArray (x) {
	var l = x.length;
	var y = new Array(l);

	for (var i = 0; i < l; ++i) {
		y[i] = clone(x[i]);
	}

	return y;
}

function cloneObject (x) {
	var keys = Object.keys(x);
	var y = {};

	for (var k, i = 0, l = keys.length; i < l; ++i) {
		k = keys[i];
		y[k] = clone(x[k]);
	}

	return y;
}


/***/ }),
/* 22 */
/***/ ((module) => {

module.exports = deepEquals;

/**
 * Compare 2 JSON values, or recursively compare 2 JSON objects or arrays
 * @param {object|array|string|number|boolean|null} a
 * @param {object|array|string|number|boolean|null} b
 * @returns {boolean} true iff a and b are recursively equal
 */
function deepEquals(a, b) {
	if(a === b) {
		return true;
	}

	if(Array.isArray(a) && Array.isArray(b)) {
		return compareArrays(a, b);
	}

	if(typeof a === 'object' && typeof b === 'object') {
		return compareObjects(a, b);
	}

	return false;
}

function compareArrays(a, b) {
	if(a.length !== b.length) {
		return false;
	}

	for(var i = 0; i<a.length; ++i) {
		if(!deepEquals(a[i], b[i])) {
			return false;
		}
	}

	return true;
}

function compareObjects(a, b) {
	if((a === null && b !== null) || (a !== null && b === null)) {
		return false;
	}

	var akeys = Object.keys(a);
	var bkeys = Object.keys(b);

	if(akeys.length !== bkeys.length) {
		return false;
	}

	for(var i = 0, k; i<akeys.length; ++i) {
		k = akeys[i];
		if(!(k in b && deepEquals(a[k], b[k]))) {
			return false;
		}
	}

	return true;
}

/***/ }),
/* 23 */
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

var jsonPointer = __webpack_require__(19);

/**
 * commute the patch sequence a,b to b,a
 * @param {object} a patch operation
 * @param {object} b patch operation
 */
module.exports = function commutePaths(a, b) {
	// TODO: cases for special paths: '' and '/'
	var left = jsonPointer.parse(a.path);
	var right = jsonPointer.parse(b.path);
	var prefix = getCommonPathPrefix(left, right);
	var isArray = isArrayPath(left, right, prefix.length);

	// Never mutate the originals
	var ac = copyPatch(a);
	var bc = copyPatch(b);

	if(prefix.length === 0 && !isArray) {
		// Paths share no common ancestor, simple swap
		return [bc, ac];
	}

	if(isArray) {
		return commuteArrayPaths(ac, left, bc, right);
	} else {
		return commuteTreePaths(ac, left, bc, right);
	}
};

function commuteTreePaths(a, left, b, right) {
	if(a.path === b.path) {
		throw new TypeError('cannot commute ' + a.op + ',' + b.op + ' with identical object paths');
	}
	// FIXME: Implement tree path commutation
	return [b, a];
}

/**
 * Commute two patches whose common ancestor (which may be the immediate parent)
 * is an array
 * @param a
 * @param left
 * @param b
 * @param right
 * @returns {*}
 */
function commuteArrayPaths(a, left, b, right) {
	if(left.length === right.length) {
		return commuteArraySiblings(a, left, b, right);
	}

	if (left.length > right.length) {
		// left is longer, commute by "moving" it to the right
		left = commuteArrayAncestor(b, right, a, left, -1);
		a.path = jsonPointer.absolute(jsonPointer.join(left));
	} else {
		// right is longer, commute by "moving" it to the left
		right = commuteArrayAncestor(a, left, b, right, 1);
		b.path = jsonPointer.absolute(jsonPointer.join(right));
	}

	return [b, a];
}

function isArrayPath(left, right, index) {
	return jsonPointer.isValidArrayIndex(left[index])
		&& jsonPointer.isValidArrayIndex(right[index]);
}

/**
 * Commute two patches referring to items in the same array
 * @param l
 * @param lpath
 * @param r
 * @param rpath
 * @returns {*[]}
 */
function commuteArraySiblings(l, lpath, r, rpath) {

	var target = lpath.length-1;
	var lindex = +lpath[target];
	var rindex = +rpath[target];

	var commuted;

	if(lindex < rindex) {
		// Adjust right path
		if(l.op === 'add' || l.op === 'copy') {
			commuted = rpath.slice();
			commuted[target] = Math.max(0, rindex - 1);
			r.path = jsonPointer.absolute(jsonPointer.join(commuted));
		} else if(l.op === 'remove') {
			commuted = rpath.slice();
			commuted[target] = rindex + 1;
			r.path = jsonPointer.absolute(jsonPointer.join(commuted));
		}
	} else if(r.op === 'add' || r.op === 'copy') {
		// Adjust left path
		commuted = lpath.slice();
		commuted[target] = lindex + 1;
		l.path = jsonPointer.absolute(jsonPointer.join(commuted));
	} else if (lindex > rindex && r.op === 'remove') {
		// Adjust left path only if remove was at a (strictly) lower index
		commuted = lpath.slice();
		commuted[target] = Math.max(0, lindex - 1);
		l.path = jsonPointer.absolute(jsonPointer.join(commuted));
	}

	return [r, l];
}

/**
 * Commute two patches with a common array ancestor
 * @param l
 * @param lpath
 * @param r
 * @param rpath
 * @param direction
 * @returns {*}
 */
function commuteArrayAncestor(l, lpath, r, rpath, direction) {
	// rpath is longer or same length

	var target = lpath.length-1;
	var lindex = +lpath[target];
	var rindex = +rpath[target];

	// Copy rpath, then adjust its array index
	var rc = rpath.slice();

	if(lindex > rindex) {
		return rc;
	}

	if(l.op === 'add' || l.op === 'copy') {
		rc[target] = Math.max(0, rindex - direction);
	} else if(l.op === 'remove') {
		rc[target] = Math.max(0, rindex + direction);
	}

	return rc;
}

function getCommonPathPrefix(p1, p2) {
	var p1l = p1.length;
	var p2l = p2.length;
	if(p1l === 0 || p2l === 0 || (p1l < 2 && p2l < 2)) {
		return [];
	}

	// If paths are same length, the last segment cannot be part
	// of a common prefix.  If not the same length, the prefix cannot
	// be longer than the shorter path.
	var l = p1l === p2l
		? p1l - 1
		: Math.min(p1l, p2l);

	var i = 0;
	while(i < l && p1[i] === p2[i]) {
		++i
	}

	return p1.slice(0, i);
}

function copyPatch(p) {
	if(p.op === 'remove') {
		return { op: p.op, path: p.path };
	}

	if(p.op === 'copy' || p.op === 'move') {
		return { op: p.op, path: p.path, from: p.from };
	}

	// test, add, replace
	return { op: p.op, path: p.path, value: p.value };
}

/***/ }),
/* 24 */
/***/ ((module) => {

module.exports = TestFailedError;

function TestFailedError(message) {
	Error.call(this);
	this.name = this.constructor.name;
	this.message = message;
	if(typeof Error.captureStackTrace === 'function') {
		Error.captureStackTrace(this, this.constructor);
	}
}

TestFailedError.prototype = Object.create(Error.prototype);
TestFailedError.prototype.constructor = TestFailedError;

/***/ }),
/* 25 */
/***/ ((module) => {

module.exports = InvalidPatchOperationError;

function InvalidPatchOperationError(message) {
	Error.call(this);
	this.name = this.constructor.name;
	this.message = message;
	if(typeof Error.captureStackTrace === 'function') {
		Error.captureStackTrace(this, this.constructor);
	}
}

InvalidPatchOperationError.prototype = Object.create(Error.prototype);
InvalidPatchOperationError.prototype.constructor = InvalidPatchOperationError;

/***/ }),
/* 26 */
/***/ ((module) => {

module.exports = PatchNotInvertibleError;

function PatchNotInvertibleError(message) {
	Error.call(this);
	this.name = this.constructor.name;
	this.message = message;
	if(typeof Error.captureStackTrace === 'function') {
		Error.captureStackTrace(this, this.constructor);
	}
}

PatchNotInvertibleError.prototype = Object.create(Error.prototype);
PatchNotInvertibleError.prototype.constructor = PatchNotInvertibleError;

/***/ }),
/* 27 */
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

var patches = __webpack_require__(18);

module.exports = function inverse(p) {
	var pr = [];
	var i, skip;
	for(i = p.length-1; i>= 0; i -= skip) {
		skip = invertOp(pr, p[i], i, p);
	}

	return pr;
};

function invertOp(patch, c, i, context) {
	var op = patches[c.op];
	return op !== void 0 && typeof op.inverse === 'function'
		? op.inverse(patch, c, i, context)
		: 1;
}


/***/ }),
/* 28 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "create": () => (/* binding */ create)
/* harmony export */ });
/** @module @hat-open/future
 */


function create() {
    const data = {
        done: false,
        error: false,
        result: undefined,
        resolve: null,
        reject: null
    };

    const future = new Promise((resolve, reject) => {
        data.resolve = resolve;
        data.reject = reject;
        if (data.error) {
            reject(data.result);
        } else if (data.done) {
            resolve(data.resolve);
        }
    });

    future.done = () => data.done;

    future.result = () => {
        if (!data.done)
            throw new Error('future is not done');
        if (data.error)
            throw data.error;
        return data.result;
    };

    future.setResult = result => {
        if (data.done)
            throw new Error('result already set');
        data.result = result;
        data.done = true;
        if (data.resolve)
            data.resolve(data.result);
    };

    future.setError = error => {
        if (data.done)
            throw new Error('result already set');
        data.error = true;
        data.result = error;
        data.done = true;
        if (data.reject)
            data.reject(error);
    };

    return future;
}


/***/ }),
/* 29 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _datetime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(30);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(12);
/* harmony import */ var _dragger__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(31);
/* harmony import */ var _orchestrator_vt__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(32);
/* harmony import */ var _monitor_vt__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(34);
/* harmony import */ var _event_vt_main__WEBPACK_IMPORTED_MODULE_7__ = __webpack_require__(36);
/* harmony import */ var _iec104_vt__WEBPACK_IMPORTED_MODULE_8__ = __webpack_require__(41);
/* harmony import */ var _modbus_vt__WEBPACK_IMPORTED_MODULE_9__ = __webpack_require__(44);














const deviceTypes = {
    orchestrator: {
        name: 'Orchestrator',
        icon: 'icons/hat.png'
    },
    monitor: {
        name: 'Monitor Server',
        icon: 'icons/hat.png'
    },
    event: {
        name: 'Event Server',
        icon: 'icons/hat.png'
    },
    iec104_master: {
        name: 'IEC104 Master',
        icon: 'icons/iec.png'
    },
    iec104_slave: {
        name: 'IEC104 Slave',
        icon: 'icons/iec.png'
    },
    modbus_master: {
        name: 'Modbus Master',
        icon: 'icons/modbus.png'
    },
    modbus_slave: {
        name: 'Modbus Slave',
        icon: 'icons/modbus.png'
    }
};


function main() {
    return ['div#main',
        globalHeader(),
        deviceHeader(),
        sidebar(),
        sidebarResizer(),
        page(),
        logResizer(),
        log(),
        addDialog(),
        settingsDialog()
    ];
}


function globalHeader() {
    return ['div.header.global',
        ['button', {
            on: {
                click: _common__WEBPACK_IMPORTED_MODULE_3__.showSettingsDialog
            }},
            ['span.fa.fa-cog'],
            ' Settings'
        ],
        ['button', {
            on: {
                click: _common__WEBPACK_IMPORTED_MODULE_3__.save
            }},
            ['span.fa.fa-floppy-o'],
            ' Save'
        ]
    ];
}


function deviceHeader() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');
    if (!deviceId)
        return ['div.header.device'];

    const device = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId);
    if (!device)
        return ['div.header.device'];

    return ['div.header.device',
        ['label', deviceTypes[device.type].name],
        ['input', {
            props: {
                type: 'text',
                value: device.name
            },
            on: {
                change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setName(deviceId, evt.target.value)
            }
        }],
        ['label',
            ['input', {
                props: {
                    type: 'checkbox',
                    checked: device.autostart
                },
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setAutoStart(deviceId, evt.target.checked)
                }
            }],
            ' Auto start'
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_3__.start(deviceId)
            }},
            ['span.fa.fa-play'],
            ' Start'
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_3__.stop(deviceId)
            }},
            ['span.fa.fa-stop'],
            ' Stop'
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_3__.remove(deviceId)
            }},
            ['span.fa.fa-minus'],
            ' Remove'
        ]
    ];
}


function sidebar() {
    const devices = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices') || [];
    const selectedDeviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');
    return ['div.sidebar',
        ['div.devices',
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.toPairs(devices).map(([deviceId, device]) => {
                const selected = selectedDeviceId == deviceId;
                return ['div.device', {
                    class: {
                        selected: selected
                    },
                    on: {
                        click: _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set('deviceId', deviceId)
                    }},
                    ['span.status.fa.fa-circle', {
                        class: {
                            [device.status]: true
                        }
                    }],
                    ['span.name', device.name],
                    ' ',
                    ['span.type', `(${deviceTypes[device.type].name})`]
                ];
            })
        ],
        ['button.add', {
            on: {
                click: _common__WEBPACK_IMPORTED_MODULE_3__.showAddDialog
            }},
            ['span.fa.fa-plus'],
            ' Add device'
        ]
    ];
}


function sidebarResizer() {
    return ['div.sidebar-resizer', {
        on: {
            mousedown: _dragger__WEBPACK_IMPORTED_MODULE_4__.mouseDownHandler(evt => {
                const sidebar = evt.target.parentNode.querySelector('.sidebar');
                const width = sidebar.clientWidth;
                return (_, dx) => {
                    sidebar.style.width = `${width + dx}px`;
                };
            })
        }
    }];
}


function page() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');
    if (!deviceId)
        return ['div.page'];

    const deviceType = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'type');

    if (deviceType == 'orchestrator')
        return _orchestrator_vt__WEBPACK_IMPORTED_MODULE_5__.main();

    if (deviceType == 'monitor')
        return _monitor_vt__WEBPACK_IMPORTED_MODULE_6__.main();

    if (deviceType == 'event')
        return _event_vt_main__WEBPACK_IMPORTED_MODULE_7__.main();

    if (deviceType == 'iec104_master')
        return _iec104_vt__WEBPACK_IMPORTED_MODULE_8__.master();

    if (deviceType == 'iec104_slave')
        return _iec104_vt__WEBPACK_IMPORTED_MODULE_8__.slave();

    if (deviceType == 'modbus_master')
        return _modbus_vt__WEBPACK_IMPORTED_MODULE_9__.master();

    if (deviceType == 'modbus_slave')
        return _modbus_vt__WEBPACK_IMPORTED_MODULE_9__.slave();

    return ['div.page'];
}


function logResizer() {
    return ['div.log-resizer', {
        on: {
            mousedown: _dragger__WEBPACK_IMPORTED_MODULE_4__.mouseDownHandler(evt => {
                const sidebar = evt.target.parentNode.querySelector('.log');
                const height = sidebar.clientHeight;
                return (_, __, dy) => {
                    sidebar.style.height = `${height - dy}px`;
                };
            })
        }
    }];
}


function log() {
    const items = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'log') || [];
    return ['div.log',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-time', 'Time'],
                    ['th.col-message', 'Message']
                ]
            ],
            ['tbody', items.map(i =>
                ['tr',
                    ['td.col-time', _datetime__WEBPACK_IMPORTED_MODULE_2__.timestampToLocalString(i.timestamp)],
                    ['td.col-message', i.message]
                ]
            )]
        ]
    ];
}


function addDialog() {
    const open = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('addDialog', 'open');
    if (!open)
        return [];

    return ['div.overlay', {
        on: {
            click: evt => {
                evt.stopPropagation();
                _common__WEBPACK_IMPORTED_MODULE_3__.hideAddDialog();
            }
        }},
        ['div.dialog.add', {
            on: {
                click: evt => evt.stopPropagation()
            }},
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.toPairs(deviceTypes).map(([deviceType, i]) => ['div.device', {
                on: {
                    click: _ => {
                        _common__WEBPACK_IMPORTED_MODULE_3__.add(deviceType);
                        _common__WEBPACK_IMPORTED_MODULE_3__.hideAddDialog();
                    }
                }},
                ['img.icon', {
                    props: {
                        src: i.icon
                    }
                }],
                ['span.name', i.name]
            ])
        ]
    ];
}


function settingsDialog() {
    const open = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('settingsDialog', 'open');
    if (!open)
        return [];

    const settings = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'settings');

    return ['div.overlay', {
        on: {
            click: evt => {
                evt.stopPropagation();
                _common__WEBPACK_IMPORTED_MODULE_3__.hideSettingsDialog();
            }
        }},
        ['div.dialog.settings', {
            on: {
                click: evt => evt.stopPropagation()
            }},
            ['span.title', 'UI'],
            ['label.label', 'Address*'],
            ['input', {
                props: {
                    type: 'text',
                    value: settings.ui.address
                },
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['ui', 'address'], evt.target.value)
                }
            }],
            ['span.title', 'Log'],
            ['label.label', 'Level*'],
            ['select', {
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['log', 'level'], evt.target.value)
                }},
                ['DEBUG', 'INFO', 'WARNING', 'ERROR'].map(level => ['option', {
                    props: {
                        selected: level == settings.log.level
                    }},
                    level
                ])
            ],
            ['span'],
            ['label',
                ['input', {
                    props: {
                        type: 'checkbox',
                        checked: settings.log.console.enabled
                    },
                    on: {
                        change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['log', 'console', 'enabled'], evt.target.checked)
                    }
                }],
                ' Console*'
            ],
            ['span'],
            ['label',
                ['input', {
                    props: {
                        type: 'checkbox',
                        checked: settings.log.syslog.enabled
                    },
                    on: {
                        change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['log', 'syslog', 'enabled'], evt.target.checked)
                    }
                }],
                ' Syslog*'
            ],
            ['label.label', 'Syslog host*'],
            ['input', {
                props: {
                    type: 'text',
                    value: settings.log.syslog.host
                },
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['log', 'syslog', 'host'], evt.target.value)
                }
            }],
            ['label.label', 'Syslog port*'],
            ['input', {
                props: {
                    type: 'number',
                    value: settings.log.syslog.port
                },
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_3__.setSettings(['log', 'syslog', 'port'], evt.target.valueAsNumber)
                }
            }],
            ['span.note', '* changes applied on restart']
        ]
    ];
}


/***/ }),
/* 30 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "timestampToLocalString": () => (/* binding */ timestampToLocalString)
/* harmony export */ });


const localTimezoneOffset = (new Date()).getTimezoneOffset() * 60;


function timestampToLocalString(timestamp) {
    const date = new Date((timestamp - localTimezoneOffset) * 1000);
    return date.toISOString().replace('T', ' ').replace('Z', '');
}


/***/ }),
/* 31 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "mouseDownHandler": () => (/* binding */ mouseDownHandler)
/* harmony export */ });


const draggers = [];


function mouseDownHandler(createMoveHandlerCb) {
    return evt => {
        draggers.push({
            initX: evt.screenX,
            initY: evt.screenY,
            moveHandler: createMoveHandlerCb(evt)
        });
    };
}


document.addEventListener('mousemove', evt => {
    for (let dragger of draggers) {
        const dx = evt.screenX - dragger.initX;
        const dy = evt.screenY - dragger.initY;
        dragger.moveHandler(evt, dx, dy);
    }
});


document.addEventListener('mouseup', _ => {
    draggers.splice(0);
});


/***/ }),
/* 32 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(33);






function main() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.orchestrator',
        header(deviceId),
        components(deviceId)
    ];
}


function header(deviceId) {
    const address = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'address');

    return ['div.header',
        ['label', 'Address'],
        ['input', {
            props: {
                value: address
            },
            on: {
                change: evt => _common__WEBPACK_IMPORTED_MODULE_2__.setAddress(deviceId, evt.target.value)
            }
        }]
    ];
}


function components(deviceId) {
    const components = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'components');

    return ['div.components',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-component', 'Component'],
                    ['th.col-delay', 'Delay'],
                    ['th.col-revive', 'Revive'],
                    ['th.col-status', 'Status'],
                    ['th.col-action', 'Action']
                ]
            ],
            ['tbody', components.map(component =>
                ['tr',
                    ['td.col-component', component.name],
                    ['td.col-delay', String(component.delay)],
                    ['td.col-revive',
                        ['input', {
                            props: {
                                type: 'checkbox',
                                checked: component.revive
                            },
                            on: {
                                change: evt => _common__WEBPACK_IMPORTED_MODULE_2__.setRevive(deviceId, component.id, evt.target.checked)
                            }
                        }]
                    ],
                    ['td.col-status', component.status],
                    ['td.col-action',
                        ['button', {
                            props: {
                                title: 'Stop',
                                disabled: _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.contains(
                                    component.status, ['STOPPING', 'STOPPED'])
                            },
                            on: {
                                click: _ => _common__WEBPACK_IMPORTED_MODULE_2__.stop(deviceId, component.id)
                            }},
                            ['span.fa.fa-times']
                        ],
                        ['button', {
                            props: {
                                title: 'Start',
                                disabled: _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.contains(
                                    component.status,
                                    ['STARTING', 'RUNNING', 'STOPPING'])
                            },
                            on: {
                                click: _ => _common__WEBPACK_IMPORTED_MODULE_2__.start(deviceId, component.id)
                            }},
                            ['span.fa.fa-play']
                        ]
                    ]
                ]
            )]
        ]
    ];
}


/***/ }),
/* 33 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "setAddress": () => (/* binding */ setAddress),
/* harmony export */   "start": () => (/* binding */ start),
/* harmony export */   "stop": () => (/* binding */ stop),
/* harmony export */   "setRevive": () => (/* binding */ setRevive)
/* harmony export */ });
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(12);



function setAddress(deviceId, address) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'set_address', address);
}


function start(deviceId, componentId) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'start', componentId);
}


function stop(deviceId, componentId) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'stop', componentId);
}


function setRevive(deviceId, componentId, revive) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'set_revive', componentId, revive);
}


/***/ }),
/* 34 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(35);





function main() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.monitor',
        header(deviceId),
        localComponents(deviceId),
        globalComponents(deviceId)
    ];
}


function header(deviceId) {
    const address = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'address');

    return ['div.header',
        ['label', 'Address'],
        ['input', {
            props: {
                value: address
            },
            on: {
                change: evt => _common__WEBPACK_IMPORTED_MODULE_1__.setAddress(deviceId, evt.target.value)
            }
        }]
    ];
}


function localComponents(deviceId) {
    const components = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'local_components');

    return ['div.components',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-id.hidden'],
                    ['th.col-name.hidden'],
                    ['th.col-group.hidden'],
                    ['th.col-address.hidden'],
                    ['th.col-rank.hidden']
                ],
                ['tr',
                    ['th', {
                        props: {
                            colSpan: 5
                        }},
                        'Local components'
                    ]
                ],
                ['tr',
                    ['th.col-id', 'CID'],
                    ['th.col-name', 'Name'],
                    ['th.col-group', 'Group'],
                    ['th.col-address', 'Address'],
                    ['th.col-rank', 'Rank']
                ]
            ],
            ['tbody', components.map(({cid, name, group, address, rank}) =>
                ['tr',
                    ['td.col-id', String(cid)],
                    ['td.col-name', name || ''],
                    ['td.col-group', group || ''],
                    ['td.col-address', address || ''],
                    ['td.col-rank',
                        ['div',
                            ['button', {
                                on: {
                                    click: _ => _common__WEBPACK_IMPORTED_MODULE_1__.setRank(cid, rank - 1)
                                }},
                                ['span.fa.fa-chevron-left']
                            ],
                            ['div', String(rank)],
                            ['button', {
                                on: {
                                    click: _ => _common__WEBPACK_IMPORTED_MODULE_1__.setRank(cid, rank + 1)
                                }},
                                ['span.fa.fa-chevron-right']
                            ]
                        ]
                    ]
                ]
            )]
        ]
    ];
}


function globalComponents(deviceId) {
    const components = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'global_components');

    return ['div.components',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-id.hidden'],
                    ['th.col-id.hidden'],
                    ['th.col-name.hidden'],
                    ['th.col-group.hidden'],
                    ['th.col-address.hidden'],
                    ['th.col-rank.hidden'],
                    ['th.col-blessing.hidden'],
                    ['th.col-ready.hidden']
                ],
                ['tr',
                    ['th', {
                        props: {
                            colSpan: 8
                        }},
                        'Global components'
                    ]
                ],
                ['tr',
                    ['th.col-id', 'CID'],
                    ['th.col-id', 'MID'],
                    ['th.col-name', 'Name'],
                    ['th.col-group', 'Group'],
                    ['th.col-address', 'Address'],
                    ['th.col-rank', 'Rank'],
                    ['th.col-blessing', 'Blessing'],
                    ['th.col-ready', 'Ready']
                ]
            ],
            ['tbody', components.map(({cid, mid, name, group, address, rank, blessing, ready}) =>
                ['tr',
                    ['td.col-id', String(cid)],
                    ['td.col-id', String(mid)],
                    ['td.col-name', name || ''],
                    ['td.col-group', group || ''],
                    ['td.col-address', address || ''],
                    ['td.col-rank', String(rank)],
                    ['td.col-blessing', (blessing !== null ? String(blessing) : '')],
                    ['td.col-ready', (ready !== null ? String(ready) : '')]
                ]
            )]
        ]
    ];
}


/***/ }),
/* 35 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "setAddress": () => (/* binding */ setAddress),
/* harmony export */   "setRank": () => (/* binding */ setRank)
/* harmony export */ });
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(12);



function setAddress(deviceId, address) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'set_address', address);
}


function setRank(deviceId, cid, rank) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'set_rank', cid, rank);
}


/***/ }),
/* 36 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(37);
/* harmony import */ var _latest__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(38);
/* harmony import */ var _changes__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(39);
/* harmony import */ var _register__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(40);









function main() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.event',
        header(deviceId),
        subpage(deviceId)
    ];
}


const subpages = [
    ['latest', 'Latest'],
    ['changes', 'Changes'],
    ['register', 'Register']
];


function header(deviceId) {
    const subpagePath = ['pages', deviceId, 'subpage'];
    const addressPath = ['remote', 'devices', deviceId, 'data', 'address'];

    const selectedSubpage = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(subpagePath) || 'latest';
    const address = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(addressPath);

    return ['div.header',
        ['div.menu', subpages.map(([subpage, title]) =>
            ['div', {
                class: {
                    selected: selectedSubpage == subpage
                },
                on: {
                    click: _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(subpagePath, subpage)
                }},
                title
            ])
        ],
        ['div.spacer'],
        ['div.form',
            ['label', 'Address'],
            ['input', {
                props: {
                    value: address
                },
                on: {
                    change: evt => _common__WEBPACK_IMPORTED_MODULE_1__.setAddress(deviceId, evt.target.value)
                }
            }]
        ]
    ];
}


function subpage(deviceId) {
    const subpage = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('pages', deviceId, 'subpage') || 'latest';

    if (subpage == 'latest')
        return _latest__WEBPACK_IMPORTED_MODULE_2__.main(deviceId);

    if (subpage == 'changes')
        return _changes__WEBPACK_IMPORTED_MODULE_3__.main(deviceId);

    if (subpage == 'register')
        return _register__WEBPACK_IMPORTED_MODULE_4__.main(deviceId);

    return ['div.subpage'];
}


/***/ }),
/* 37 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "setAddress": () => (/* binding */ setAddress),
/* harmony export */   "register": () => (/* binding */ register)
/* harmony export */ });
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(12);



function setAddress(deviceId, address) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'set_address', address);
}


function register(deviceId, text, withSourceTimestamp) {
    _common__WEBPACK_IMPORTED_MODULE_0__.execute(deviceId, 'register', text, withSourceTimestamp);
}


/***/ }),
/* 38 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _datetime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(30);





function main(deviceId) {
    const latest = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'latest') || [];
    const eventTree = getEventTree(latest);

    return ['div.subpage.latest',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-type', 'Type'],
                    ['th.col-id', 'Server'],
                    ['th.col-id', 'Instance'],
                    ['th.col-timestamp', 'Timestamp'],
                    ['th.col-timestamp', 'Source timestamp'],
                    ['th.col-payload', 'Payload']
                ]
            ],
            ['tbody',
                eventTree.children.map(node => eventTreeNode(deviceId, node))
            ]
        ]
    ];
}


function eventTreeNode(deviceId, node) {
    const expandedPath = ['pages', deviceId, 'latest', 'expanded'];

    const expandedNodes = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(expandedPath) || [];
    const isExpanded = Boolean(_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.find(_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.equals(node.type), expandedNodes));

    let expandIcon;
    let expandClick;
    if (!node.children.length) {
        expandIcon = '.fa-square-o';
        expandClick = _ => null;
    } else if (!isExpanded) {
        expandIcon = '.fa-plus-square-o';
        expandClick = _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].change(expandedPath, _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.pipe(
            i => i || [],
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.append(node.type)
        ));
    } else {
        expandIcon = '.fa-minus-square-o';
        expandClick = _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].change(expandedPath, _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.pipe(
            i => i || [],
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.filter(i => !_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.equals(node.type, i))
        ));
    }

    const type = node.type[node.type.length - 1];
    const serverId = !node.event ? '' : String(node.event.event_id.server);
    const instanceId = !node.event ? '' : String(node.event.event_id.instance);
    const timestamp = !node.event ? '' : _datetime__WEBPACK_IMPORTED_MODULE_2__.timestampToLocalString(node.event.timestamp);
    const sourceTimestamp = !node.event ? '' : _datetime__WEBPACK_IMPORTED_MODULE_2__.timestampToLocalString(node.event.source_timestamp);
    const payload = !node.event ? '' : JSON.stringify(node.event.payload);

    return [
        ['tr',
            ['td.col-type', {
                props: {
                    title: type
                }},
                [`span.expand-icon.fa.${expandIcon}`, {
                    props: {
                        style: `margin-left: ${(node.type.length - 1) * 1}rem`},
                    on: {
                        click: expandClick}
                }],
                type
            ],
            ['td.col-id', {props: {title: serverId}}, serverId],
            ['td.col-id', {props: {title: instanceId}}, instanceId],
            ['td.col-timestamp', {props: {title: timestamp}}, timestamp],
            ['td.col-timestamp', {props: {title: sourceTimestamp}}, sourceTimestamp],
            ['td.col-payload', {props: {title: payload}}, payload]
        ],
        (!isExpanded ? [] :
            node.children.map(i => eventTreeNode(deviceId, i)))
    ];
}


function getEventTree(latest) {
    const createNode = (eventType, node) => ({
        'event': node['*'],
        'type': eventType,
        'children': _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.pipe(
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.toPairs,
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.filter(([k, _]) => k != '*'),
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.sortBy(_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.get(0)),
            _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.map(([k, v]) => createNode(_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.append(k, eventType), v))
        )(node)
    });
    return createNode(
        [], _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.reduce(
            (acc, val) => _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set([val.event_type, '*'], val, acc),
            {},
            latest));
}


/***/ }),
/* 39 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _datetime__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(30);




function main(deviceId) {
    const changes = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'changes');

    return ['div.subpage.changes',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-type', 'Type'],
                    ['th.col-id', 'Server'],
                    ['th.col-id', 'Instance'],
                    ['th.col-timestamp', 'Timestamp'],
                    ['th.col-timestamp', 'Source timestamp'],
                    ['th.col-payload', 'Payload']
                ]
            ],
            ['tbody',
                changes.map(event => {
                    const columns = [
                        ['col-type', event.event_type.join(', ')],
                        ['col-id', String(event.event_id.server)],
                        ['col-id', String(event.event_id.instance)],
                        ['col-timestamp', _datetime__WEBPACK_IMPORTED_MODULE_1__.timestampToLocalString(event.timestamp)],
                        ['col-timestamp', _datetime__WEBPACK_IMPORTED_MODULE_1__.timestampToLocalString(event.source_timestamp)],
                        ['col-payload', JSON.stringify(event.payload)]
                    ];
                    return ['tr', columns.map(([type, value]) =>
                        [`td.${type}`, {
                            props: {
                                title: value
                            }},
                            value
                        ]
                    )];
                })
            ]
        ]
    ];
}


/***/ }),
/* 40 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "main": () => (/* binding */ main)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(37);





function main(deviceId) {
    const textPath = ['pages', deviceId, 'register', 'text'];
    const withSourceTimestampPath = ['pages', deviceId, 'register', 'withSourceTimestamp'];

    const text = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(textPath) || '';
    const withSourceTimestamp = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(withSourceTimestampPath) || false;

    return ['div.subpage.register',
        ['label.source-timestamp',
            ['input', {
                props: {
                    type: 'checkbox',
                    checked: withSourceTimestamp
                },
                on: {
                    change: evt => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(withSourceTimestampPath, evt.target.checked)
                }
            }],
            ' source timestamp'
        ],
        ['textarea.text', {
            props: {
                value: text
            },
            on: {
                change: evt => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(textPath, evt.target.value)
            }
        }],
        ['button.register', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_1__.register(deviceId, text, withSourceTimestamp)
            }},
            'Register'
        ]
    ];
}


/***/ }),
/* 41 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "master": () => (/* binding */ master),
/* harmony export */   "slave": () => (/* binding */ slave)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _iter__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(42);
/* harmony import */ var _datetime__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(30);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(43);









const dataTypes = ['Single', 'Double', 'StepPosition',
                   'Bitstring', 'Normalized', 'Scaled', 'Floating',
                   'BinaryCounter'];

const commandTypes = ['Single', 'Double', 'Regulating', 'Normalized',
                      'Scaled', 'Floating'];

const dataCauses = ['UNDEFINED',
                    'PERIODIC',
                    'BACKGROUND_SCAN',
                    'SPONTANEOUS',
                    'INITIALIZED',
                    'REQUEST',
                    'ACTIVATION',
                    'ACTIVATION_CONFIRMATION',
                    'DEACTIVATION',
                    'DEACTIVATION_CONFIRMATION',
                    'ACTIVATION_TERMINATION',
                    'REMOTE_COMMAND',
                    'LOCAL_COMMAND',
                    'FILE_TRANSFER',
                    'INTERROGATED_STATION',
                    'INTERROGATED_GROUP01',
                    'INTERROGATED_GROUP02',
                    'INTERROGATED_GROUP03',
                    'INTERROGATED_GROUP04',
                    'INTERROGATED_GROUP05',
                    'INTERROGATED_GROUP06',
                    'INTERROGATED_GROUP07',
                    'INTERROGATED_GROUP08',
                    'INTERROGATED_GROUP09',
                    'INTERROGATED_GROUP10',
                    'INTERROGATED_GROUP11',
                    'INTERROGATED_GROUP12',
                    'INTERROGATED_GROUP13',
                    'INTERROGATED_GROUP14',
                    'INTERROGATED_GROUP15',
                    'INTERROGATED_GROUP16',
                    'INTERROGATED_COUNTER',
                    'INTERROGATED_COUNTER01',
                    'INTERROGATED_COUNTER02',
                    'INTERROGATED_COUNTER03',
                    'INTERROGATED_COUNTER04',
                    'UNKNOWN_TYPE',
                    'UNKNOWN_CAUSE',
                    'UNKNOWN_ASDU_ADDRESS',
                    'UNKNOWN_IO_ADDRESS'];

const singleValues = ['OFF', 'ON'];

const doubleValues = ['INTERMEDIATE', 'OFF', 'ON', 'FAULT'];

const regulatingValues = ['LOWER', 'HIGHER'];

const commandActions = ['EXECUTE', 'SELECT', 'CANCEL'];

const freezeValues = ['READ', 'FREEZE', 'FREEZE_AND_RESET', 'RESET'];


function master() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.iec104.master',
        properties(deviceId),
        masterControl(deviceId),
        masterData(deviceId)
    ];
}


function slave() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.iec104.slave',
        properties(deviceId),
        slaveData(deviceId),
        slaveCommands(deviceId),
        slavePanel(deviceId)
    ];
}


function properties(deviceId) {
    const properties = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'properties');

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) =>
        _common__WEBPACK_IMPORTED_MODULE_4__.setProperty(deviceId, property, value)
    );

    return ['div.properties',
        formEntryText('Host', properties.host, onChange('host')),
        formEntryNumber('Port', properties.port, onChange('port')),
        formEntryNumber('Response timeout', properties.response_timeout, onChange('response_timeout')),
        formEntryNumber('Supervisory timeout', properties.supervisory_timeout, onChange('supervisory_timeout')),
        formEntryNumber('Test timeout', properties.test_timeout, onChange('test_timeout')),
        formEntryNumber('Send window', properties.send_window_size, onChange('send_window_size')),
        formEntryNumber('Receive window', properties.receive_window_size, onChange('receive_window_size'))
    ];
}


function masterControl(deviceId) {
    const selectedTypePath = ['pages', deviceId, 'control', 'type'];

    const selectedType = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(selectedTypePath)  || 'interrogate';

    return ['div.control',
        ['div.header',
            [['interrogate', 'Interrogate'],
             ['counterInterrogate', 'Counter interrogate'],
             ['command', 'Command']
            ].map(([type, label]) => ['div', {
                class: {
                    selected: selectedType == type
                },
                on: {
                    click: _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(selectedTypePath, type)
                }},
                label
            ])
        ],
        {
            'interrogate': masterControlInterrogate,
            'counterInterrogate': masterControlCounterInterrogate,
            'command': masterControlCommand
        }[selectedType](deviceId)
    ];
}


function masterControlInterrogate(deviceId) {
    const dataPath = ['pages', deviceId, 'control', 'interrogate'];
    const asduPath = [dataPath, 'asdu'];

    const asdu = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(asduPath) || 0xFFFF;

    return ['div.content',
        ['div.form',
            formEntryNumber('ASDU', asdu, val => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(asduPath, val))
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.interrogate(deviceId, asdu)
            }},
            'Interrogate'
        ]
    ];
}


function masterControlCounterInterrogate(deviceId) {
    const dataPath = ['pages', deviceId, 'control', 'counterInterrogate'];
    const asduPath = [dataPath, 'asdu'];
    const freezePath = [dataPath, 'freeze'];

    const asdu = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(asduPath) || 0xFFFF;
    const freeze = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(freezePath) || 'READ';

    return ['div.content',
        ['div.form',
            formEntryNumber('ASDU',
                            asdu,
                            val => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(asduPath, val)),
            formEntrySelect('Freeze',
                            freeze,
                            freezeValues,
                            val => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(freezePath, val))
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.counterInterrogate(deviceId, asdu, freeze)
            }},
            'Counter interrogate'
        ]
    ];
}


function masterControlCommand(deviceId) {
    const cmdPath = ['pages', deviceId, 'control', 'command'];

    const cmd = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(cmdPath) || {
        type: 'Single',
        asdu: 1,
        io: 1,
        action: 'EXECUTE',
        value: {
            Single: 'OFF',
            Double: 'OFF',
            Regulating: 'LOWER',
            Normalized: 0,
            Scaled: 0,
            Floating: 0
        },
        time: null,
        qualifier: 0
    };

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) =>
        _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(cmdPath, _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set(property, value, cmd))
    );

    return ['div.content',
        ['div.form',
            formEntrySelect('Type', cmd.type, commandTypes, onChange('type')),
            formEntryNumber('ASDU', cmd.asdu, onChange('asdu')),
            formEntryNumber('IO', cmd.io, onChange('io')),
            formEntrySelect('Action', cmd.action, commandActions, onChange('action')),
            masterControlCommandValue(deviceId, cmd),
            timeFormEntries(cmd.time, onChange('time')),
            formEntryNumber('Qualifier', cmd.qualifier, onChange('qualifier'))
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.sendCommand(deviceId, cmd)
            }},
            'Send command'
        ]
    ];
}


function masterControlCommandValue(deviceId, cmd) {
    const cmdPath = ['pages', deviceId, 'control', 'command'];

    const value = cmd.value[cmd.type];

    const onChange = val => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(cmdPath, _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set(['value', cmd.type], val, cmd));

    if (cmd.type == 'Single')
        return formEntrySelect('Value', value, singleValues, onChange);

    if (cmd.type == 'Double')
        return formEntrySelect('Value', value, doubleValues, onChange);

    if (cmd.type == 'Regulating')
        return formEntrySelect('Value', value, regulatingValues, onChange);

    return formEntryNumber('Value', value, onChange);
}


function masterData(deviceId) {
    const data = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data') || [];

    return ['div.data',
        tableData(_iter__WEBPACK_IMPORTED_MODULE_2__.map(i => [null, i], data), true)
    ];
}


function slaveData(deviceId) {
    const selectedPath = ['pages', deviceId, 'selected'];

    const data = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data') || {};
    const [selectedType, selectedId] = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(selectedPath) || [];

    const isSelected = id => selectedType == 'data' && selectedId == id;
    const onClick = id => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(selectedPath, ['data', id]);
    const onRemove = id => _common__WEBPACK_IMPORTED_MODULE_4__.removeData(deviceId, id);

    return ['div.data',
        tableData(Object.entries(data), false, isSelected, onClick, onRemove),
        ['div.control',
            ['button', {
                on: {
                    click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.addData(deviceId)
                }},
                ['span.fa.fa-plus'],
                ' Add data'
            ]
        ]
    ];
}


function slaveCommands(deviceId) {
    const selectedPath = ['pages', deviceId, 'selected'];

    const commands = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'commands') || {};
    const [selectedType, selectedId] = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(selectedPath) || [];

    const isSelected = id => selectedType == 'command' && selectedId == id;
    const onClick = id => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(selectedPath, ['command', id]);
    const onRemove = id => _common__WEBPACK_IMPORTED_MODULE_4__.removeCommand(deviceId, id);

    return ['div.commands',
        tableCommands(Object.entries(commands), isSelected, onClick, onRemove),
        ['div.control',
            ['button', {
                on: {
                    click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.addCommand(deviceId)
                }},
                ['span.fa.fa-plus'],
                ' Add command'
            ]
        ]
    ];
}


function slavePanel(deviceId) {
    const [selectedType, selectedId] = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('pages', deviceId, 'selected') || [];

    if (selectedType == 'data')
        return slavePanelData(deviceId, selectedId);

    if (selectedType == 'command')
        return slavePanelCommand(deviceId, selectedId);

    return [];
}


function slavePanelData(deviceId, selectedDataId) {
    const selectedData = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data', selectedDataId);
    if (!selectedData)
        return [];

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) =>
        _common__WEBPACK_IMPORTED_MODULE_4__.changeData(deviceId, selectedDataId, property, value)
    );

    return ['div.panel',
        ['div.form',
            formEntrySelect('Type', selectedData.type, dataTypes, onChange('type')),
            formEntryNumber('ASDU', selectedData.asdu, onChange('asdu')),
            formEntryNumber('IO', selectedData.io, onChange('io')),
            slavePanelDataValue(deviceId, selectedDataId),
            slavePanelDataQuality(deviceId, selectedDataId),
            timeFormEntries(selectedData.time, onChange('time')),
            formEntrySelect('Cause', selectedData.cause, dataCauses, onChange('cause')),
            formEntryCheckbox('Is test', selectedData.is_test, onChange('is_test'))
        ],
        ['button', {
            on: {
                click: _ => _common__WEBPACK_IMPORTED_MODULE_4__.notifyData(deviceId, selectedDataId)
            }},
            'Notify change'
        ]
    ];
}


function slavePanelCommand(deviceId, selectedCommandId) {
    const selectedCommand = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'commands', selectedCommandId);
    if (!selectedCommand)
        return [];

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((path, value) =>
        _common__WEBPACK_IMPORTED_MODULE_4__.changeCommand(deviceId, selectedCommandId, path, value)
    );

    return ['div.panel',
        ['div.form',
            formEntrySelect('Type', selectedCommand.type, commandTypes, onChange('type')),
            formEntryNumber('ASDU', selectedCommand.asdu, onChange('asdu')),
            formEntryNumber('IO', selectedCommand.io, onChange('io')),
            formEntryCheckbox('Success', selectedCommand.success, onChange('success'))
        ]
    ];
}


function slavePanelDataValue(deviceId, dataId) {
    const dataType = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data', dataId, 'type');
    const value = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data', dataId, 'value');

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((path, value) =>
        _common__WEBPACK_IMPORTED_MODULE_4__.changeData(deviceId, dataId, ['value', path], value)
    );

    if (dataType == 'Single')
        return [
            formEntrySelect('Value', value.Single, singleValues, onChange('Single'))
        ];

    if (dataType == 'Double')
        return [
            formEntrySelect('Value', value.Double, doubleValues, onChange('Double'))
        ];

    if (dataType == 'StepPosition')
        return [
            formEntryNumber('Value', value.StepPosition.value,
                            onChange(['StepPosition', 'value'])),
            formEntryCheckbox('Value - transient', value.StepPosition.transient,
                              onChange(['StepPosition', 'transient']))
        ];

    if (dataType == 'Bitstring')
        return [
            formEntryText('Value', value.Bitstring, onChange('Bitstring'))
        ];

    if (dataType == 'Normalized')
        return [
            formEntryNumber('Value', value.Normalized, onChange('Normalized'))
        ];

    if (dataType == 'Scaled')
        return [
            formEntryNumber('Value', value.Scaled, onChange('Scaled'))
        ];

    if (dataType == 'Floating')
        return [
            formEntryNumber('Value', value.Floating, onChange('Floating'))
        ];

    if (dataType == 'BinaryCounter')
        return [
            formEntryNumber('Value', value.BinaryCounter.value,
                            onChange(['BinaryCounter', 'value'])),
            formEntryNumber('Value - sequence', value.BinaryCounter.sequence,
                            onChange(['BinaryCounter', 'sequence'])),
            formEntryCheckbox('Value - overflow', value.BinaryCounter.overflow,
                              onChange(['BinaryCounter', 'overflow'])),
            formEntryCheckbox('Value - adjusted', value.BinaryCounter.adjusted,
                              onChange(['BinaryCounter', 'adjusted'])),
            formEntryCheckbox('Value - invalid', value.BinaryCounter.invalid,
                              onChange(['BinaryCounter', 'invalid']))
        ];

    return [];
}


function slavePanelDataQuality(deviceId, dataId) {
    const quality = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data', dataId, 'quality');

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((path, value) =>
        _common__WEBPACK_IMPORTED_MODULE_4__.changeData(deviceId, dataId, ['quality', path], value)
    );

    return [
        formEntryCheckbox('Quality - invalid', quality.invalid, onChange('invalid')),
        formEntryCheckbox('Quality - not topical', quality.not_topical, onChange('not_topical')),
        formEntryCheckbox('Quality - substituted', quality.substituted, onChange('substituted')),
        formEntryCheckbox('Quality - blocked', quality.blocked, onChange('blocked')),
        formEntryCheckbox('Quality - overflow', quality.overflow, onChange('overflow'))
    ];
}


function formEntryText(label, value, onChange) {
    return [
        ['label.label', label],
        ['input', {
            props: {
                type: 'text',
                value: value
            },
            on: {
                change: evt => onChange(evt.target.value)
            }
        }]
    ];
}


function formEntryNumber(label, value, onChange) {
    return [
        ['label.label', label],
        ['input', {
            props: {
                type: 'number',
                value: value
            },
            on: {
                change: evt => onChange(evt.target.valueAsNumber)
            }
        }]
    ];
}


function formEntryCheckbox(label, value, onChange) {
    return [
        ['span'],
        ['label',
            ['input', {
                props: {
                    type: 'checkbox',
                    checked: value
                },
                on: {
                    change: evt => onChange(evt.target.checked)
                }
            }],
            ` ${label}`
        ]
    ];
}


function formEntrySelect(label, selected, values, onChange) {
    return [
        ['label.label', label],
        ['select', {
            on: {
                change: evt => onChange(evt.target.value)
            }},
            values.map(i => ['option', {
                props: {
                    selected: i == selected
                }},
                i
            ])
        ]
    ];
}


function timeFormEntries(time, onChange) {
    const change = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) =>
        onChange(_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set(property, value, time))
    );

    if (!time)
        return [
            formEntryCheckbox('Time', false, _ => change([], {
                milliseconds: 0,
                invalid: false,
                minutes: 0,
                summer_time: false,
                hours: 0,
                day_of_week: 0,
                day_of_month: 0,
                months: 0,
                years: 0
            }))
        ];

    return [
        formEntryCheckbox('Time', true, _ => change([], null)),
        formEntryNumber('Time - milliseconds', time.milliseconds, change('milliseconds')),
        formEntryCheckbox('Time - invalid', time.invalid, change('invalid')),
        formEntryNumber('Time - minutes', time.minutes, change('minutes')),
        formEntryCheckbox('Time - summer time', time.summer_time, change('summer_time')),
        formEntryNumber('Time - hours', time.hours, change('hours')),
        formEntryNumber('Time - day of week', time.day_of_week, change('day_of_week')),
        formEntryNumber('Time - day of month', time.day_of_month, change('day_of_month')),
        formEntryNumber('Time - months', time.months, change('months')),
        formEntryNumber('Time - years', time.years, change('years'))
    ];
}


function tableData(data, showTimestamp, isSelected=null, onClick=null, onRemove=null) {
    return ['table',
        ['thead',
            ['tr',
                (!showTimestamp ? [] : ['th.col-timestamp.hidden']),
                ['th.col-str.hidden'],    // type
                ['th.col-int.hidden'],    // asdu
                ['th.col-int.hidden'],    // io
                ['th.hidden'],            // value
                ['th.col-bool.hidden'],   // quality-invalid
                ['th.col-bool.hidden'],   // quality-not_topical
                ['th.col-bool.hidden'],   // quality-substituted
                ['th.col-bool.hidden'],   // quality-blocked
                ['th.col-bool.hidden'],   // quality-overflow
                ['th.col-int.hidden'],    // time-years
                ['th.col-short.hidden'],  // time-months
                ['th.col-short.hidden'],  // time-day_of_month
                ['th.col-short.hidden'],  // time-day_of_week
                ['th.col-short.hidden'],  // time-hours
                ['th.col-short.hidden'],  // time-minutes
                ['th.col-long.hidden'],   // time-milliseconds
                ['th.col-bool.hidden'],   // time-invalid
                ['th.col-bool.hidden'],   // time-summer_time
                ['th.col-str.hidden'],    // cause
                ['th.col-bool.hidden'],   // is_test
                (!onRemove ? [] : ['th.col-remove.hidden'])
            ],
            ['tr',
                (!showTimestamp ? [] : ['th', {props: {rowSpan: 2}}, 'Timestamp']),
                ['th', {props: {rowSpan: 2}}, 'Type'],
                ['th', {props: {rowSpan: 2}}, 'ASDU'],
                ['th', {props: {rowSpan: 2}}, 'IO'],
                ['th', {props: {rowSpan: 2}}, 'Value'],
                ['th', {props: {colSpan: 5}}, 'Quality'],
                ['th', {props: {colSpan: 9}}, 'Time'],
                ['th', {props: {rowSpan: 2}}, 'Cause'],
                ['th', {props: {rowSpan: 2}}, 'Test'],
                (!onRemove ? [] : ['th', {props: {rowSpan: 2}}, 'Remove'])
            ],
            ['tr',
                ['th', 'IN'],
                ['th', 'NT'],
                ['th', 'SU'],
                ['th', 'BL'],
                ['th', 'OV'],
                ['th', 'Y'],
                ['th', 'M'],
                ['th', 'DoM'],
                ['th', 'DoW'],
                ['th', 'H'],
                ['th', 'MIN'],
                ['th', 'MS'],
                ['th', 'IN'],
                ['th', 'ST']
            ]
        ],
        ['tbody', Array.from(data, ([id, i]) => {
            const val = (...path) => {
                const x = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.get(path, i);
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isNil(x))
                    return '';
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isBoolean(x))
                    return ['span.fa.fa-' + (x ? 'check' : 'times')];
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isObject(x))
                    return JSON.stringify(x);
                return String(x);
            };
            return ['tr', {
                class: {
                    selected: isSelected && isSelected(id)
                },
                on: {
                    click: _ => (onClick ? onClick(id) : null)
                }},
                (!showTimestamp ? [] : ['td.col-timestamp',
                    _datetime__WEBPACK_IMPORTED_MODULE_3__.timestampToLocalString(i.timestamp)
                ]),
                ['td.col-str', val('type')],
                ['td.col-int', val('asdu')],
                ['td.col-int', val('io')],
                ['td', val('value', i.type)],
                ['td.col-bool', val('quality', 'invalid')],
                ['td.col-bool', val('quality', 'not_topical')],
                ['td.col-bool', val('quality', 'substituted')],
                ['td.col-bool', val('quality', 'blocked')],
                ['td.col-bool', val('quality', 'overflow')],
                ['td.col-int', val('time', 'years')],
                ['td.col-short', val('time', 'months')],
                ['td.col-short', val('time', 'day_of_month')],
                ['td.col-short', val('time', 'day_of_week')],
                ['td.col-short', val('time', 'hours')],
                ['td.col-short', val('time', 'minutes')],
                ['td.col-long', val('time', 'milliseconds')],
                ['td.col-bool', val('time', 'invalid')],
                ['td.col-bool', val('time', 'summer_time')],
                ['td.col-str', val('cause')],
                ['td.col-bool', val('is_test')],
                (!onRemove ? [] : ['td.col-remove',
                    ['button', {
                        on: {
                            click: evt => {
                                evt.stopPropagation();
                                onRemove(id);
                            }
                        }},
                        ['span.fa.fa-times']
                    ]
                ])
            ];
        })]
    ];
}


function tableCommands(commands, isSelected, onClick, onRemove) {
    return ['table',
        ['thead',
            ['tr',
                ['th.col-str.hidden'],    // type
                ['th.col-int.hidden'],    // asdu
                ['th.col-int.hidden'],    // io
                ['th.hidden'],            // value
                ['th.col-bool.hidden'],   // success
                ['th.col-remove.hidden']
            ],
            ['tr',
                ['th', 'Type'],
                ['th', 'ASDU'],
                ['th', 'IO'],
                ['th', 'Value'],
                ['th', 'Res'],
                ['th', 'Remove']
            ]
        ],
        ['tbody', Array.from(commands, ([id, i]) => {
            const val = (...path) => {
                const x = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.get(path, i);
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isNil(x))
                    return '';
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isBoolean(x))
                    return ['span.fa.fa-' + (x ? 'check' : 'times')];
                if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isObject(x))
                    return JSON.stringify(x);
                return String(x);
            };
            return ['tr', {
                class: {
                    selected: isSelected(id)
                },
                on: {
                    click: _ => onClick(id)
                }},
                ['td.col-str', val('type')],
                ['td.col-int', val('asdu')],
                ['td.col-int', val('io')],
                ['td', val('value', i.type)],
                ['td.col-bool', val('success')],
                ['td.col-remove',
                    ['button', {
                        on: {
                            click: evt => {
                                evt.stopPropagation();
                                onRemove(id);
                            }
                        }},
                        ['span.fa.fa-times']
                    ]
                ]
            ];
        })]
    ];
}


/***/ }),
/* 42 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "enumerate": () => (/* binding */ enumerate),
/* harmony export */   "chain": () => (/* binding */ chain),
/* harmony export */   "count": () => (/* binding */ count),
/* harmony export */   "cycle": () => (/* binding */ cycle),
/* harmony export */   "repeat": () => (/* binding */ repeat),
/* harmony export */   "map": () => (/* binding */ map),
/* harmony export */   "filter": () => (/* binding */ filter),
/* harmony export */   "zip": () => (/* binding */ zip),
/* harmony export */   "slice": () => (/* binding */ slice)
/* harmony export */ });
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(2);



function* enumerate(x) {
    let counter = 0;
    for (const i of x)
        yield [(counter++), i];
}


function* chain(x) {
    for (const i of x)
        yield* i;
}


function* count(start, step=1) {
    let i = start;
    for (;;) {
        yield i;
        i += step;
    }
}


function* cycle(x) {
    const cache = [];
    for (const i of x) {
        cache.push(i);
        yield i;
    }
    if (!cache.length)
        return;
    for (;;) {
        yield* cache;
    }
}


function* repeat(x) {
    for (;;)
        yield x;
}


const map = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.curry(function* (fn, x) {
    for (const i of x)
        yield fn(i);
});


const filter = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.curry(function* (fn, x) {
    for (const i of x)
        if (fn(i))
            yield i;
});


const zip = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.curry(function* (x, y) {
    const is = x[Symbol.iterator]();
    const js = y[Symbol.iterator]();
    for (;;) {
        const i = is.next();
        const j = js.next();
        if (i.done || j.done)
            break;
        yield [i.value, j.value];
    }
});


const slice = _hat_open_util__WEBPACK_IMPORTED_MODULE_0__.curry(function* (start, stop, x) {
    let counter = 0;
    for (const i of x) {
        if (counter >= stop)
            break;
        if (counter >= start)
            yield i;
        counter++;
    }
});


/***/ }),
/* 43 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "setProperty": () => (/* binding */ setProperty),
/* harmony export */   "interrogate": () => (/* binding */ interrogate),
/* harmony export */   "counterInterrogate": () => (/* binding */ counterInterrogate),
/* harmony export */   "sendCommand": () => (/* binding */ sendCommand),
/* harmony export */   "addData": () => (/* binding */ addData),
/* harmony export */   "removeData": () => (/* binding */ removeData),
/* harmony export */   "changeData": () => (/* binding */ changeData),
/* harmony export */   "notifyData": () => (/* binding */ notifyData),
/* harmony export */   "addCommand": () => (/* binding */ addCommand),
/* harmony export */   "removeCommand": () => (/* binding */ removeCommand),
/* harmony export */   "changeCommand": () => (/* binding */ changeCommand)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(12);





function setProperty(deviceId, path, value) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'set_property', path, value);
}


function interrogate(deviceId, asdu) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'interrogate', asdu);
}


function counterInterrogate(deviceId, asdu, freeze) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'counter_interrogate', asdu, freeze);
}


async function sendCommand(deviceId, cmd) {
    await _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'send_command', cmd);
}


async function addData(deviceId) {
    const dataId = await _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'add_data');
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(['pages', deviceId, 'selected'], ['data', dataId]);
}


function removeData(deviceId, dataId) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'remove_data', dataId);
}


function changeData(deviceId, dataId, path, value) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'change_data', dataId, path, value);
}


function notifyData(deviceId, dataId) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'notify_data', dataId);
}


async function addCommand(deviceId) {
    const commandId = await _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'add_command');
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(['pages', deviceId, 'selected'], ['command', commandId]);
}


function removeCommand(deviceId, commandId) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'remove_command', commandId);
}


function changeCommand(deviceId, commandId, path, value) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'change_command', commandId, path, value);
}


/***/ }),
/* 44 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "master": () => (/* binding */ master),
/* harmony export */   "slave": () => (/* binding */ slave)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _datetime__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(30);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(45);







const linkTypes = ['TCP', 'SERIAL'];

const modbusTypes = ['TCP', 'RTU', 'ASCII'];

const dataTypes = ['COIL', 'DISCRETE_INPUT', 'HOLDING_REGISTER', 'INPUT_REGISTER', 'QUEUE'];


function master() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.modbus.master',
        properties(deviceId),
        masterAction(deviceId),
        masterData(deviceId)
    ];
}


function slave() {
    const deviceId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('deviceId');

    return ['div.page.modbus.slave',
        properties(deviceId),
        slaveData(deviceId),
        slavePanel(deviceId)
    ];
}


function properties(deviceId) {
    const properties = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'properties');

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) => _common__WEBPACK_IMPORTED_MODULE_3__.setProperty(deviceId, property, value));

    return ['div.properties',
        formEntrySelect('Link type', properties.link_type, linkTypes, onChange('link_type')),
        formEntrySelect('Modbus type', properties.modbus_type, modbusTypes, onChange('modbus_type')),
        (properties.link_type != 'TCP' ? [] : [
            formEntryText('Host', properties.tcp_host, onChange('tcp_host')),
            formEntryNumber('Port', properties.tcp_port, onChange('tcp_port'))
        ]),
        (properties.link_type != 'SERIAL' ? [] : [
            formEntryText('Serial port', properties.serial_port, onChange('serial_port')),
            formEntryNumber('Silent interval', properties.serial_silent_interval, onChange('serial_silent_interval'))
        ])
    ];
}


function masterAction(deviceId) {
    const actionPath = ['pages', deviceId, 'action'];

    const action = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(actionPath) || {
        action: 'READ',
        modbusDeviceId: 1,
        dataType: 'COIL',
        startAddress: 0,
        quantity: 1,
        values: '1'
    };

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(actionPath, _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.set(property, value, action)));

    return ['div.action',
        ['div.form',
            formEntrySelect('Action', action.action, ['READ', 'WRITE'], onChange('action')),
            formEntryNumber('Device ID', action.modbusDeviceId, onChange('modbusDeviceId')),
            formEntrySelect('Data type', action.dataType, dataTypes, onChange('dataType')),
            formEntryNumber('Start address', action.startAddress, onChange('startAddress')),
            (action.action != 'READ' ? [] : [
                formEntryNumber('Quantity', action.quantity, onChange('quantity'))
            ]),
            (action.action != 'WRITE' ? [] : [
                formEntryText('Value', action.values, onChange('values'))
            ]),
        ],
        ['button', {
            on: {
                click: _ => {
                    if (action.action == 'READ') {
                        _common__WEBPACK_IMPORTED_MODULE_3__.read(deviceId, action.modbusDeviceId, action.dataType, action.startAddress, action.quantity);
                    } else if (action.action == 'WRITE') {
                        const values = action.values.split(',').map(i => _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.strictParseInt(i.trim()));
                        if (values.includes(NaN))
                            return;
                        _common__WEBPACK_IMPORTED_MODULE_3__.write(deviceId, action.modbusDeviceId, action.dataType, action.startAddress, values);
                    }
                }
            }},
            'Execute'
        ]
    ];
}


function masterData(deviceId) {
    const data = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data') || [];

    const val = x => _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isNil(x) ? '' : String(x);

    return ['div.data',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-timestamp', 'Time'],
                    ['th.col-str', 'Action'],
                    ['th.col-int', 'Device ID'],
                    ['th.col-str', 'Data type'],
                    ['th.col-int', 'Start address'],
                    ['th', 'Value']
                ]
            ],
            ['tbody', data.map(i =>
                ['tr',
                    ['td.col-timestamp',
                        _datetime__WEBPACK_IMPORTED_MODULE_2__.timestampToLocalString(i.timestamp)
                    ],
                    ['td.col-str', val(i.action)],
                    ['td.col-int', val(i.device_id)],
                    ['td.col-str', val(i.data_type)],
                    ['td.col-int', val(i.start_address)],
                    ['td.col-str', val(i.value)]
                ]
            )]
        ]
    ];
}


function slaveData(deviceId) {
    const selectedPath = ['pages', deviceId, 'selected'];

    const data = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data') || {};
    const selected = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get(selectedPath);

    const val = x => _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isNil(x) ? '' : String(x);

    return ['div.data',
        ['table',
            ['thead',
                ['tr',
                    ['th.col-int', 'Device ID'],
                    ['th.col-str', 'Data type'],
                    ['th.col-int', 'Address'],
                    ['th', 'Value'],
                    ['th.col-remove']
                ]
            ],
            ['tbody', Array.from(Object.entries(data), ([id, i]) =>
                ['tr', {
                    class: {
                        selected: selected == id
                    },
                    on: {
                        click: _ => _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(selectedPath, id)
                    }},
                    ['td.col-int', val(i.device_id)],
                    ['td.col-str', val(i.data_type)],
                    ['td.col-int', val(i.address)],
                    ['td', val(i.value)],
                    ['td.col-remove',
                        ['button', {
                            on: {
                                click: evt => {
                                    evt.stopPropagation();
                                    _common__WEBPACK_IMPORTED_MODULE_3__.removeData(deviceId, id);
                                }
                            }},
                            ['span.fa.fa-times']
                        ]
                    ]
                ]
            )]
        ],
        ['div.control',
            ['button', {
                on: {
                    click: _ => _common__WEBPACK_IMPORTED_MODULE_3__.addData(deviceId)
                }},
                ['span.fa.fa-plus'],
                ' Add data'
            ]
        ]
    ];
}


function slavePanel(deviceId) {
    const dataId = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('pages', deviceId, 'selected');
    if (_hat_open_util__WEBPACK_IMPORTED_MODULE_1__.isNil(dataId))
        return [];

    const data = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].get('remote', 'devices', deviceId, 'data', 'data', dataId);
    if (!data)
        return [];

    const onChange = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__.curry((property, value) => _common__WEBPACK_IMPORTED_MODULE_3__.changeData(deviceId, dataId, property, value));

    return ['div.panel',
        formEntryNumber('Device ID', data.device_id, onChange('device_id')),
        formEntrySelect('Data type', data.data_type, dataTypes, onChange('data_type')),
        formEntryNumber('Address', data.address, onChange('address')),
        formEntryNumber('Value', data.value, onChange('value'))
    ];
}


function formEntryText(label, value, onChange) {
    return [
        ['label.label', label],
        ['input', {
            props: {
                type: 'text',
                value: value
            },
            on: {
                change: evt => onChange(evt.target.value)
            }
        }]
    ];
}


function formEntryNumber(label, value, onChange) {
    return [
        ['label.label', label],
        ['input', {
            props: {
                type: 'number',
                value: value
            },
            on: {
                change: evt => onChange(evt.target.valueAsNumber)
            }
        }]
    ];
}


function formEntrySelect(label, selected, values, onChange) {
    return [
        ['label.label', label],
        ['select', {
            on: {
                change: evt => onChange(evt.target.value)
            }},
            values.map(i => ['option', {
                props: {
                    selected: i == selected
                }},
                i
            ])
        ]
    ];
}


/***/ }),
/* 45 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "setProperty": () => (/* binding */ setProperty),
/* harmony export */   "read": () => (/* binding */ read),
/* harmony export */   "write": () => (/* binding */ write),
/* harmony export */   "addData": () => (/* binding */ addData),
/* harmony export */   "removeData": () => (/* binding */ removeData),
/* harmony export */   "changeData": () => (/* binding */ changeData)
/* harmony export */ });
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(12);





function setProperty(deviceId, path, value) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'set_property', path, value);
}


function read(deviceId, modbusDeviceId, dataType, startAddress, quantity) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'read', modbusDeviceId, dataType, startAddress, quantity);
}


function write(deviceId, modbusDeviceId, dataType, startAddress, values) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'write', modbusDeviceId, dataType, startAddress, values);
}


async function addData(deviceId) {
    const dataId = await _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'add_data');
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].set(['pages', deviceId, 'selected'], ['data', dataId]);
}


function removeData(deviceId, dataId) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'remove_data', dataId);
}


function changeData(deviceId, dataId, path, value) {
    _common__WEBPACK_IMPORTED_MODULE_1__.execute(deviceId, 'change_data', dataId, path, value);
}


/***/ }),
/* 46 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__)
/* harmony export */ });
/* harmony import */ var _node_modules_style_loader_dist_runtime_injectStylesIntoStyleTag_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(47);
/* harmony import */ var _node_modules_style_loader_dist_runtime_injectStylesIntoStyleTag_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_injectStylesIntoStyleTag_js__WEBPACK_IMPORTED_MODULE_0__);
/* harmony import */ var _node_modules_style_loader_dist_runtime_styleDomAPI_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(48);
/* harmony import */ var _node_modules_style_loader_dist_runtime_styleDomAPI_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_styleDomAPI_js__WEBPACK_IMPORTED_MODULE_1__);
/* harmony import */ var _node_modules_style_loader_dist_runtime_insertBySelector_js__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(49);
/* harmony import */ var _node_modules_style_loader_dist_runtime_insertBySelector_js__WEBPACK_IMPORTED_MODULE_2___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_insertBySelector_js__WEBPACK_IMPORTED_MODULE_2__);
/* harmony import */ var _node_modules_style_loader_dist_runtime_setAttributesWithoutAttributes_js__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(50);
/* harmony import */ var _node_modules_style_loader_dist_runtime_setAttributesWithoutAttributes_js__WEBPACK_IMPORTED_MODULE_3___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_setAttributesWithoutAttributes_js__WEBPACK_IMPORTED_MODULE_3__);
/* harmony import */ var _node_modules_style_loader_dist_runtime_insertStyleElement_js__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(51);
/* harmony import */ var _node_modules_style_loader_dist_runtime_insertStyleElement_js__WEBPACK_IMPORTED_MODULE_4___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_insertStyleElement_js__WEBPACK_IMPORTED_MODULE_4__);
/* harmony import */ var _node_modules_style_loader_dist_runtime_styleTagTransform_js__WEBPACK_IMPORTED_MODULE_5__ = __webpack_require__(52);
/* harmony import */ var _node_modules_style_loader_dist_runtime_styleTagTransform_js__WEBPACK_IMPORTED_MODULE_5___default = /*#__PURE__*/__webpack_require__.n(_node_modules_style_loader_dist_runtime_styleTagTransform_js__WEBPACK_IMPORTED_MODULE_5__);
/* harmony import */ var _node_modules_css_loader_dist_cjs_js_ruleSet_1_rules_0_use_1_node_modules_sass_loader_dist_cjs_js_ruleSet_1_rules_0_use_2_main_scss__WEBPACK_IMPORTED_MODULE_6__ = __webpack_require__(53);

      
      
      
      
      
      
      
      
      

var options = {};

options.styleTagTransform = (_node_modules_style_loader_dist_runtime_styleTagTransform_js__WEBPACK_IMPORTED_MODULE_5___default());
options.setAttributes = (_node_modules_style_loader_dist_runtime_setAttributesWithoutAttributes_js__WEBPACK_IMPORTED_MODULE_3___default());

      options.insert = _node_modules_style_loader_dist_runtime_insertBySelector_js__WEBPACK_IMPORTED_MODULE_2___default().bind(null, "head");
    
options.domAPI = (_node_modules_style_loader_dist_runtime_styleDomAPI_js__WEBPACK_IMPORTED_MODULE_1___default());
options.insertStyleElement = (_node_modules_style_loader_dist_runtime_insertStyleElement_js__WEBPACK_IMPORTED_MODULE_4___default());

var update = _node_modules_style_loader_dist_runtime_injectStylesIntoStyleTag_js__WEBPACK_IMPORTED_MODULE_0___default()(_node_modules_css_loader_dist_cjs_js_ruleSet_1_rules_0_use_1_node_modules_sass_loader_dist_cjs_js_ruleSet_1_rules_0_use_2_main_scss__WEBPACK_IMPORTED_MODULE_6__["default"], options);




       /* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (_node_modules_css_loader_dist_cjs_js_ruleSet_1_rules_0_use_1_node_modules_sass_loader_dist_cjs_js_ruleSet_1_rules_0_use_2_main_scss__WEBPACK_IMPORTED_MODULE_6__["default"] && _node_modules_css_loader_dist_cjs_js_ruleSet_1_rules_0_use_1_node_modules_sass_loader_dist_cjs_js_ruleSet_1_rules_0_use_2_main_scss__WEBPACK_IMPORTED_MODULE_6__["default"].locals ? _node_modules_css_loader_dist_cjs_js_ruleSet_1_rules_0_use_1_node_modules_sass_loader_dist_cjs_js_ruleSet_1_rules_0_use_2_main_scss__WEBPACK_IMPORTED_MODULE_6__["default"].locals : undefined);


/***/ }),
/* 47 */
/***/ ((module) => {

"use strict";


var stylesInDOM = [];

function getIndexByIdentifier(identifier) {
  var result = -1;

  for (var i = 0; i < stylesInDOM.length; i++) {
    if (stylesInDOM[i].identifier === identifier) {
      result = i;
      break;
    }
  }

  return result;
}

function modulesToDom(list, options) {
  var idCountMap = {};
  var identifiers = [];

  for (var i = 0; i < list.length; i++) {
    var item = list[i];
    var id = options.base ? item[0] + options.base : item[0];
    var count = idCountMap[id] || 0;
    var identifier = "".concat(id, " ").concat(count);
    idCountMap[id] = count + 1;
    var indexByIdentifier = getIndexByIdentifier(identifier);
    var obj = {
      css: item[1],
      media: item[2],
      sourceMap: item[3],
      supports: item[4],
      layer: item[5]
    };

    if (indexByIdentifier !== -1) {
      stylesInDOM[indexByIdentifier].references++;
      stylesInDOM[indexByIdentifier].updater(obj);
    } else {
      var updater = addElementStyle(obj, options);
      options.byIndex = i;
      stylesInDOM.splice(i, 0, {
        identifier: identifier,
        updater: updater,
        references: 1
      });
    }

    identifiers.push(identifier);
  }

  return identifiers;
}

function addElementStyle(obj, options) {
  var api = options.domAPI(options);
  api.update(obj);

  var updater = function updater(newObj) {
    if (newObj) {
      if (newObj.css === obj.css && newObj.media === obj.media && newObj.sourceMap === obj.sourceMap && newObj.supports === obj.supports && newObj.layer === obj.layer) {
        return;
      }

      api.update(obj = newObj);
    } else {
      api.remove();
    }
  };

  return updater;
}

module.exports = function (list, options) {
  options = options || {};
  list = list || [];
  var lastIdentifiers = modulesToDom(list, options);
  return function update(newList) {
    newList = newList || [];

    for (var i = 0; i < lastIdentifiers.length; i++) {
      var identifier = lastIdentifiers[i];
      var index = getIndexByIdentifier(identifier);
      stylesInDOM[index].references--;
    }

    var newLastIdentifiers = modulesToDom(newList, options);

    for (var _i = 0; _i < lastIdentifiers.length; _i++) {
      var _identifier = lastIdentifiers[_i];

      var _index = getIndexByIdentifier(_identifier);

      if (stylesInDOM[_index].references === 0) {
        stylesInDOM[_index].updater();

        stylesInDOM.splice(_index, 1);
      }
    }

    lastIdentifiers = newLastIdentifiers;
  };
};

/***/ }),
/* 48 */
/***/ ((module) => {

"use strict";


/* istanbul ignore next  */
function apply(styleElement, options, obj) {
  var css = "";

  if (obj.supports) {
    css += "@supports (".concat(obj.supports, ") {");
  }

  if (obj.media) {
    css += "@media ".concat(obj.media, " {");
  }

  var needLayer = typeof obj.layer !== "undefined";

  if (needLayer) {
    css += "@layer".concat(obj.layer.length > 0 ? " ".concat(obj.layer) : "", " {");
  }

  css += obj.css;

  if (needLayer) {
    css += "}";
  }

  if (obj.media) {
    css += "}";
  }

  if (obj.supports) {
    css += "}";
  }

  var sourceMap = obj.sourceMap;

  if (sourceMap && typeof btoa !== "undefined") {
    css += "\n/*# sourceMappingURL=data:application/json;base64,".concat(btoa(unescape(encodeURIComponent(JSON.stringify(sourceMap)))), " */");
  } // For old IE

  /* istanbul ignore if  */


  options.styleTagTransform(css, styleElement, options.options);
}

function removeStyleElement(styleElement) {
  // istanbul ignore if
  if (styleElement.parentNode === null) {
    return false;
  }

  styleElement.parentNode.removeChild(styleElement);
}
/* istanbul ignore next  */


function domAPI(options) {
  var styleElement = options.insertStyleElement(options);
  return {
    update: function update(obj) {
      apply(styleElement, options, obj);
    },
    remove: function remove() {
      removeStyleElement(styleElement);
    }
  };
}

module.exports = domAPI;

/***/ }),
/* 49 */
/***/ ((module) => {

"use strict";


var memo = {};
/* istanbul ignore next  */

function getTarget(target) {
  if (typeof memo[target] === "undefined") {
    var styleTarget = document.querySelector(target); // Special case to return head of iframe instead of iframe itself

    if (window.HTMLIFrameElement && styleTarget instanceof window.HTMLIFrameElement) {
      try {
        // This will throw an exception if access to iframe is blocked
        // due to cross-origin restrictions
        styleTarget = styleTarget.contentDocument.head;
      } catch (e) {
        // istanbul ignore next
        styleTarget = null;
      }
    }

    memo[target] = styleTarget;
  }

  return memo[target];
}
/* istanbul ignore next  */


function insertBySelector(insert, style) {
  var target = getTarget(insert);

  if (!target) {
    throw new Error("Couldn't find a style target. This probably means that the value for the 'insert' parameter is invalid.");
  }

  target.appendChild(style);
}

module.exports = insertBySelector;

/***/ }),
/* 50 */
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {

"use strict";


/* istanbul ignore next  */
function setAttributesWithoutAttributes(styleElement) {
  var nonce =  true ? __webpack_require__.nc : 0;

  if (nonce) {
    styleElement.setAttribute("nonce", nonce);
  }
}

module.exports = setAttributesWithoutAttributes;

/***/ }),
/* 51 */
/***/ ((module) => {

"use strict";


/* istanbul ignore next  */
function insertStyleElement(options) {
  var element = document.createElement("style");
  options.setAttributes(element, options.attributes);
  options.insert(element, options.options);
  return element;
}

module.exports = insertStyleElement;

/***/ }),
/* 52 */
/***/ ((module) => {

"use strict";


/* istanbul ignore next  */
function styleTagTransform(css, styleElement) {
  if (styleElement.styleSheet) {
    styleElement.styleSheet.cssText = css;
  } else {
    while (styleElement.firstChild) {
      styleElement.removeChild(styleElement.firstChild);
    }

    styleElement.appendChild(document.createTextNode(css));
  }
}

module.exports = styleTagTransform;

/***/ }),
/* 53 */
/***/ ((module, __webpack_exports__, __webpack_require__) => {

"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__)
/* harmony export */ });
/* harmony import */ var _node_modules_css_loader_dist_runtime_sourceMaps_js__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(54);
/* harmony import */ var _node_modules_css_loader_dist_runtime_sourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(_node_modules_css_loader_dist_runtime_sourceMaps_js__WEBPACK_IMPORTED_MODULE_0__);
/* harmony import */ var _node_modules_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(55);
/* harmony import */ var _node_modules_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(_node_modules_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1__);
// Imports


var ___CSS_LOADER_EXPORT___ = _node_modules_css_loader_dist_runtime_api_js__WEBPACK_IMPORTED_MODULE_1___default()((_node_modules_css_loader_dist_runtime_sourceMaps_js__WEBPACK_IMPORTED_MODULE_0___default()));
// Module
___CSS_LOADER_EXPORT___.push([module.id, "#main > .log > table, #main > .page.modbus > .data > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table, #main > .page.monitor > .components table, #main > .page.orchestrator > .components table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th, #main > .page.monitor > .components table th, #main > .page.orchestrator > .components table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td, #main > .page.monitor > .components table td, #main > .page.orchestrator > .components table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n@font-face {\n  font-family: \"Inter\";\n  src: url(\"fonts/Inter/Inter.woff2\") format(\"woff2\");\n  font-weight: 100 900;\n  font-style: oblique 0deg 10deg;\n  font-display: swap;\n}\n@font-face {\n  font-family: \"RobotoMono\";\n  src: url(\"fonts/RobotoMono/RobotoMono.woff2\") format(\"woff2\");\n  font-weight: 100 900;\n  font-style: normal;\n  font-display: swap;\n}\n@font-face {\n  font-family: \"RobotoMono\";\n  src: url(\"fonts/RobotoMono/RobotoMono-Italic.woff2\") format(\"woff2\");\n  font-weight: 100 900;\n  font-style: italic;\n  font-display: swap;\n}\n@font-face {\n  font-family: \"FontAwesome\";\n  src: url(\"fonts/FontAwesome/FontAwesome.woff2\") format(\"woff2\");\n  font-weight: normal;\n  font-style: normal;\n}\n.fa {\n  display: inline-block;\n  font: normal normal normal 14px/1 FontAwesome;\n  font-size: inherit;\n  text-rendering: auto;\n  -webkit-font-smoothing: antialiased;\n  -moz-osx-font-smoothing: grayscale;\n}\n\n/* makes the font 33% larger relative to the icon container */\n.fa-lg {\n  font-size: 1.33333333em;\n  line-height: 0.75em;\n  vertical-align: -15%;\n}\n\n.fa-2x {\n  font-size: 2em;\n}\n\n.fa-3x {\n  font-size: 3em;\n}\n\n.fa-4x {\n  font-size: 4em;\n}\n\n.fa-5x {\n  font-size: 5em;\n}\n\n.fa-fw {\n  width: 1.28571429em;\n  text-align: center;\n}\n\n.fa-ul {\n  padding-left: 0;\n  margin-left: 2.14285714em;\n  list-style-type: none;\n}\n\n.fa-ul > li {\n  position: relative;\n}\n\n.fa-li {\n  position: absolute;\n  left: -2.14285714em;\n  width: 2.14285714em;\n  top: 0.14285714em;\n  text-align: center;\n}\n\n.fa-li.fa-lg {\n  left: -1.85714286em;\n}\n\n.fa-border {\n  padding: 0.2em 0.25em 0.15em;\n  border: solid 0.08em #eeeeee;\n  border-radius: 0.1em;\n}\n\n.fa-pull-left {\n  float: left;\n}\n\n.fa-pull-right {\n  float: right;\n}\n\n.fa.fa-pull-left {\n  margin-right: 0.3em;\n}\n\n.fa.fa-pull-right {\n  margin-left: 0.3em;\n}\n\n/* Deprecated as of 4.4.0 */\n.pull-right {\n  float: right;\n}\n\n.pull-left {\n  float: left;\n}\n\n.fa.pull-left {\n  margin-right: 0.3em;\n}\n\n.fa.pull-right {\n  margin-left: 0.3em;\n}\n\n.fa-spin {\n  -webkit-animation: fa-spin 2s infinite linear;\n  animation: fa-spin 2s infinite linear;\n}\n\n.fa-pulse {\n  -webkit-animation: fa-spin 1s infinite steps(8);\n  animation: fa-spin 1s infinite steps(8);\n}\n\n@-webkit-keyframes fa-spin {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(359deg);\n    transform: rotate(359deg);\n  }\n}\n@keyframes fa-spin {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(359deg);\n    transform: rotate(359deg);\n  }\n}\n.fa-rotate-90 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=1)\";\n  -webkit-transform: rotate(90deg);\n  -ms-transform: rotate(90deg);\n  transform: rotate(90deg);\n}\n\n.fa-rotate-180 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=2)\";\n  -webkit-transform: rotate(180deg);\n  -ms-transform: rotate(180deg);\n  transform: rotate(180deg);\n}\n\n.fa-rotate-270 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=3)\";\n  -webkit-transform: rotate(270deg);\n  -ms-transform: rotate(270deg);\n  transform: rotate(270deg);\n}\n\n.fa-flip-horizontal {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=0, mirror=1)\";\n  -webkit-transform: scale(-1, 1);\n  -ms-transform: scale(-1, 1);\n  transform: scale(-1, 1);\n}\n\n.fa-flip-vertical {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=2, mirror=1)\";\n  -webkit-transform: scale(1, -1);\n  -ms-transform: scale(1, -1);\n  transform: scale(1, -1);\n}\n\n:root .fa-rotate-90,\n:root .fa-rotate-180,\n:root .fa-rotate-270,\n:root .fa-flip-horizontal,\n:root .fa-flip-vertical {\n  filter: none;\n}\n\n.fa-stack {\n  position: relative;\n  display: inline-block;\n  width: 2em;\n  height: 2em;\n  line-height: 2em;\n  vertical-align: middle;\n}\n\n.fa-stack-1x,\n.fa-stack-2x {\n  position: absolute;\n  left: 0;\n  width: 100%;\n  text-align: center;\n}\n\n.fa-stack-1x {\n  line-height: inherit;\n}\n\n.fa-stack-2x {\n  font-size: 2em;\n}\n\n.fa-inverse {\n  color: #ffffff;\n}\n\n/* Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen\n   readers do not read off random characters that represent icons */\n.fa-glass:before {\n  content: \"\\f000\";\n}\n\n.fa-music:before {\n  content: \"\\f001\";\n}\n\n.fa-search:before {\n  content: \"\\f002\";\n}\n\n.fa-envelope-o:before {\n  content: \"\\f003\";\n}\n\n.fa-heart:before {\n  content: \"\\f004\";\n}\n\n.fa-star:before {\n  content: \"\\f005\";\n}\n\n.fa-star-o:before {\n  content: \"\\f006\";\n}\n\n.fa-user:before {\n  content: \"\\f007\";\n}\n\n.fa-film:before {\n  content: \"\\f008\";\n}\n\n.fa-th-large:before {\n  content: \"\\f009\";\n}\n\n.fa-th:before {\n  content: \"\\f00a\";\n}\n\n.fa-th-list:before {\n  content: \"\\f00b\";\n}\n\n.fa-check:before {\n  content: \"\\f00c\";\n}\n\n.fa-remove:before,\n.fa-close:before,\n.fa-times:before {\n  content: \"\\f00d\";\n}\n\n.fa-search-plus:before {\n  content: \"\\f00e\";\n}\n\n.fa-search-minus:before {\n  content: \"\\f010\";\n}\n\n.fa-power-off:before {\n  content: \"\\f011\";\n}\n\n.fa-signal:before {\n  content: \"\\f012\";\n}\n\n.fa-gear:before,\n.fa-cog:before {\n  content: \"\\f013\";\n}\n\n.fa-trash-o:before {\n  content: \"\\f014\";\n}\n\n.fa-home:before {\n  content: \"\\f015\";\n}\n\n.fa-file-o:before {\n  content: \"\\f016\";\n}\n\n.fa-clock-o:before {\n  content: \"\\f017\";\n}\n\n.fa-road:before {\n  content: \"\\f018\";\n}\n\n.fa-download:before {\n  content: \"\\f019\";\n}\n\n.fa-arrow-circle-o-down:before {\n  content: \"\\f01a\";\n}\n\n.fa-arrow-circle-o-up:before {\n  content: \"\\f01b\";\n}\n\n.fa-inbox:before {\n  content: \"\\f01c\";\n}\n\n.fa-play-circle-o:before {\n  content: \"\\f01d\";\n}\n\n.fa-rotate-right:before,\n.fa-repeat:before {\n  content: \"\\f01e\";\n}\n\n.fa-refresh:before {\n  content: \"\\f021\";\n}\n\n.fa-list-alt:before {\n  content: \"\\f022\";\n}\n\n.fa-lock:before {\n  content: \"\\f023\";\n}\n\n.fa-flag:before {\n  content: \"\\f024\";\n}\n\n.fa-headphones:before {\n  content: \"\\f025\";\n}\n\n.fa-volume-off:before {\n  content: \"\\f026\";\n}\n\n.fa-volume-down:before {\n  content: \"\\f027\";\n}\n\n.fa-volume-up:before {\n  content: \"\\f028\";\n}\n\n.fa-qrcode:before {\n  content: \"\\f029\";\n}\n\n.fa-barcode:before {\n  content: \"\\f02a\";\n}\n\n.fa-tag:before {\n  content: \"\\f02b\";\n}\n\n.fa-tags:before {\n  content: \"\\f02c\";\n}\n\n.fa-book:before {\n  content: \"\\f02d\";\n}\n\n.fa-bookmark:before {\n  content: \"\\f02e\";\n}\n\n.fa-print:before {\n  content: \"\\f02f\";\n}\n\n.fa-camera:before {\n  content: \"\\f030\";\n}\n\n.fa-font:before {\n  content: \"\\f031\";\n}\n\n.fa-bold:before {\n  content: \"\\f032\";\n}\n\n.fa-italic:before {\n  content: \"\\f033\";\n}\n\n.fa-text-height:before {\n  content: \"\\f034\";\n}\n\n.fa-text-width:before {\n  content: \"\\f035\";\n}\n\n.fa-align-left:before {\n  content: \"\\f036\";\n}\n\n.fa-align-center:before {\n  content: \"\\f037\";\n}\n\n.fa-align-right:before {\n  content: \"\\f038\";\n}\n\n.fa-align-justify:before {\n  content: \"\\f039\";\n}\n\n.fa-list:before {\n  content: \"\\f03a\";\n}\n\n.fa-dedent:before,\n.fa-outdent:before {\n  content: \"\\f03b\";\n}\n\n.fa-indent:before {\n  content: \"\\f03c\";\n}\n\n.fa-video-camera:before {\n  content: \"\\f03d\";\n}\n\n.fa-photo:before,\n.fa-image:before,\n.fa-picture-o:before {\n  content: \"\\f03e\";\n}\n\n.fa-pencil:before {\n  content: \"\\f040\";\n}\n\n.fa-map-marker:before {\n  content: \"\\f041\";\n}\n\n.fa-adjust:before {\n  content: \"\\f042\";\n}\n\n.fa-tint:before {\n  content: \"\\f043\";\n}\n\n.fa-edit:before,\n.fa-pencil-square-o:before {\n  content: \"\\f044\";\n}\n\n.fa-share-square-o:before {\n  content: \"\\f045\";\n}\n\n.fa-check-square-o:before {\n  content: \"\\f046\";\n}\n\n.fa-arrows:before {\n  content: \"\\f047\";\n}\n\n.fa-step-backward:before {\n  content: \"\\f048\";\n}\n\n.fa-fast-backward:before {\n  content: \"\\f049\";\n}\n\n.fa-backward:before {\n  content: \"\\f04a\";\n}\n\n.fa-play:before {\n  content: \"\\f04b\";\n}\n\n.fa-pause:before {\n  content: \"\\f04c\";\n}\n\n.fa-stop:before {\n  content: \"\\f04d\";\n}\n\n.fa-forward:before {\n  content: \"\\f04e\";\n}\n\n.fa-fast-forward:before {\n  content: \"\\f050\";\n}\n\n.fa-step-forward:before {\n  content: \"\\f051\";\n}\n\n.fa-eject:before {\n  content: \"\\f052\";\n}\n\n.fa-chevron-left:before {\n  content: \"\\f053\";\n}\n\n.fa-chevron-right:before {\n  content: \"\\f054\";\n}\n\n.fa-plus-circle:before {\n  content: \"\\f055\";\n}\n\n.fa-minus-circle:before {\n  content: \"\\f056\";\n}\n\n.fa-times-circle:before {\n  content: \"\\f057\";\n}\n\n.fa-check-circle:before {\n  content: \"\\f058\";\n}\n\n.fa-question-circle:before {\n  content: \"\\f059\";\n}\n\n.fa-info-circle:before {\n  content: \"\\f05a\";\n}\n\n.fa-crosshairs:before {\n  content: \"\\f05b\";\n}\n\n.fa-times-circle-o:before {\n  content: \"\\f05c\";\n}\n\n.fa-check-circle-o:before {\n  content: \"\\f05d\";\n}\n\n.fa-ban:before {\n  content: \"\\f05e\";\n}\n\n.fa-arrow-left:before {\n  content: \"\\f060\";\n}\n\n.fa-arrow-right:before {\n  content: \"\\f061\";\n}\n\n.fa-arrow-up:before {\n  content: \"\\f062\";\n}\n\n.fa-arrow-down:before {\n  content: \"\\f063\";\n}\n\n.fa-mail-forward:before,\n.fa-share:before {\n  content: \"\\f064\";\n}\n\n.fa-expand:before {\n  content: \"\\f065\";\n}\n\n.fa-compress:before {\n  content: \"\\f066\";\n}\n\n.fa-plus:before {\n  content: \"\\f067\";\n}\n\n.fa-minus:before {\n  content: \"\\f068\";\n}\n\n.fa-asterisk:before {\n  content: \"\\f069\";\n}\n\n.fa-exclamation-circle:before {\n  content: \"\\f06a\";\n}\n\n.fa-gift:before {\n  content: \"\\f06b\";\n}\n\n.fa-leaf:before {\n  content: \"\\f06c\";\n}\n\n.fa-fire:before {\n  content: \"\\f06d\";\n}\n\n.fa-eye:before {\n  content: \"\\f06e\";\n}\n\n.fa-eye-slash:before {\n  content: \"\\f070\";\n}\n\n.fa-warning:before,\n.fa-exclamation-triangle:before {\n  content: \"\\f071\";\n}\n\n.fa-plane:before {\n  content: \"\\f072\";\n}\n\n.fa-calendar:before {\n  content: \"\\f073\";\n}\n\n.fa-random:before {\n  content: \"\\f074\";\n}\n\n.fa-comment:before {\n  content: \"\\f075\";\n}\n\n.fa-magnet:before {\n  content: \"\\f076\";\n}\n\n.fa-chevron-up:before {\n  content: \"\\f077\";\n}\n\n.fa-chevron-down:before {\n  content: \"\\f078\";\n}\n\n.fa-retweet:before {\n  content: \"\\f079\";\n}\n\n.fa-shopping-cart:before {\n  content: \"\\f07a\";\n}\n\n.fa-folder:before {\n  content: \"\\f07b\";\n}\n\n.fa-folder-open:before {\n  content: \"\\f07c\";\n}\n\n.fa-arrows-v:before {\n  content: \"\\f07d\";\n}\n\n.fa-arrows-h:before {\n  content: \"\\f07e\";\n}\n\n.fa-bar-chart-o:before,\n.fa-bar-chart:before {\n  content: \"\\f080\";\n}\n\n.fa-twitter-square:before {\n  content: \"\\f081\";\n}\n\n.fa-facebook-square:before {\n  content: \"\\f082\";\n}\n\n.fa-camera-retro:before {\n  content: \"\\f083\";\n}\n\n.fa-key:before {\n  content: \"\\f084\";\n}\n\n.fa-gears:before,\n.fa-cogs:before {\n  content: \"\\f085\";\n}\n\n.fa-comments:before {\n  content: \"\\f086\";\n}\n\n.fa-thumbs-o-up:before {\n  content: \"\\f087\";\n}\n\n.fa-thumbs-o-down:before {\n  content: \"\\f088\";\n}\n\n.fa-star-half:before {\n  content: \"\\f089\";\n}\n\n.fa-heart-o:before {\n  content: \"\\f08a\";\n}\n\n.fa-sign-out:before {\n  content: \"\\f08b\";\n}\n\n.fa-linkedin-square:before {\n  content: \"\\f08c\";\n}\n\n.fa-thumb-tack:before {\n  content: \"\\f08d\";\n}\n\n.fa-external-link:before {\n  content: \"\\f08e\";\n}\n\n.fa-sign-in:before {\n  content: \"\\f090\";\n}\n\n.fa-trophy:before {\n  content: \"\\f091\";\n}\n\n.fa-github-square:before {\n  content: \"\\f092\";\n}\n\n.fa-upload:before {\n  content: \"\\f093\";\n}\n\n.fa-lemon-o:before {\n  content: \"\\f094\";\n}\n\n.fa-phone:before {\n  content: \"\\f095\";\n}\n\n.fa-square-o:before {\n  content: \"\\f096\";\n}\n\n.fa-bookmark-o:before {\n  content: \"\\f097\";\n}\n\n.fa-phone-square:before {\n  content: \"\\f098\";\n}\n\n.fa-twitter:before {\n  content: \"\\f099\";\n}\n\n.fa-facebook-f:before,\n.fa-facebook:before {\n  content: \"\\f09a\";\n}\n\n.fa-github:before {\n  content: \"\\f09b\";\n}\n\n.fa-unlock:before {\n  content: \"\\f09c\";\n}\n\n.fa-credit-card:before {\n  content: \"\\f09d\";\n}\n\n.fa-feed:before,\n.fa-rss:before {\n  content: \"\\f09e\";\n}\n\n.fa-hdd-o:before {\n  content: \"\\f0a0\";\n}\n\n.fa-bullhorn:before {\n  content: \"\\f0a1\";\n}\n\n.fa-bell:before {\n  content: \"\\f0f3\";\n}\n\n.fa-certificate:before {\n  content: \"\\f0a3\";\n}\n\n.fa-hand-o-right:before {\n  content: \"\\f0a4\";\n}\n\n.fa-hand-o-left:before {\n  content: \"\\f0a5\";\n}\n\n.fa-hand-o-up:before {\n  content: \"\\f0a6\";\n}\n\n.fa-hand-o-down:before {\n  content: \"\\f0a7\";\n}\n\n.fa-arrow-circle-left:before {\n  content: \"\\f0a8\";\n}\n\n.fa-arrow-circle-right:before {\n  content: \"\\f0a9\";\n}\n\n.fa-arrow-circle-up:before {\n  content: \"\\f0aa\";\n}\n\n.fa-arrow-circle-down:before {\n  content: \"\\f0ab\";\n}\n\n.fa-globe:before {\n  content: \"\\f0ac\";\n}\n\n.fa-wrench:before {\n  content: \"\\f0ad\";\n}\n\n.fa-tasks:before {\n  content: \"\\f0ae\";\n}\n\n.fa-filter:before {\n  content: \"\\f0b0\";\n}\n\n.fa-briefcase:before {\n  content: \"\\f0b1\";\n}\n\n.fa-arrows-alt:before {\n  content: \"\\f0b2\";\n}\n\n.fa-group:before,\n.fa-users:before {\n  content: \"\\f0c0\";\n}\n\n.fa-chain:before,\n.fa-link:before {\n  content: \"\\f0c1\";\n}\n\n.fa-cloud:before {\n  content: \"\\f0c2\";\n}\n\n.fa-flask:before {\n  content: \"\\f0c3\";\n}\n\n.fa-cut:before,\n.fa-scissors:before {\n  content: \"\\f0c4\";\n}\n\n.fa-copy:before,\n.fa-files-o:before {\n  content: \"\\f0c5\";\n}\n\n.fa-paperclip:before {\n  content: \"\\f0c6\";\n}\n\n.fa-save:before,\n.fa-floppy-o:before {\n  content: \"\\f0c7\";\n}\n\n.fa-square:before {\n  content: \"\\f0c8\";\n}\n\n.fa-navicon:before,\n.fa-reorder:before,\n.fa-bars:before {\n  content: \"\\f0c9\";\n}\n\n.fa-list-ul:before {\n  content: \"\\f0ca\";\n}\n\n.fa-list-ol:before {\n  content: \"\\f0cb\";\n}\n\n.fa-strikethrough:before {\n  content: \"\\f0cc\";\n}\n\n.fa-underline:before {\n  content: \"\\f0cd\";\n}\n\n.fa-table:before {\n  content: \"\\f0ce\";\n}\n\n.fa-magic:before {\n  content: \"\\f0d0\";\n}\n\n.fa-truck:before {\n  content: \"\\f0d1\";\n}\n\n.fa-pinterest:before {\n  content: \"\\f0d2\";\n}\n\n.fa-pinterest-square:before {\n  content: \"\\f0d3\";\n}\n\n.fa-google-plus-square:before {\n  content: \"\\f0d4\";\n}\n\n.fa-google-plus:before {\n  content: \"\\f0d5\";\n}\n\n.fa-money:before {\n  content: \"\\f0d6\";\n}\n\n.fa-caret-down:before {\n  content: \"\\f0d7\";\n}\n\n.fa-caret-up:before {\n  content: \"\\f0d8\";\n}\n\n.fa-caret-left:before {\n  content: \"\\f0d9\";\n}\n\n.fa-caret-right:before {\n  content: \"\\f0da\";\n}\n\n.fa-columns:before {\n  content: \"\\f0db\";\n}\n\n.fa-unsorted:before,\n.fa-sort:before {\n  content: \"\\f0dc\";\n}\n\n.fa-sort-down:before,\n.fa-sort-desc:before {\n  content: \"\\f0dd\";\n}\n\n.fa-sort-up:before,\n.fa-sort-asc:before {\n  content: \"\\f0de\";\n}\n\n.fa-envelope:before {\n  content: \"\\f0e0\";\n}\n\n.fa-linkedin:before {\n  content: \"\\f0e1\";\n}\n\n.fa-rotate-left:before,\n.fa-undo:before {\n  content: \"\\f0e2\";\n}\n\n.fa-legal:before,\n.fa-gavel:before {\n  content: \"\\f0e3\";\n}\n\n.fa-dashboard:before,\n.fa-tachometer:before {\n  content: \"\\f0e4\";\n}\n\n.fa-comment-o:before {\n  content: \"\\f0e5\";\n}\n\n.fa-comments-o:before {\n  content: \"\\f0e6\";\n}\n\n.fa-flash:before,\n.fa-bolt:before {\n  content: \"\\f0e7\";\n}\n\n.fa-sitemap:before {\n  content: \"\\f0e8\";\n}\n\n.fa-umbrella:before {\n  content: \"\\f0e9\";\n}\n\n.fa-paste:before,\n.fa-clipboard:before {\n  content: \"\\f0ea\";\n}\n\n.fa-lightbulb-o:before {\n  content: \"\\f0eb\";\n}\n\n.fa-exchange:before {\n  content: \"\\f0ec\";\n}\n\n.fa-cloud-download:before {\n  content: \"\\f0ed\";\n}\n\n.fa-cloud-upload:before {\n  content: \"\\f0ee\";\n}\n\n.fa-user-md:before {\n  content: \"\\f0f0\";\n}\n\n.fa-stethoscope:before {\n  content: \"\\f0f1\";\n}\n\n.fa-suitcase:before {\n  content: \"\\f0f2\";\n}\n\n.fa-bell-o:before {\n  content: \"\\f0a2\";\n}\n\n.fa-coffee:before {\n  content: \"\\f0f4\";\n}\n\n.fa-cutlery:before {\n  content: \"\\f0f5\";\n}\n\n.fa-file-text-o:before {\n  content: \"\\f0f6\";\n}\n\n.fa-building-o:before {\n  content: \"\\f0f7\";\n}\n\n.fa-hospital-o:before {\n  content: \"\\f0f8\";\n}\n\n.fa-ambulance:before {\n  content: \"\\f0f9\";\n}\n\n.fa-medkit:before {\n  content: \"\\f0fa\";\n}\n\n.fa-fighter-jet:before {\n  content: \"\\f0fb\";\n}\n\n.fa-beer:before {\n  content: \"\\f0fc\";\n}\n\n.fa-h-square:before {\n  content: \"\\f0fd\";\n}\n\n.fa-plus-square:before {\n  content: \"\\f0fe\";\n}\n\n.fa-angle-double-left:before {\n  content: \"\\f100\";\n}\n\n.fa-angle-double-right:before {\n  content: \"\\f101\";\n}\n\n.fa-angle-double-up:before {\n  content: \"\\f102\";\n}\n\n.fa-angle-double-down:before {\n  content: \"\\f103\";\n}\n\n.fa-angle-left:before {\n  content: \"\\f104\";\n}\n\n.fa-angle-right:before {\n  content: \"\\f105\";\n}\n\n.fa-angle-up:before {\n  content: \"\\f106\";\n}\n\n.fa-angle-down:before {\n  content: \"\\f107\";\n}\n\n.fa-desktop:before {\n  content: \"\\f108\";\n}\n\n.fa-laptop:before {\n  content: \"\\f109\";\n}\n\n.fa-tablet:before {\n  content: \"\\f10a\";\n}\n\n.fa-mobile-phone:before,\n.fa-mobile:before {\n  content: \"\\f10b\";\n}\n\n.fa-circle-o:before {\n  content: \"\\f10c\";\n}\n\n.fa-quote-left:before {\n  content: \"\\f10d\";\n}\n\n.fa-quote-right:before {\n  content: \"\\f10e\";\n}\n\n.fa-spinner:before {\n  content: \"\\f110\";\n}\n\n.fa-circle:before {\n  content: \"\\f111\";\n}\n\n.fa-mail-reply:before,\n.fa-reply:before {\n  content: \"\\f112\";\n}\n\n.fa-github-alt:before {\n  content: \"\\f113\";\n}\n\n.fa-folder-o:before {\n  content: \"\\f114\";\n}\n\n.fa-folder-open-o:before {\n  content: \"\\f115\";\n}\n\n.fa-smile-o:before {\n  content: \"\\f118\";\n}\n\n.fa-frown-o:before {\n  content: \"\\f119\";\n}\n\n.fa-meh-o:before {\n  content: \"\\f11a\";\n}\n\n.fa-gamepad:before {\n  content: \"\\f11b\";\n}\n\n.fa-keyboard-o:before {\n  content: \"\\f11c\";\n}\n\n.fa-flag-o:before {\n  content: \"\\f11d\";\n}\n\n.fa-flag-checkered:before {\n  content: \"\\f11e\";\n}\n\n.fa-terminal:before {\n  content: \"\\f120\";\n}\n\n.fa-code:before {\n  content: \"\\f121\";\n}\n\n.fa-mail-reply-all:before,\n.fa-reply-all:before {\n  content: \"\\f122\";\n}\n\n.fa-star-half-empty:before,\n.fa-star-half-full:before,\n.fa-star-half-o:before {\n  content: \"\\f123\";\n}\n\n.fa-location-arrow:before {\n  content: \"\\f124\";\n}\n\n.fa-crop:before {\n  content: \"\\f125\";\n}\n\n.fa-code-fork:before {\n  content: \"\\f126\";\n}\n\n.fa-unlink:before,\n.fa-chain-broken:before {\n  content: \"\\f127\";\n}\n\n.fa-question:before {\n  content: \"\\f128\";\n}\n\n.fa-info:before {\n  content: \"\\f129\";\n}\n\n.fa-exclamation:before {\n  content: \"\\f12a\";\n}\n\n.fa-superscript:before {\n  content: \"\\f12b\";\n}\n\n.fa-subscript:before {\n  content: \"\\f12c\";\n}\n\n.fa-eraser:before {\n  content: \"\\f12d\";\n}\n\n.fa-puzzle-piece:before {\n  content: \"\\f12e\";\n}\n\n.fa-microphone:before {\n  content: \"\\f130\";\n}\n\n.fa-microphone-slash:before {\n  content: \"\\f131\";\n}\n\n.fa-shield:before {\n  content: \"\\f132\";\n}\n\n.fa-calendar-o:before {\n  content: \"\\f133\";\n}\n\n.fa-fire-extinguisher:before {\n  content: \"\\f134\";\n}\n\n.fa-rocket:before {\n  content: \"\\f135\";\n}\n\n.fa-maxcdn:before {\n  content: \"\\f136\";\n}\n\n.fa-chevron-circle-left:before {\n  content: \"\\f137\";\n}\n\n.fa-chevron-circle-right:before {\n  content: \"\\f138\";\n}\n\n.fa-chevron-circle-up:before {\n  content: \"\\f139\";\n}\n\n.fa-chevron-circle-down:before {\n  content: \"\\f13a\";\n}\n\n.fa-html5:before {\n  content: \"\\f13b\";\n}\n\n.fa-css3:before {\n  content: \"\\f13c\";\n}\n\n.fa-anchor:before {\n  content: \"\\f13d\";\n}\n\n.fa-unlock-alt:before {\n  content: \"\\f13e\";\n}\n\n.fa-bullseye:before {\n  content: \"\\f140\";\n}\n\n.fa-ellipsis-h:before {\n  content: \"\\f141\";\n}\n\n.fa-ellipsis-v:before {\n  content: \"\\f142\";\n}\n\n.fa-rss-square:before {\n  content: \"\\f143\";\n}\n\n.fa-play-circle:before {\n  content: \"\\f144\";\n}\n\n.fa-ticket:before {\n  content: \"\\f145\";\n}\n\n.fa-minus-square:before {\n  content: \"\\f146\";\n}\n\n.fa-minus-square-o:before {\n  content: \"\\f147\";\n}\n\n.fa-level-up:before {\n  content: \"\\f148\";\n}\n\n.fa-level-down:before {\n  content: \"\\f149\";\n}\n\n.fa-check-square:before {\n  content: \"\\f14a\";\n}\n\n.fa-pencil-square:before {\n  content: \"\\f14b\";\n}\n\n.fa-external-link-square:before {\n  content: \"\\f14c\";\n}\n\n.fa-share-square:before {\n  content: \"\\f14d\";\n}\n\n.fa-compass:before {\n  content: \"\\f14e\";\n}\n\n.fa-toggle-down:before,\n.fa-caret-square-o-down:before {\n  content: \"\\f150\";\n}\n\n.fa-toggle-up:before,\n.fa-caret-square-o-up:before {\n  content: \"\\f151\";\n}\n\n.fa-toggle-right:before,\n.fa-caret-square-o-right:before {\n  content: \"\\f152\";\n}\n\n.fa-euro:before,\n.fa-eur:before {\n  content: \"\\f153\";\n}\n\n.fa-gbp:before {\n  content: \"\\f154\";\n}\n\n.fa-dollar:before,\n.fa-usd:before {\n  content: \"\\f155\";\n}\n\n.fa-rupee:before,\n.fa-inr:before {\n  content: \"\\f156\";\n}\n\n.fa-cny:before,\n.fa-rmb:before,\n.fa-yen:before,\n.fa-jpy:before {\n  content: \"\\f157\";\n}\n\n.fa-ruble:before,\n.fa-rouble:before,\n.fa-rub:before {\n  content: \"\\f158\";\n}\n\n.fa-won:before,\n.fa-krw:before {\n  content: \"\\f159\";\n}\n\n.fa-bitcoin:before,\n.fa-btc:before {\n  content: \"\\f15a\";\n}\n\n.fa-file:before {\n  content: \"\\f15b\";\n}\n\n.fa-file-text:before {\n  content: \"\\f15c\";\n}\n\n.fa-sort-alpha-asc:before {\n  content: \"\\f15d\";\n}\n\n.fa-sort-alpha-desc:before {\n  content: \"\\f15e\";\n}\n\n.fa-sort-amount-asc:before {\n  content: \"\\f160\";\n}\n\n.fa-sort-amount-desc:before {\n  content: \"\\f161\";\n}\n\n.fa-sort-numeric-asc:before {\n  content: \"\\f162\";\n}\n\n.fa-sort-numeric-desc:before {\n  content: \"\\f163\";\n}\n\n.fa-thumbs-up:before {\n  content: \"\\f164\";\n}\n\n.fa-thumbs-down:before {\n  content: \"\\f165\";\n}\n\n.fa-youtube-square:before {\n  content: \"\\f166\";\n}\n\n.fa-youtube:before {\n  content: \"\\f167\";\n}\n\n.fa-xing:before {\n  content: \"\\f168\";\n}\n\n.fa-xing-square:before {\n  content: \"\\f169\";\n}\n\n.fa-youtube-play:before {\n  content: \"\\f16a\";\n}\n\n.fa-dropbox:before {\n  content: \"\\f16b\";\n}\n\n.fa-stack-overflow:before {\n  content: \"\\f16c\";\n}\n\n.fa-instagram:before {\n  content: \"\\f16d\";\n}\n\n.fa-flickr:before {\n  content: \"\\f16e\";\n}\n\n.fa-adn:before {\n  content: \"\\f170\";\n}\n\n.fa-bitbucket:before {\n  content: \"\\f171\";\n}\n\n.fa-bitbucket-square:before {\n  content: \"\\f172\";\n}\n\n.fa-tumblr:before {\n  content: \"\\f173\";\n}\n\n.fa-tumblr-square:before {\n  content: \"\\f174\";\n}\n\n.fa-long-arrow-down:before {\n  content: \"\\f175\";\n}\n\n.fa-long-arrow-up:before {\n  content: \"\\f176\";\n}\n\n.fa-long-arrow-left:before {\n  content: \"\\f177\";\n}\n\n.fa-long-arrow-right:before {\n  content: \"\\f178\";\n}\n\n.fa-apple:before {\n  content: \"\\f179\";\n}\n\n.fa-windows:before {\n  content: \"\\f17a\";\n}\n\n.fa-android:before {\n  content: \"\\f17b\";\n}\n\n.fa-linux:before {\n  content: \"\\f17c\";\n}\n\n.fa-dribbble:before {\n  content: \"\\f17d\";\n}\n\n.fa-skype:before {\n  content: \"\\f17e\";\n}\n\n.fa-foursquare:before {\n  content: \"\\f180\";\n}\n\n.fa-trello:before {\n  content: \"\\f181\";\n}\n\n.fa-female:before {\n  content: \"\\f182\";\n}\n\n.fa-male:before {\n  content: \"\\f183\";\n}\n\n.fa-gittip:before,\n.fa-gratipay:before {\n  content: \"\\f184\";\n}\n\n.fa-sun-o:before {\n  content: \"\\f185\";\n}\n\n.fa-moon-o:before {\n  content: \"\\f186\";\n}\n\n.fa-archive:before {\n  content: \"\\f187\";\n}\n\n.fa-bug:before {\n  content: \"\\f188\";\n}\n\n.fa-vk:before {\n  content: \"\\f189\";\n}\n\n.fa-weibo:before {\n  content: \"\\f18a\";\n}\n\n.fa-renren:before {\n  content: \"\\f18b\";\n}\n\n.fa-pagelines:before {\n  content: \"\\f18c\";\n}\n\n.fa-stack-exchange:before {\n  content: \"\\f18d\";\n}\n\n.fa-arrow-circle-o-right:before {\n  content: \"\\f18e\";\n}\n\n.fa-arrow-circle-o-left:before {\n  content: \"\\f190\";\n}\n\n.fa-toggle-left:before,\n.fa-caret-square-o-left:before {\n  content: \"\\f191\";\n}\n\n.fa-dot-circle-o:before {\n  content: \"\\f192\";\n}\n\n.fa-wheelchair:before {\n  content: \"\\f193\";\n}\n\n.fa-vimeo-square:before {\n  content: \"\\f194\";\n}\n\n.fa-turkish-lira:before,\n.fa-try:before {\n  content: \"\\f195\";\n}\n\n.fa-plus-square-o:before {\n  content: \"\\f196\";\n}\n\n.fa-space-shuttle:before {\n  content: \"\\f197\";\n}\n\n.fa-slack:before {\n  content: \"\\f198\";\n}\n\n.fa-envelope-square:before {\n  content: \"\\f199\";\n}\n\n.fa-wordpress:before {\n  content: \"\\f19a\";\n}\n\n.fa-openid:before {\n  content: \"\\f19b\";\n}\n\n.fa-institution:before,\n.fa-bank:before,\n.fa-university:before {\n  content: \"\\f19c\";\n}\n\n.fa-mortar-board:before,\n.fa-graduation-cap:before {\n  content: \"\\f19d\";\n}\n\n.fa-yahoo:before {\n  content: \"\\f19e\";\n}\n\n.fa-google:before {\n  content: \"\\f1a0\";\n}\n\n.fa-reddit:before {\n  content: \"\\f1a1\";\n}\n\n.fa-reddit-square:before {\n  content: \"\\f1a2\";\n}\n\n.fa-stumbleupon-circle:before {\n  content: \"\\f1a3\";\n}\n\n.fa-stumbleupon:before {\n  content: \"\\f1a4\";\n}\n\n.fa-delicious:before {\n  content: \"\\f1a5\";\n}\n\n.fa-digg:before {\n  content: \"\\f1a6\";\n}\n\n.fa-pied-piper-pp:before {\n  content: \"\\f1a7\";\n}\n\n.fa-pied-piper-alt:before {\n  content: \"\\f1a8\";\n}\n\n.fa-drupal:before {\n  content: \"\\f1a9\";\n}\n\n.fa-joomla:before {\n  content: \"\\f1aa\";\n}\n\n.fa-language:before {\n  content: \"\\f1ab\";\n}\n\n.fa-fax:before {\n  content: \"\\f1ac\";\n}\n\n.fa-building:before {\n  content: \"\\f1ad\";\n}\n\n.fa-child:before {\n  content: \"\\f1ae\";\n}\n\n.fa-paw:before {\n  content: \"\\f1b0\";\n}\n\n.fa-spoon:before {\n  content: \"\\f1b1\";\n}\n\n.fa-cube:before {\n  content: \"\\f1b2\";\n}\n\n.fa-cubes:before {\n  content: \"\\f1b3\";\n}\n\n.fa-behance:before {\n  content: \"\\f1b4\";\n}\n\n.fa-behance-square:before {\n  content: \"\\f1b5\";\n}\n\n.fa-steam:before {\n  content: \"\\f1b6\";\n}\n\n.fa-steam-square:before {\n  content: \"\\f1b7\";\n}\n\n.fa-recycle:before {\n  content: \"\\f1b8\";\n}\n\n.fa-automobile:before,\n.fa-car:before {\n  content: \"\\f1b9\";\n}\n\n.fa-cab:before,\n.fa-taxi:before {\n  content: \"\\f1ba\";\n}\n\n.fa-tree:before {\n  content: \"\\f1bb\";\n}\n\n.fa-spotify:before {\n  content: \"\\f1bc\";\n}\n\n.fa-deviantart:before {\n  content: \"\\f1bd\";\n}\n\n.fa-soundcloud:before {\n  content: \"\\f1be\";\n}\n\n.fa-database:before {\n  content: \"\\f1c0\";\n}\n\n.fa-file-pdf-o:before {\n  content: \"\\f1c1\";\n}\n\n.fa-file-word-o:before {\n  content: \"\\f1c2\";\n}\n\n.fa-file-excel-o:before {\n  content: \"\\f1c3\";\n}\n\n.fa-file-powerpoint-o:before {\n  content: \"\\f1c4\";\n}\n\n.fa-file-photo-o:before,\n.fa-file-picture-o:before,\n.fa-file-image-o:before {\n  content: \"\\f1c5\";\n}\n\n.fa-file-zip-o:before,\n.fa-file-archive-o:before {\n  content: \"\\f1c6\";\n}\n\n.fa-file-sound-o:before,\n.fa-file-audio-o:before {\n  content: \"\\f1c7\";\n}\n\n.fa-file-movie-o:before,\n.fa-file-video-o:before {\n  content: \"\\f1c8\";\n}\n\n.fa-file-code-o:before {\n  content: \"\\f1c9\";\n}\n\n.fa-vine:before {\n  content: \"\\f1ca\";\n}\n\n.fa-codepen:before {\n  content: \"\\f1cb\";\n}\n\n.fa-jsfiddle:before {\n  content: \"\\f1cc\";\n}\n\n.fa-life-bouy:before,\n.fa-life-buoy:before,\n.fa-life-saver:before,\n.fa-support:before,\n.fa-life-ring:before {\n  content: \"\\f1cd\";\n}\n\n.fa-circle-o-notch:before {\n  content: \"\\f1ce\";\n}\n\n.fa-ra:before,\n.fa-resistance:before,\n.fa-rebel:before {\n  content: \"\\f1d0\";\n}\n\n.fa-ge:before,\n.fa-empire:before {\n  content: \"\\f1d1\";\n}\n\n.fa-git-square:before {\n  content: \"\\f1d2\";\n}\n\n.fa-git:before {\n  content: \"\\f1d3\";\n}\n\n.fa-y-combinator-square:before,\n.fa-yc-square:before,\n.fa-hacker-news:before {\n  content: \"\\f1d4\";\n}\n\n.fa-tencent-weibo:before {\n  content: \"\\f1d5\";\n}\n\n.fa-qq:before {\n  content: \"\\f1d6\";\n}\n\n.fa-wechat:before,\n.fa-weixin:before {\n  content: \"\\f1d7\";\n}\n\n.fa-send:before,\n.fa-paper-plane:before {\n  content: \"\\f1d8\";\n}\n\n.fa-send-o:before,\n.fa-paper-plane-o:before {\n  content: \"\\f1d9\";\n}\n\n.fa-history:before {\n  content: \"\\f1da\";\n}\n\n.fa-circle-thin:before {\n  content: \"\\f1db\";\n}\n\n.fa-header:before {\n  content: \"\\f1dc\";\n}\n\n.fa-paragraph:before {\n  content: \"\\f1dd\";\n}\n\n.fa-sliders:before {\n  content: \"\\f1de\";\n}\n\n.fa-share-alt:before {\n  content: \"\\f1e0\";\n}\n\n.fa-share-alt-square:before {\n  content: \"\\f1e1\";\n}\n\n.fa-bomb:before {\n  content: \"\\f1e2\";\n}\n\n.fa-soccer-ball-o:before,\n.fa-futbol-o:before {\n  content: \"\\f1e3\";\n}\n\n.fa-tty:before {\n  content: \"\\f1e4\";\n}\n\n.fa-binoculars:before {\n  content: \"\\f1e5\";\n}\n\n.fa-plug:before {\n  content: \"\\f1e6\";\n}\n\n.fa-slideshare:before {\n  content: \"\\f1e7\";\n}\n\n.fa-twitch:before {\n  content: \"\\f1e8\";\n}\n\n.fa-yelp:before {\n  content: \"\\f1e9\";\n}\n\n.fa-newspaper-o:before {\n  content: \"\\f1ea\";\n}\n\n.fa-wifi:before {\n  content: \"\\f1eb\";\n}\n\n.fa-calculator:before {\n  content: \"\\f1ec\";\n}\n\n.fa-paypal:before {\n  content: \"\\f1ed\";\n}\n\n.fa-google-wallet:before {\n  content: \"\\f1ee\";\n}\n\n.fa-cc-visa:before {\n  content: \"\\f1f0\";\n}\n\n.fa-cc-mastercard:before {\n  content: \"\\f1f1\";\n}\n\n.fa-cc-discover:before {\n  content: \"\\f1f2\";\n}\n\n.fa-cc-amex:before {\n  content: \"\\f1f3\";\n}\n\n.fa-cc-paypal:before {\n  content: \"\\f1f4\";\n}\n\n.fa-cc-stripe:before {\n  content: \"\\f1f5\";\n}\n\n.fa-bell-slash:before {\n  content: \"\\f1f6\";\n}\n\n.fa-bell-slash-o:before {\n  content: \"\\f1f7\";\n}\n\n.fa-trash:before {\n  content: \"\\f1f8\";\n}\n\n.fa-copyright:before {\n  content: \"\\f1f9\";\n}\n\n.fa-at:before {\n  content: \"\\f1fa\";\n}\n\n.fa-eyedropper:before {\n  content: \"\\f1fb\";\n}\n\n.fa-paint-brush:before {\n  content: \"\\f1fc\";\n}\n\n.fa-birthday-cake:before {\n  content: \"\\f1fd\";\n}\n\n.fa-area-chart:before {\n  content: \"\\f1fe\";\n}\n\n.fa-pie-chart:before {\n  content: \"\\f200\";\n}\n\n.fa-line-chart:before {\n  content: \"\\f201\";\n}\n\n.fa-lastfm:before {\n  content: \"\\f202\";\n}\n\n.fa-lastfm-square:before {\n  content: \"\\f203\";\n}\n\n.fa-toggle-off:before {\n  content: \"\\f204\";\n}\n\n.fa-toggle-on:before {\n  content: \"\\f205\";\n}\n\n.fa-bicycle:before {\n  content: \"\\f206\";\n}\n\n.fa-bus:before {\n  content: \"\\f207\";\n}\n\n.fa-ioxhost:before {\n  content: \"\\f208\";\n}\n\n.fa-angellist:before {\n  content: \"\\f209\";\n}\n\n.fa-cc:before {\n  content: \"\\f20a\";\n}\n\n.fa-shekel:before,\n.fa-sheqel:before,\n.fa-ils:before {\n  content: \"\\f20b\";\n}\n\n.fa-meanpath:before {\n  content: \"\\f20c\";\n}\n\n.fa-buysellads:before {\n  content: \"\\f20d\";\n}\n\n.fa-connectdevelop:before {\n  content: \"\\f20e\";\n}\n\n.fa-dashcube:before {\n  content: \"\\f210\";\n}\n\n.fa-forumbee:before {\n  content: \"\\f211\";\n}\n\n.fa-leanpub:before {\n  content: \"\\f212\";\n}\n\n.fa-sellsy:before {\n  content: \"\\f213\";\n}\n\n.fa-shirtsinbulk:before {\n  content: \"\\f214\";\n}\n\n.fa-simplybuilt:before {\n  content: \"\\f215\";\n}\n\n.fa-skyatlas:before {\n  content: \"\\f216\";\n}\n\n.fa-cart-plus:before {\n  content: \"\\f217\";\n}\n\n.fa-cart-arrow-down:before {\n  content: \"\\f218\";\n}\n\n.fa-diamond:before {\n  content: \"\\f219\";\n}\n\n.fa-ship:before {\n  content: \"\\f21a\";\n}\n\n.fa-user-secret:before {\n  content: \"\\f21b\";\n}\n\n.fa-motorcycle:before {\n  content: \"\\f21c\";\n}\n\n.fa-street-view:before {\n  content: \"\\f21d\";\n}\n\n.fa-heartbeat:before {\n  content: \"\\f21e\";\n}\n\n.fa-venus:before {\n  content: \"\\f221\";\n}\n\n.fa-mars:before {\n  content: \"\\f222\";\n}\n\n.fa-mercury:before {\n  content: \"\\f223\";\n}\n\n.fa-intersex:before,\n.fa-transgender:before {\n  content: \"\\f224\";\n}\n\n.fa-transgender-alt:before {\n  content: \"\\f225\";\n}\n\n.fa-venus-double:before {\n  content: \"\\f226\";\n}\n\n.fa-mars-double:before {\n  content: \"\\f227\";\n}\n\n.fa-venus-mars:before {\n  content: \"\\f228\";\n}\n\n.fa-mars-stroke:before {\n  content: \"\\f229\";\n}\n\n.fa-mars-stroke-v:before {\n  content: \"\\f22a\";\n}\n\n.fa-mars-stroke-h:before {\n  content: \"\\f22b\";\n}\n\n.fa-neuter:before {\n  content: \"\\f22c\";\n}\n\n.fa-genderless:before {\n  content: \"\\f22d\";\n}\n\n.fa-facebook-official:before {\n  content: \"\\f230\";\n}\n\n.fa-pinterest-p:before {\n  content: \"\\f231\";\n}\n\n.fa-whatsapp:before {\n  content: \"\\f232\";\n}\n\n.fa-server:before {\n  content: \"\\f233\";\n}\n\n.fa-user-plus:before {\n  content: \"\\f234\";\n}\n\n.fa-user-times:before {\n  content: \"\\f235\";\n}\n\n.fa-hotel:before,\n.fa-bed:before {\n  content: \"\\f236\";\n}\n\n.fa-viacoin:before {\n  content: \"\\f237\";\n}\n\n.fa-train:before {\n  content: \"\\f238\";\n}\n\n.fa-subway:before {\n  content: \"\\f239\";\n}\n\n.fa-medium:before {\n  content: \"\\f23a\";\n}\n\n.fa-yc:before,\n.fa-y-combinator:before {\n  content: \"\\f23b\";\n}\n\n.fa-optin-monster:before {\n  content: \"\\f23c\";\n}\n\n.fa-opencart:before {\n  content: \"\\f23d\";\n}\n\n.fa-expeditedssl:before {\n  content: \"\\f23e\";\n}\n\n.fa-battery-4:before,\n.fa-battery:before,\n.fa-battery-full:before {\n  content: \"\\f240\";\n}\n\n.fa-battery-3:before,\n.fa-battery-three-quarters:before {\n  content: \"\\f241\";\n}\n\n.fa-battery-2:before,\n.fa-battery-half:before {\n  content: \"\\f242\";\n}\n\n.fa-battery-1:before,\n.fa-battery-quarter:before {\n  content: \"\\f243\";\n}\n\n.fa-battery-0:before,\n.fa-battery-empty:before {\n  content: \"\\f244\";\n}\n\n.fa-mouse-pointer:before {\n  content: \"\\f245\";\n}\n\n.fa-i-cursor:before {\n  content: \"\\f246\";\n}\n\n.fa-object-group:before {\n  content: \"\\f247\";\n}\n\n.fa-object-ungroup:before {\n  content: \"\\f248\";\n}\n\n.fa-sticky-note:before {\n  content: \"\\f249\";\n}\n\n.fa-sticky-note-o:before {\n  content: \"\\f24a\";\n}\n\n.fa-cc-jcb:before {\n  content: \"\\f24b\";\n}\n\n.fa-cc-diners-club:before {\n  content: \"\\f24c\";\n}\n\n.fa-clone:before {\n  content: \"\\f24d\";\n}\n\n.fa-balance-scale:before {\n  content: \"\\f24e\";\n}\n\n.fa-hourglass-o:before {\n  content: \"\\f250\";\n}\n\n.fa-hourglass-1:before,\n.fa-hourglass-start:before {\n  content: \"\\f251\";\n}\n\n.fa-hourglass-2:before,\n.fa-hourglass-half:before {\n  content: \"\\f252\";\n}\n\n.fa-hourglass-3:before,\n.fa-hourglass-end:before {\n  content: \"\\f253\";\n}\n\n.fa-hourglass:before {\n  content: \"\\f254\";\n}\n\n.fa-hand-grab-o:before,\n.fa-hand-rock-o:before {\n  content: \"\\f255\";\n}\n\n.fa-hand-stop-o:before,\n.fa-hand-paper-o:before {\n  content: \"\\f256\";\n}\n\n.fa-hand-scissors-o:before {\n  content: \"\\f257\";\n}\n\n.fa-hand-lizard-o:before {\n  content: \"\\f258\";\n}\n\n.fa-hand-spock-o:before {\n  content: \"\\f259\";\n}\n\n.fa-hand-pointer-o:before {\n  content: \"\\f25a\";\n}\n\n.fa-hand-peace-o:before {\n  content: \"\\f25b\";\n}\n\n.fa-trademark:before {\n  content: \"\\f25c\";\n}\n\n.fa-registered:before {\n  content: \"\\f25d\";\n}\n\n.fa-creative-commons:before {\n  content: \"\\f25e\";\n}\n\n.fa-gg:before {\n  content: \"\\f260\";\n}\n\n.fa-gg-circle:before {\n  content: \"\\f261\";\n}\n\n.fa-tripadvisor:before {\n  content: \"\\f262\";\n}\n\n.fa-odnoklassniki:before {\n  content: \"\\f263\";\n}\n\n.fa-odnoklassniki-square:before {\n  content: \"\\f264\";\n}\n\n.fa-get-pocket:before {\n  content: \"\\f265\";\n}\n\n.fa-wikipedia-w:before {\n  content: \"\\f266\";\n}\n\n.fa-safari:before {\n  content: \"\\f267\";\n}\n\n.fa-chrome:before {\n  content: \"\\f268\";\n}\n\n.fa-firefox:before {\n  content: \"\\f269\";\n}\n\n.fa-opera:before {\n  content: \"\\f26a\";\n}\n\n.fa-internet-explorer:before {\n  content: \"\\f26b\";\n}\n\n.fa-tv:before,\n.fa-television:before {\n  content: \"\\f26c\";\n}\n\n.fa-contao:before {\n  content: \"\\f26d\";\n}\n\n.fa-500px:before {\n  content: \"\\f26e\";\n}\n\n.fa-amazon:before {\n  content: \"\\f270\";\n}\n\n.fa-calendar-plus-o:before {\n  content: \"\\f271\";\n}\n\n.fa-calendar-minus-o:before {\n  content: \"\\f272\";\n}\n\n.fa-calendar-times-o:before {\n  content: \"\\f273\";\n}\n\n.fa-calendar-check-o:before {\n  content: \"\\f274\";\n}\n\n.fa-industry:before {\n  content: \"\\f275\";\n}\n\n.fa-map-pin:before {\n  content: \"\\f276\";\n}\n\n.fa-map-signs:before {\n  content: \"\\f277\";\n}\n\n.fa-map-o:before {\n  content: \"\\f278\";\n}\n\n.fa-map:before {\n  content: \"\\f279\";\n}\n\n.fa-commenting:before {\n  content: \"\\f27a\";\n}\n\n.fa-commenting-o:before {\n  content: \"\\f27b\";\n}\n\n.fa-houzz:before {\n  content: \"\\f27c\";\n}\n\n.fa-vimeo:before {\n  content: \"\\f27d\";\n}\n\n.fa-black-tie:before {\n  content: \"\\f27e\";\n}\n\n.fa-fonticons:before {\n  content: \"\\f280\";\n}\n\n.fa-reddit-alien:before {\n  content: \"\\f281\";\n}\n\n.fa-edge:before {\n  content: \"\\f282\";\n}\n\n.fa-credit-card-alt:before {\n  content: \"\\f283\";\n}\n\n.fa-codiepie:before {\n  content: \"\\f284\";\n}\n\n.fa-modx:before {\n  content: \"\\f285\";\n}\n\n.fa-fort-awesome:before {\n  content: \"\\f286\";\n}\n\n.fa-usb:before {\n  content: \"\\f287\";\n}\n\n.fa-product-hunt:before {\n  content: \"\\f288\";\n}\n\n.fa-mixcloud:before {\n  content: \"\\f289\";\n}\n\n.fa-scribd:before {\n  content: \"\\f28a\";\n}\n\n.fa-pause-circle:before {\n  content: \"\\f28b\";\n}\n\n.fa-pause-circle-o:before {\n  content: \"\\f28c\";\n}\n\n.fa-stop-circle:before {\n  content: \"\\f28d\";\n}\n\n.fa-stop-circle-o:before {\n  content: \"\\f28e\";\n}\n\n.fa-shopping-bag:before {\n  content: \"\\f290\";\n}\n\n.fa-shopping-basket:before {\n  content: \"\\f291\";\n}\n\n.fa-hashtag:before {\n  content: \"\\f292\";\n}\n\n.fa-bluetooth:before {\n  content: \"\\f293\";\n}\n\n.fa-bluetooth-b:before {\n  content: \"\\f294\";\n}\n\n.fa-percent:before {\n  content: \"\\f295\";\n}\n\n.fa-gitlab:before {\n  content: \"\\f296\";\n}\n\n.fa-wpbeginner:before {\n  content: \"\\f297\";\n}\n\n.fa-wpforms:before {\n  content: \"\\f298\";\n}\n\n.fa-envira:before {\n  content: \"\\f299\";\n}\n\n.fa-universal-access:before {\n  content: \"\\f29a\";\n}\n\n.fa-wheelchair-alt:before {\n  content: \"\\f29b\";\n}\n\n.fa-question-circle-o:before {\n  content: \"\\f29c\";\n}\n\n.fa-blind:before {\n  content: \"\\f29d\";\n}\n\n.fa-audio-description:before {\n  content: \"\\f29e\";\n}\n\n.fa-volume-control-phone:before {\n  content: \"\\f2a0\";\n}\n\n.fa-braille:before {\n  content: \"\\f2a1\";\n}\n\n.fa-assistive-listening-systems:before {\n  content: \"\\f2a2\";\n}\n\n.fa-asl-interpreting:before,\n.fa-american-sign-language-interpreting:before {\n  content: \"\\f2a3\";\n}\n\n.fa-deafness:before,\n.fa-hard-of-hearing:before,\n.fa-deaf:before {\n  content: \"\\f2a4\";\n}\n\n.fa-glide:before {\n  content: \"\\f2a5\";\n}\n\n.fa-glide-g:before {\n  content: \"\\f2a6\";\n}\n\n.fa-signing:before,\n.fa-sign-language:before {\n  content: \"\\f2a7\";\n}\n\n.fa-low-vision:before {\n  content: \"\\f2a8\";\n}\n\n.fa-viadeo:before {\n  content: \"\\f2a9\";\n}\n\n.fa-viadeo-square:before {\n  content: \"\\f2aa\";\n}\n\n.fa-snapchat:before {\n  content: \"\\f2ab\";\n}\n\n.fa-snapchat-ghost:before {\n  content: \"\\f2ac\";\n}\n\n.fa-snapchat-square:before {\n  content: \"\\f2ad\";\n}\n\n.fa-pied-piper:before {\n  content: \"\\f2ae\";\n}\n\n.fa-first-order:before {\n  content: \"\\f2b0\";\n}\n\n.fa-yoast:before {\n  content: \"\\f2b1\";\n}\n\n.fa-themeisle:before {\n  content: \"\\f2b2\";\n}\n\n.fa-google-plus-circle:before,\n.fa-google-plus-official:before {\n  content: \"\\f2b3\";\n}\n\n.fa-fa:before,\n.fa-font-awesome:before {\n  content: \"\\f2b4\";\n}\n\n.fa-handshake-o:before {\n  content: \"\\f2b5\";\n}\n\n.fa-envelope-open:before {\n  content: \"\\f2b6\";\n}\n\n.fa-envelope-open-o:before {\n  content: \"\\f2b7\";\n}\n\n.fa-linode:before {\n  content: \"\\f2b8\";\n}\n\n.fa-address-book:before {\n  content: \"\\f2b9\";\n}\n\n.fa-address-book-o:before {\n  content: \"\\f2ba\";\n}\n\n.fa-vcard:before,\n.fa-address-card:before {\n  content: \"\\f2bb\";\n}\n\n.fa-vcard-o:before,\n.fa-address-card-o:before {\n  content: \"\\f2bc\";\n}\n\n.fa-user-circle:before {\n  content: \"\\f2bd\";\n}\n\n.fa-user-circle-o:before {\n  content: \"\\f2be\";\n}\n\n.fa-user-o:before {\n  content: \"\\f2c0\";\n}\n\n.fa-id-badge:before {\n  content: \"\\f2c1\";\n}\n\n.fa-drivers-license:before,\n.fa-id-card:before {\n  content: \"\\f2c2\";\n}\n\n.fa-drivers-license-o:before,\n.fa-id-card-o:before {\n  content: \"\\f2c3\";\n}\n\n.fa-quora:before {\n  content: \"\\f2c4\";\n}\n\n.fa-free-code-camp:before {\n  content: \"\\f2c5\";\n}\n\n.fa-telegram:before {\n  content: \"\\f2c6\";\n}\n\n.fa-thermometer-4:before,\n.fa-thermometer:before,\n.fa-thermometer-full:before {\n  content: \"\\f2c7\";\n}\n\n.fa-thermometer-3:before,\n.fa-thermometer-three-quarters:before {\n  content: \"\\f2c8\";\n}\n\n.fa-thermometer-2:before,\n.fa-thermometer-half:before {\n  content: \"\\f2c9\";\n}\n\n.fa-thermometer-1:before,\n.fa-thermometer-quarter:before {\n  content: \"\\f2ca\";\n}\n\n.fa-thermometer-0:before,\n.fa-thermometer-empty:before {\n  content: \"\\f2cb\";\n}\n\n.fa-shower:before {\n  content: \"\\f2cc\";\n}\n\n.fa-bathtub:before,\n.fa-s15:before,\n.fa-bath:before {\n  content: \"\\f2cd\";\n}\n\n.fa-podcast:before {\n  content: \"\\f2ce\";\n}\n\n.fa-window-maximize:before {\n  content: \"\\f2d0\";\n}\n\n.fa-window-minimize:before {\n  content: \"\\f2d1\";\n}\n\n.fa-window-restore:before {\n  content: \"\\f2d2\";\n}\n\n.fa-times-rectangle:before,\n.fa-window-close:before {\n  content: \"\\f2d3\";\n}\n\n.fa-times-rectangle-o:before,\n.fa-window-close-o:before {\n  content: \"\\f2d4\";\n}\n\n.fa-bandcamp:before {\n  content: \"\\f2d5\";\n}\n\n.fa-grav:before {\n  content: \"\\f2d6\";\n}\n\n.fa-etsy:before {\n  content: \"\\f2d7\";\n}\n\n.fa-imdb:before {\n  content: \"\\f2d8\";\n}\n\n.fa-ravelry:before {\n  content: \"\\f2d9\";\n}\n\n.fa-eercast:before {\n  content: \"\\f2da\";\n}\n\n.fa-microchip:before {\n  content: \"\\f2db\";\n}\n\n.fa-snowflake-o:before {\n  content: \"\\f2dc\";\n}\n\n.fa-superpowers:before {\n  content: \"\\f2dd\";\n}\n\n.fa-wpexplorer:before {\n  content: \"\\f2de\";\n}\n\n.fa-meetup:before {\n  content: \"\\f2e0\";\n}\n\n.sr-only {\n  position: absolute;\n  width: 1px;\n  height: 1px;\n  padding: 0;\n  margin: -1px;\n  overflow: hidden;\n  clip: rect(0, 0, 0, 0);\n  border: 0;\n}\n\n.sr-only-focusable:active,\n.sr-only-focusable:focus {\n  position: static;\n  width: auto;\n  height: auto;\n  margin: 0;\n  overflow: visible;\n  clip: auto;\n}\n\n#main > .log > table, #main > .page.modbus > .data > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table, #main > .page.monitor > .components table, #main > .page.orchestrator > .components table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th, #main > .page.monitor > .components table th, #main > .page.orchestrator > .components table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td, #main > .page.monitor > .components table td, #main > .page.orchestrator > .components table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n#main > .page.orchestrator {\n  display: flex;\n  flex-direction: column;\n}\n#main > .page.orchestrator > .header {\n  display: flex;\n  align-items: center;\n  justify-content: flex-end;\n  padding: 0.2rem;\n}\n#main > .page.orchestrator > .header > *:not(:last-child) {\n  margin-right: 0.2rem;\n}\n#main > .page.orchestrator > .components {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .page.orchestrator > .components table th.col-delay {\n  width: 5rem;\n}\n#main > .page.orchestrator > .components table th.col-revive {\n  width: 5rem;\n}\n#main > .page.orchestrator > .components table th.col-status {\n  width: 8rem;\n}\n#main > .page.orchestrator > .components table th.col-action {\n  width: 5rem;\n}\n#main > .page.orchestrator > .components table td.col-delay {\n  text-align: right;\n}\n#main > .page.orchestrator > .components table td.col-revive {\n  text-align: center;\n}\n#main > .page.orchestrator > .components table td.col-status {\n  text-align: center;\n}\n#main > .page.orchestrator > .components table td.col-action {\n  text-align: center;\n}\n#main > .page.orchestrator > .components table td.col-action button {\n  margin: 0 0.1rem;\n}\n\n#main > .log > table, #main > .page.modbus > .data > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table, #main > .page.monitor > .components table, #main > .page.orchestrator > .components table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th, #main > .page.monitor > .components table th, #main > .page.orchestrator > .components table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td, #main > .page.monitor > .components table td, #main > .page.orchestrator > .components table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n#main > .page.monitor {\n  display: flex;\n  flex-direction: column;\n}\n#main > .page.monitor > .header {\n  display: flex;\n  align-items: center;\n  justify-content: flex-end;\n  padding: 0.2rem;\n}\n#main > .page.monitor > .header > *:not(:last-child) {\n  margin-right: 0.2rem;\n}\n#main > .page.monitor > .components {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .page.monitor > .components table th.hidden {\n  padding: 0;\n}\n#main > .page.monitor > .components table th.col-id {\n  width: 5rem;\n}\n#main > .page.monitor > .components table th.col-group {\n  width: 10rem;\n}\n#main > .page.monitor > .components table th.col-address {\n  width: 15rem;\n}\n#main > .page.monitor > .components table th.col-blessing {\n  width: 5rem;\n}\n#main > .page.monitor > .components table th.col-ready {\n  width: 5rem;\n}\n#main > .page.monitor > .components table th.col-rank {\n  width: 8rem;\n}\n#main > .page.monitor > .components table td.col-id {\n  text-align: center;\n}\n#main > .page.monitor > .components table td.col-blessing {\n  text-align: center;\n}\n#main > .page.monitor > .components table td.col-ready {\n  text-align: center;\n}\n#main > .page.monitor > .components table td.col-rank {\n  text-align: center;\n}\n#main > .page.monitor > .components table td.col-rank > div {\n  display: flex;\n  justify-content: space-between;\n  align-items: center;\n}\n#main > .page.monitor > .components table td.col-rank > div button {\n  margin: 0 0.1rem;\n}\n\n#main > .log > table, #main > .page.modbus > .data > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table, #main > .page.orchestrator > .components table, #main > .page.monitor > .components table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th, #main > .page.orchestrator > .components table th, #main > .page.monitor > .components table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td, #main > .page.orchestrator > .components table td, #main > .page.monitor > .components table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n#main > .page.event {\n  display: flex;\n  flex-direction: column;\n}\n#main > .page.event > .header {\n  display: flex;\n  align-items: center;\n  padding: 0.2rem;\n  border-bottom: 1px solid #616161;\n}\n#main > .page.event > .header > .menu {\n  display: flex;\n}\n#main > .page.event > .header > .menu > * {\n  margin-right: 0.2rem;\n  padding: 0.2rem 0.5rem;\n  border-radius: 0.2rem;\n}\n#main > .page.event > .header > .menu > *.selected, #main > .page.event > .header > .menu > *:hover {\n  color: whitesmoke;\n  background-color: #616161;\n}\n#main > .page.event > .header > .spacer {\n  flex-grow: 1;\n}\n#main > .page.event > .header > .form {\n  display: flex;\n  align-items: center;\n}\n#main > .page.event > .header > .form > *:not(:last-child) {\n  margin-right: 0.2rem;\n}\n#main > .page.event > .subpage {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .page.event > .subpage.latest > table th.col-type, #main > .page.event > .subpage.changes > table th.col-type {\n  width: 25rem;\n}\n#main > .page.event > .subpage.latest > table th.col-id, #main > .page.event > .subpage.changes > table th.col-id {\n  width: 5rem;\n}\n#main > .page.event > .subpage.latest > table th.col-timestamp, #main > .page.event > .subpage.changes > table th.col-timestamp {\n  width: 13rem;\n}\n#main > .page.event > .subpage.latest > table td.col-id, #main > .page.event > .subpage.changes > table td.col-id {\n  text-align: right;\n}\n#main > .page.event > .subpage.latest > table td.col-timestamp, #main > .page.event > .subpage.changes > table td.col-timestamp {\n  text-align: center;\n}\n#main > .page.event > .subpage.latest > table .expand-icon, #main > .page.event > .subpage.changes > table .expand-icon {\n  padding: 5px;\n  color: #616161;\n}\n#main > .page.event > .subpage.register {\n  padding: 0.5rem;\n  display: grid;\n  grid-gap: 0.5rem;\n  grid-template-columns: 1fr auto;\n  grid-template-rows: auto auto 1fr;\n  grid-template-areas: \"text sourceTimestamp\" \"text register\" \"text .\";\n}\n#main > .page.event > .subpage.register > .source-timestamp {\n  grid-area: sourceTimestamp;\n}\n#main > .page.event > .subpage.register > .text {\n  grid-area: text;\n  resize: none;\n}\n#main > .page.event > .subpage.register > .register {\n  grid-area: register;\n}\n\n#main > .log > table, #main > .page.modbus > .data > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table, #main > .page.orchestrator > .components table, #main > .page.monitor > .components table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th, #main > .page.orchestrator > .components table th, #main > .page.monitor > .components table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td, #main > .page.orchestrator > .components table td, #main > .page.monitor > .components table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n#main > .page.iec104 {\n  display: flex;\n  flex-direction: column;\n}\n#main > .page.iec104 > .properties {\n  padding: 0.2rem;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 10rem 8rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n}\n#main > .page.iec104 > .properties > .label {\n  justify-self: end;\n}\n#main > .page.iec104 > .data, #main > .page.iec104 > .commands {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .page.iec104 > .data > table th.hidden, #main > .page.iec104 > .commands > table th.hidden {\n  padding: 0;\n}\n#main > .page.iec104 > .data > table th.col-timestamp, #main > .page.iec104 > .commands > table th.col-timestamp {\n  width: 13rem;\n}\n#main > .page.iec104 > .data > table th.col-str, #main > .page.iec104 > .commands > table th.col-str {\n  width: 8rem;\n}\n#main > .page.iec104 > .data > table th.col-int, #main > .page.iec104 > .commands > table th.col-int {\n  width: 4rem;\n}\n#main > .page.iec104 > .data > table th.col-short, #main > .page.iec104 > .commands > table th.col-short {\n  width: 3rem;\n}\n#main > .page.iec104 > .data > table th.col-long, #main > .page.iec104 > .commands > table th.col-long {\n  width: 5rem;\n}\n#main > .page.iec104 > .data > table th.col-bool, #main > .page.iec104 > .commands > table th.col-bool {\n  width: 3rem;\n}\n#main > .page.iec104 > .data > table th.col-remove, #main > .page.iec104 > .commands > table th.col-remove {\n  width: 4.5rem;\n}\n#main > .page.iec104 > .data > table td.col-int, #main > .page.iec104 > .data > table td.col-short, #main > .page.iec104 > .data > table td.col-long, #main > .page.iec104 > .commands > table td.col-int, #main > .page.iec104 > .commands > table td.col-short, #main > .page.iec104 > .commands > table td.col-long {\n  text-align: right;\n}\n#main > .page.iec104 > .data > table td.col-bool, #main > .page.iec104 > .commands > table td.col-bool {\n  text-align: center;\n}\n#main > .page.iec104 > .data > table td.col-remove, #main > .page.iec104 > .commands > table td.col-remove {\n  text-align: center;\n}\n#main > .page.iec104 > .data > table td.col-timestamp, #main > .page.iec104 > .commands > table td.col-timestamp {\n  font-family: \"RobotoMono\";\n  font-size: 0.9rem;\n}\n#main > .page.iec104 > .data > table > tbody > tr.selected, #main > .page.iec104 > .data > table > tbody > tr:hover, #main > .page.iec104 > .commands > table > tbody > tr.selected, #main > .page.iec104 > .commands > table > tbody > tr:hover {\n  background-color: #bdbdbd;\n}\n#main > .page.iec104 > .data > .control, #main > .page.iec104 > .commands > .control {\n  padding: 0.2rem;\n  display: flex;\n  justify-content: flex-end;\n}\n#main > .page.iec104.master > .control {\n  margin: 0.2rem 0;\n}\n#main > .page.iec104.master > .control > .header {\n  display: flex;\n  padding: 0.2rem;\n  border-bottom: 1px solid #616161;\n}\n#main > .page.iec104.master > .control > .header > * {\n  margin-right: 0.2rem;\n  padding: 0.2rem 0.5rem;\n  border-radius: 0.2rem;\n}\n#main > .page.iec104.master > .control > .header > *.selected, #main > .page.iec104.master > .control > .header > *:hover {\n  color: whitesmoke;\n  background-color: #616161;\n}\n#main > .page.iec104.master > .control > .content {\n  margin: 0.2rem;\n  display: flex;\n  align-items: stretch;\n}\n#main > .page.iec104.master > .control > .content > .form {\n  flex-grow: 1;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 8rem 12rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n}\n#main > .page.iec104.master > .control > .content > .form > label {\n  justify-self: end;\n}\n#main > .page.iec104.master > .control > .content > button {\n  margin-left: 0.2rem;\n}\n#main > .page.iec104.slave > .panel {\n  display: flex;\n  align-items: stretch;\n  padding: 0.2rem;\n  border-top: 1px solid #616161;\n  margin-top: 0.2rem;\n}\n#main > .page.iec104.slave > .panel > .form {\n  flex-grow: 1;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 12rem 12rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n}\n#main > .page.iec104.slave > .panel > .form > .label {\n  justify-self: end;\n}\n#main > .page.iec104.slave > .panel > button {\n  margin-left: 0.2rem;\n}\n\n#main > .log > table, #main > .page.modbus > .data > table, #main > .page.orchestrator > .components table, #main > .page.monitor > .components table, #main > .page.event > .subpage.latest > table, #main > .page.event > .subpage.changes > table, #main > .page.iec104 > .data > table, #main > .page.iec104 > .commands > table {\n  width: 100%;\n  border-spacing: 0;\n  table-layout: fixed;\n  position: relative;\n}\n#main > .log > table th, #main > .page.modbus > .data > table th, #main > .page.orchestrator > .components table th, #main > .page.monitor > .components table th, #main > .page.event > .subpage.latest > table th, #main > .page.event > .subpage.changes > table th, #main > .page.iec104 > .data > table th, #main > .page.iec104 > .commands > table th {\n  padding: 0.2rem;\n  background: #616161;\n  color: whitesmoke;\n  position: sticky;\n  top: 0;\n}\n#main > .log > table tr:nth-child(even), #main > .page.modbus > .data > table tr:nth-child(even), #main > .page.orchestrator > .components table tr:nth-child(even), #main > .page.monitor > .components table tr:nth-child(even), #main > .page.event > .subpage.latest > table tr:nth-child(even), #main > .page.event > .subpage.changes > table tr:nth-child(even), #main > .page.iec104 > .data > table tr:nth-child(even), #main > .page.iec104 > .commands > table tr:nth-child(even) {\n  background-color: #e0e0e0;\n}\n#main > .log > table tr:nth-child(odd), #main > .page.modbus > .data > table tr:nth-child(odd), #main > .page.orchestrator > .components table tr:nth-child(odd), #main > .page.monitor > .components table tr:nth-child(odd), #main > .page.event > .subpage.latest > table tr:nth-child(odd), #main > .page.event > .subpage.changes > table tr:nth-child(odd), #main > .page.iec104 > .data > table tr:nth-child(odd), #main > .page.iec104 > .commands > table tr:nth-child(odd) {\n  background-color: whitesmoke;\n}\n#main > .log > table td, #main > .page.modbus > .data > table td, #main > .page.orchestrator > .components table td, #main > .page.monitor > .components table td, #main > .page.event > .subpage.latest > table td, #main > .page.event > .subpage.changes > table td, #main > .page.iec104 > .data > table td, #main > .page.iec104 > .commands > table td {\n  padding: 0.2rem;\n  overflow: hidden;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n#main > .page.modbus {\n  display: flex;\n  flex-direction: column;\n}\n#main > .page.modbus > .properties {\n  padding: 0.2rem;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 10rem 8rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n}\n#main > .page.modbus > .properties > .label {\n  justify-self: end;\n}\n#main > .page.modbus > .action {\n  border-top: 1px solid #616161;\n  padding: 0.2rem;\n  display: flex;\n  align-items: stretch;\n}\n#main > .page.modbus > .action > .form {\n  flex-grow: 1;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 8rem 12rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n}\n#main > .page.modbus > .action > .form > label {\n  justify-self: end;\n}\n#main > .page.modbus > .action > button {\n  margin-left: 0.2rem;\n}\n#main > .page.modbus > .data {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .page.modbus > .data > table th.col-timestamp {\n  width: 13rem;\n}\n#main > .page.modbus > .data > table th.col-str {\n  width: 10rem;\n}\n#main > .page.modbus > .data > table th.col-int {\n  width: 5rem;\n}\n#main > .page.modbus > .data > table th.col-remove {\n  width: 4.5rem;\n}\n#main > .page.modbus > .data > table td.col-remove {\n  text-align: center;\n}\n#main > .page.modbus > .data > table td.col-timestamp {\n  font-family: \"RobotoMono\";\n  font-size: 0.9rem;\n}\n#main > .page.modbus > .data > table > tbody > tr.selected, #main > .page.modbus > .data > table > tbody > tr:hover {\n  background-color: #bdbdbd;\n}\n#main > .page.modbus > .data > .control {\n  padding: 0.2rem;\n  display: flex;\n  justify-content: flex-end;\n}\n#main > .page.modbus > .panel {\n  padding: 0.2rem;\n  display: grid;\n  grid-template-columns: repeat(auto-fit, 12rem 12rem);\n  grid-gap: 0.2rem;\n  align-items: center;\n  border-top: 1px solid #616161;\n  margin-top: 0.2rem;\n}\n#main > .page.modbus > .panel > .label {\n  justify-self: end;\n}\n\nhtml, body, button, input, select {\n  font-family: \"Inter\";\n  font-size: 12pt;\n}\n\nhtml, body {\n  margin: 0;\n  user-select: none;\n  height: 100%;\n  overflow: hidden;\n}\n\n#main {\n  height: 100%;\n  background-color: #eeeeee;\n  display: grid;\n  grid-template-columns: max-content 0.4rem 1fr;\n  grid-template-rows: auto 1fr 0.4rem auto;\n  grid-template-areas: \"globalHeader deviceHeader deviceHeader\" \"sidebar sidebarResizer page\" \"logResizer logResizer logResizer\" \"log log log\";\n}\n#main > .header {\n  padding: 0.2rem;\n  background-color: #616161;\n  color: whitesmoke;\n}\n#main > .header > button:not(:last-child) {\n  margin-right: 0.2rem;\n}\n#main > .header.global {\n  grid-area: globalHeader;\n}\n#main > .header.device {\n  grid-area: deviceHeader;\n}\n#main > .header.device > *:not(button) {\n  margin-right: 1rem;\n}\n#main > .sidebar {\n  grid-area: sidebar;\n  background-color: whitesmoke;\n  display: flex;\n  flex-direction: column;\n  overflow: hidden;\n  min-width: 10rem;\n}\n#main > .sidebar > .devices {\n  flex-grow: 1;\n  overflow: auto;\n}\n#main > .sidebar > .devices > .device {\n  padding: 0.5rem;\n  overflow: hidden;\n  white-space: nowrap;\n  text-overflow: ellipsis;\n}\n#main > .sidebar > .devices > .device:hover, #main > .sidebar > .devices > .device.selected {\n  background-color: #e0e0e0;\n}\n#main > .sidebar > .devices > .device > .status {\n  margin-right: 0.5rem;\n}\n#main > .sidebar > .devices > .device > .status.stopped {\n  color: red;\n}\n#main > .sidebar > .devices > .device > .status.starting, #main > .sidebar > .devices > .device > .status.stopping {\n  color: yellow;\n}\n#main > .sidebar > .devices > .device > .status.started {\n  color: green;\n}\n#main > .sidebar > .devices > .device > .type {\n  color: #757575;\n}\n#main > .sidebar > .add {\n  margin: 0.5rem;\n}\n#main > .sidebar-resizer {\n  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.2), 0 1px 5px 0 rgba(0, 0, 0, 0.12);\n  grid-area: sidebarResizer;\n  cursor: ew-resize;\n  background-color: #bdbdbd;\n}\n#main > .page {\n  grid-area: page;\n  overflow: auto;\n}\n#main > .log-resizer {\n  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 3px 1px -2px rgba(0, 0, 0, 0.2), 0 1px 5px 0 rgba(0, 0, 0, 0.12);\n  grid-area: logResizer;\n  cursor: ns-resize;\n  background-color: #bdbdbd;\n  z-index: 1;\n}\n#main > .log {\n  box-shadow: 0 4px 5px 0 rgba(0, 0, 0, 0.14), 0 1px 10px 0 rgba(0, 0, 0, 0.12), 0 2px 4px -1px rgba(0, 0, 0, 0.2);\n  grid-area: log;\n  border-top: 1px solid #bdbdbd;\n  background-color: whitesmoke;\n  overflow: auto;\n  height: 10rem;\n}\n#main > .log > table .col-time {\n  width: 15rem;\n  text-align: center;\n}\n#main > .log > table .col-message {\n  text-align: left;\n}\n#main > .log > table td.col-time {\n  font-family: \"RobotoMono\";\n  font-size: 0.9rem;\n}\n#main > .overlay {\n  position: absolute;\n  width: 100%;\n  height: 100%;\n  background-color: rgba(0, 0, 0, 0.2);\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  z-index: 10;\n}\n#main > .overlay > .dialog {\n  box-shadow: 0 4px 5px 0 rgba(0, 0, 0, 0.14), 0 1px 10px 0 rgba(0, 0, 0, 0.12), 0 2px 4px -1px rgba(0, 0, 0, 0.2);\n  border: 1px solid #9e9e9e;\n  border-radius: 0.2rem;\n  background-color: whitesmoke;\n  padding: 0.5rem;\n}\n#main > .overlay > .dialog.add {\n  display: flex;\n  flex-direction: column;\n}\n#main > .overlay > .dialog.add > .device {\n  padding: 0.5rem;\n  cursor: pointer;\n  display: flex;\n  align-items: center;\n}\n#main > .overlay > .dialog.add > .device .icon {\n  width: 32px;\n  margin-right: 0.5rem;\n}\n#main > .overlay > .dialog.add > .device:hover {\n  background-color: #e0e0e0;\n}\n#main > .overlay > .dialog.settings {\n  display: grid;\n  grid-template-columns: auto auto;\n  grid-gap: 0.5rem;\n  align-items: center;\n}\n#main > .overlay > .dialog.settings > .title {\n  grid-column: span 2;\n  background: #616161;\n  color: whitesmoke;\n  padding: 0.2rem;\n  font-weight: 600;\n}\n#main > .overlay > .dialog.settings > .label {\n  justify-self: end;\n}\n#main > .overlay > .dialog.settings > .note {\n  grid-column: span 2;\n  color: #616161;\n  justify-self: end;\n}", "",{"version":3,"sources":["webpack://./src_scss/_table.scss","webpack://./src_scss/main.scss","webpack://./src_scss/_color.scss","webpack://./src_scss/fonts/inter.scss","webpack://./src_scss/fonts/robotomono.scss","webpack://./src_scss/fonts/fontawesome.scss","webpack://./src_scss/orchestrator.scss","webpack://./src_scss/monitor.scss","webpack://./src_scss/event.scss","webpack://./src_scss/iec104.scss","webpack://./src_scss/modbus.scss","webpack://./src_scss/_shadow.scss"],"names":[],"mappings":"AAGA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;ACFJ;ADII;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;ACFR;ADMQ;EACI,yBEfK;ADWjB;ADOQ;EACI,4BErBK;ADgBjB;ADSI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;ACPR;;AEvBA;EACI,oBAAA;EACA,mDAAA;EACA,oBAAA;EACA,8BAAA;EACA,kBAAA;AF0BJ;AG/BA;EACI,yBAAA;EACA,6DAAA;EACA,oBAAA;EACA,kBAAA;EACA,kBAAA;AHiCJ;AG9BA;EACI,yBAAA;EACA,oEAAA;EACA,oBAAA;EACA,kBAAA;EACA,kBAAA;AHgCJ;AI7CA;EACI,0BAAA;EACA,+DAAA;EACA,mBAAA;EACA,kBAAA;AJ+CJ;AI3CA;EACE,qBAAA;EACA,6CAAA;EACA,kBAAA;EACA,oBAAA;EACA,mCAAA;EACA,kCAAA;AJ6CF;;AI3CA,6DAAA;AACA;EACE,uBAAA;EACA,mBAAA;EACA,oBAAA;AJ8CF;;AI5CA;EACE,cAAA;AJ+CF;;AI7CA;EACE,cAAA;AJgDF;;AI9CA;EACE,cAAA;AJiDF;;AI/CA;EACE,cAAA;AJkDF;;AIhDA;EACE,mBAAA;EACA,kBAAA;AJmDF;;AIjDA;EACE,eAAA;EACA,yBAAA;EACA,qBAAA;AJoDF;;AIlDA;EACE,kBAAA;AJqDF;;AInDA;EACE,kBAAA;EACA,mBAAA;EACA,mBAAA;EACA,iBAAA;EACA,kBAAA;AJsDF;;AIpDA;EACE,mBAAA;AJuDF;;AIrDA;EACE,4BAAA;EACA,4BAAA;EACA,oBAAA;AJwDF;;AItDA;EACE,WAAA;AJyDF;;AIvDA;EACE,YAAA;AJ0DF;;AIxDA;EACE,mBAAA;AJ2DF;;AIzDA;EACE,kBAAA;AJ4DF;;AI1DA,2BAAA;AACA;EACE,YAAA;AJ6DF;;AI3DA;EACE,WAAA;AJ8DF;;AI5DA;EACE,mBAAA;AJ+DF;;AI7DA;EACE,kBAAA;AJgEF;;AI9DA;EACE,6CAAA;EACA,qCAAA;AJiEF;;AI/DA;EACE,+CAAA;EACA,uCAAA;AJkEF;;AIhEA;EACE;IACE,+BAAA;IACA,uBAAA;EJmEF;EIjEA;IACE,iCAAA;IACA,yBAAA;EJmEF;AACF;AIjEA;EACE;IACE,+BAAA;IACA,uBAAA;EJmEF;EIjEA;IACE,iCAAA;IACA,yBAAA;EJmEF;AACF;AIjEA;EACE,sEAAA;EACA,gCAAA;EACA,4BAAA;EACA,wBAAA;AJmEF;;AIjEA;EACE,sEAAA;EACA,iCAAA;EACA,6BAAA;EACA,yBAAA;AJoEF;;AIlEA;EACE,sEAAA;EACA,iCAAA;EACA,6BAAA;EACA,yBAAA;AJqEF;;AInEA;EACE,gFAAA;EACA,+BAAA;EACA,2BAAA;EACA,uBAAA;AJsEF;;AIpEA;EACE,gFAAA;EACA,+BAAA;EACA,2BAAA;EACA,uBAAA;AJuEF;;AIrEA;;;;;EAKE,YAAA;AJwEF;;AItEA;EACE,kBAAA;EACA,qBAAA;EACA,UAAA;EACA,WAAA;EACA,gBAAA;EACA,sBAAA;AJyEF;;AIvEA;;EAEE,kBAAA;EACA,OAAA;EACA,WAAA;EACA,kBAAA;AJ0EF;;AIxEA;EACE,oBAAA;AJ2EF;;AIzEA;EACE,cAAA;AJ4EF;;AI1EA;EACE,cAAA;AJ6EF;;AI3EA;mEAAA;AAEA;EACE,gBAAA;AJ8EF;;AI5EA;EACE,gBAAA;AJ+EF;;AI7EA;EACE,gBAAA;AJgFF;;AI9EA;EACE,gBAAA;AJiFF;;AI/EA;EACE,gBAAA;AJkFF;;AIhFA;EACE,gBAAA;AJmFF;;AIjFA;EACE,gBAAA;AJoFF;;AIlFA;EACE,gBAAA;AJqFF;;AInFA;EACE,gBAAA;AJsFF;;AIpFA;EACE,gBAAA;AJuFF;;AIrFA;EACE,gBAAA;AJwFF;;AItFA;EACE,gBAAA;AJyFF;;AIvFA;EACE,gBAAA;AJ0FF;;AIxFA;;;EAGE,gBAAA;AJ2FF;;AIzFA;EACE,gBAAA;AJ4FF;;AI1FA;EACE,gBAAA;AJ6FF;;AI3FA;EACE,gBAAA;AJ8FF;;AI5FA;EACE,gBAAA;AJ+FF;;AI7FA;;EAEE,gBAAA;AJgGF;;AI9FA;EACE,gBAAA;AJiGF;;AI/FA;EACE,gBAAA;AJkGF;;AIhGA;EACE,gBAAA;AJmGF;;AIjGA;EACE,gBAAA;AJoGF;;AIlGA;EACE,gBAAA;AJqGF;;AInGA;EACE,gBAAA;AJsGF;;AIpGA;EACE,gBAAA;AJuGF;;AIrGA;EACE,gBAAA;AJwGF;;AItGA;EACE,gBAAA;AJyGF;;AIvGA;EACE,gBAAA;AJ0GF;;AIxGA;;EAEE,gBAAA;AJ2GF;;AIzGA;EACE,gBAAA;AJ4GF;;AI1GA;EACE,gBAAA;AJ6GF;;AI3GA;EACE,gBAAA;AJ8GF;;AI5GA;EACE,gBAAA;AJ+GF;;AI7GA;EACE,gBAAA;AJgHF;;AI9GA;EACE,gBAAA;AJiHF;;AI/GA;EACE,gBAAA;AJkHF;;AIhHA;EACE,gBAAA;AJmHF;;AIjHA;EACE,gBAAA;AJoHF;;AIlHA;EACE,gBAAA;AJqHF;;AInHA;EACE,gBAAA;AJsHF;;AIpHA;EACE,gBAAA;AJuHF;;AIrHA;EACE,gBAAA;AJwHF;;AItHA;EACE,gBAAA;AJyHF;;AIvHA;EACE,gBAAA;AJ0HF;;AIxHA;EACE,gBAAA;AJ2HF;;AIzHA;EACE,gBAAA;AJ4HF;;AI1HA;EACE,gBAAA;AJ6HF;;AI3HA;EACE,gBAAA;AJ8HF;;AI5HA;EACE,gBAAA;AJ+HF;;AI7HA;EACE,gBAAA;AJgIF;;AI9HA;EACE,gBAAA;AJiIF;;AI/HA;EACE,gBAAA;AJkIF;;AIhIA;EACE,gBAAA;AJmIF;;AIjIA;EACE,gBAAA;AJoIF;;AIlIA;EACE,gBAAA;AJqIF;;AInIA;;EAEE,gBAAA;AJsIF;;AIpIA;EACE,gBAAA;AJuIF;;AIrIA;EACE,gBAAA;AJwIF;;AItIA;;;EAGE,gBAAA;AJyIF;;AIvIA;EACE,gBAAA;AJ0IF;;AIxIA;EACE,gBAAA;AJ2IF;;AIzIA;EACE,gBAAA;AJ4IF;;AI1IA;EACE,gBAAA;AJ6IF;;AI3IA;;EAEE,gBAAA;AJ8IF;;AI5IA;EACE,gBAAA;AJ+IF;;AI7IA;EACE,gBAAA;AJgJF;;AI9IA;EACE,gBAAA;AJiJF;;AI/IA;EACE,gBAAA;AJkJF;;AIhJA;EACE,gBAAA;AJmJF;;AIjJA;EACE,gBAAA;AJoJF;;AIlJA;EACE,gBAAA;AJqJF;;AInJA;EACE,gBAAA;AJsJF;;AIpJA;EACE,gBAAA;AJuJF;;AIrJA;EACE,gBAAA;AJwJF;;AItJA;EACE,gBAAA;AJyJF;;AIvJA;EACE,gBAAA;AJ0JF;;AIxJA;EACE,gBAAA;AJ2JF;;AIzJA;EACE,gBAAA;AJ4JF;;AI1JA;EACE,gBAAA;AJ6JF;;AI3JA;EACE,gBAAA;AJ8JF;;AI5JA;EACE,gBAAA;AJ+JF;;AI7JA;EACE,gBAAA;AJgKF;;AI9JA;EACE,gBAAA;AJiKF;;AI/JA;EACE,gBAAA;AJkKF;;AIhKA;EACE,gBAAA;AJmKF;;AIjKA;EACE,gBAAA;AJoKF;;AIlKA;EACE,gBAAA;AJqKF;;AInKA;EACE,gBAAA;AJsKF;;AIpKA;EACE,gBAAA;AJuKF;;AIrKA;EACE,gBAAA;AJwKF;;AItKA;EACE,gBAAA;AJyKF;;AIvKA;EACE,gBAAA;AJ0KF;;AIxKA;EACE,gBAAA;AJ2KF;;AIzKA;;EAEE,gBAAA;AJ4KF;;AI1KA;EACE,gBAAA;AJ6KF;;AI3KA;EACE,gBAAA;AJ8KF;;AI5KA;EACE,gBAAA;AJ+KF;;AI7KA;EACE,gBAAA;AJgLF;;AI9KA;EACE,gBAAA;AJiLF;;AI/KA;EACE,gBAAA;AJkLF;;AIhLA;EACE,gBAAA;AJmLF;;AIjLA;EACE,gBAAA;AJoLF;;AIlLA;EACE,gBAAA;AJqLF;;AInLA;EACE,gBAAA;AJsLF;;AIpLA;EACE,gBAAA;AJuLF;;AIrLA;;EAEE,gBAAA;AJwLF;;AItLA;EACE,gBAAA;AJyLF;;AIvLA;EACE,gBAAA;AJ0LF;;AIxLA;EACE,gBAAA;AJ2LF;;AIzLA;EACE,gBAAA;AJ4LF;;AI1LA;EACE,gBAAA;AJ6LF;;AI3LA;EACE,gBAAA;AJ8LF;;AI5LA;EACE,gBAAA;AJ+LF;;AI7LA;EACE,gBAAA;AJgMF;;AI9LA;EACE,gBAAA;AJiMF;;AI/LA;EACE,gBAAA;AJkMF;;AIhMA;EACE,gBAAA;AJmMF;;AIjMA;EACE,gBAAA;AJoMF;;AIlMA;EACE,gBAAA;AJqMF;;AInMA;;EAEE,gBAAA;AJsMF;;AIpMA;EACE,gBAAA;AJuMF;;AIrMA;EACE,gBAAA;AJwMF;;AItMA;EACE,gBAAA;AJyMF;;AIvMA;EACE,gBAAA;AJ0MF;;AIxMA;;EAEE,gBAAA;AJ2MF;;AIzMA;EACE,gBAAA;AJ4MF;;AI1MA;EACE,gBAAA;AJ6MF;;AI3MA;EACE,gBAAA;AJ8MF;;AI5MA;EACE,gBAAA;AJ+MF;;AI7MA;EACE,gBAAA;AJgNF;;AI9MA;EACE,gBAAA;AJiNF;;AI/MA;EACE,gBAAA;AJkNF;;AIhNA;EACE,gBAAA;AJmNF;;AIjNA;EACE,gBAAA;AJoNF;;AIlNA;EACE,gBAAA;AJqNF;;AInNA;EACE,gBAAA;AJsNF;;AIpNA;EACE,gBAAA;AJuNF;;AIrNA;EACE,gBAAA;AJwNF;;AItNA;EACE,gBAAA;AJyNF;;AIvNA;EACE,gBAAA;AJ0NF;;AIxNA;EACE,gBAAA;AJ2NF;;AIzNA;EACE,gBAAA;AJ4NF;;AI1NA;EACE,gBAAA;AJ6NF;;AI3NA;EACE,gBAAA;AJ8NF;;AI5NA;;EAEE,gBAAA;AJ+NF;;AI7NA;EACE,gBAAA;AJgOF;;AI9NA;EACE,gBAAA;AJiOF;;AI/NA;EACE,gBAAA;AJkOF;;AIhOA;;EAEE,gBAAA;AJmOF;;AIjOA;EACE,gBAAA;AJoOF;;AIlOA;EACE,gBAAA;AJqOF;;AInOA;EACE,gBAAA;AJsOF;;AIpOA;EACE,gBAAA;AJuOF;;AIrOA;EACE,gBAAA;AJwOF;;AItOA;EACE,gBAAA;AJyOF;;AIvOA;EACE,gBAAA;AJ0OF;;AIxOA;EACE,gBAAA;AJ2OF;;AIzOA;EACE,gBAAA;AJ4OF;;AI1OA;EACE,gBAAA;AJ6OF;;AI3OA;EACE,gBAAA;AJ8OF;;AI5OA;EACE,gBAAA;AJ+OF;;AI7OA;EACE,gBAAA;AJgPF;;AI9OA;EACE,gBAAA;AJiPF;;AI/OA;EACE,gBAAA;AJkPF;;AIhPA;EACE,gBAAA;AJmPF;;AIjPA;EACE,gBAAA;AJoPF;;AIlPA;EACE,gBAAA;AJqPF;;AInPA;;EAEE,gBAAA;AJsPF;;AIpPA;;EAEE,gBAAA;AJuPF;;AIrPA;EACE,gBAAA;AJwPF;;AItPA;EACE,gBAAA;AJyPF;;AIvPA;;EAEE,gBAAA;AJ0PF;;AIxPA;;EAEE,gBAAA;AJ2PF;;AIzPA;EACE,gBAAA;AJ4PF;;AI1PA;;EAEE,gBAAA;AJ6PF;;AI3PA;EACE,gBAAA;AJ8PF;;AI5PA;;;EAGE,gBAAA;AJ+PF;;AI7PA;EACE,gBAAA;AJgQF;;AI9PA;EACE,gBAAA;AJiQF;;AI/PA;EACE,gBAAA;AJkQF;;AIhQA;EACE,gBAAA;AJmQF;;AIjQA;EACE,gBAAA;AJoQF;;AIlQA;EACE,gBAAA;AJqQF;;AInQA;EACE,gBAAA;AJsQF;;AIpQA;EACE,gBAAA;AJuQF;;AIrQA;EACE,gBAAA;AJwQF;;AItQA;EACE,gBAAA;AJyQF;;AIvQA;EACE,gBAAA;AJ0QF;;AIxQA;EACE,gBAAA;AJ2QF;;AIzQA;EACE,gBAAA;AJ4QF;;AI1QA;EACE,gBAAA;AJ6QF;;AI3QA;EACE,gBAAA;AJ8QF;;AI5QA;EACE,gBAAA;AJ+QF;;AI7QA;EACE,gBAAA;AJgRF;;AI9QA;;EAEE,gBAAA;AJiRF;;AI/QA;;EAEE,gBAAA;AJkRF;;AIhRA;;EAEE,gBAAA;AJmRF;;AIjRA;EACE,gBAAA;AJoRF;;AIlRA;EACE,gBAAA;AJqRF;;AInRA;;EAEE,gBAAA;AJsRF;;AIpRA;;EAEE,gBAAA;AJuRF;;AIrRA;;EAEE,gBAAA;AJwRF;;AItRA;EACE,gBAAA;AJyRF;;AIvRA;EACE,gBAAA;AJ0RF;;AIxRA;;EAEE,gBAAA;AJ2RF;;AIzRA;EACE,gBAAA;AJ4RF;;AI1RA;EACE,gBAAA;AJ6RF;;AI3RA;;EAEE,gBAAA;AJ8RF;;AI5RA;EACE,gBAAA;AJ+RF;;AI7RA;EACE,gBAAA;AJgSF;;AI9RA;EACE,gBAAA;AJiSF;;AI/RA;EACE,gBAAA;AJkSF;;AIhSA;EACE,gBAAA;AJmSF;;AIjSA;EACE,gBAAA;AJoSF;;AIlSA;EACE,gBAAA;AJqSF;;AInSA;EACE,gBAAA;AJsSF;;AIpSA;EACE,gBAAA;AJuSF;;AIrSA;EACE,gBAAA;AJwSF;;AItSA;EACE,gBAAA;AJySF;;AIvSA;EACE,gBAAA;AJ0SF;;AIxSA;EACE,gBAAA;AJ2SF;;AIzSA;EACE,gBAAA;AJ4SF;;AI1SA;EACE,gBAAA;AJ6SF;;AI3SA;EACE,gBAAA;AJ8SF;;AI5SA;EACE,gBAAA;AJ+SF;;AI7SA;EACE,gBAAA;AJgTF;;AI9SA;EACE,gBAAA;AJiTF;;AI/SA;EACE,gBAAA;AJkTF;;AIhTA;EACE,gBAAA;AJmTF;;AIjTA;EACE,gBAAA;AJoTF;;AIlTA;EACE,gBAAA;AJqTF;;AInTA;EACE,gBAAA;AJsTF;;AIpTA;EACE,gBAAA;AJuTF;;AIrTA;EACE,gBAAA;AJwTF;;AItTA;EACE,gBAAA;AJyTF;;AIvTA;EACE,gBAAA;AJ0TF;;AIxTA;EACE,gBAAA;AJ2TF;;AIzTA;EACE,gBAAA;AJ4TF;;AI1TA;;EAEE,gBAAA;AJ6TF;;AI3TA;EACE,gBAAA;AJ8TF;;AI5TA;EACE,gBAAA;AJ+TF;;AI7TA;EACE,gBAAA;AJgUF;;AI9TA;EACE,gBAAA;AJiUF;;AI/TA;EACE,gBAAA;AJkUF;;AIhUA;;EAEE,gBAAA;AJmUF;;AIjUA;EACE,gBAAA;AJoUF;;AIlUA;EACE,gBAAA;AJqUF;;AInUA;EACE,gBAAA;AJsUF;;AIpUA;EACE,gBAAA;AJuUF;;AIrUA;EACE,gBAAA;AJwUF;;AItUA;EACE,gBAAA;AJyUF;;AIvUA;EACE,gBAAA;AJ0UF;;AIxUA;EACE,gBAAA;AJ2UF;;AIzUA;EACE,gBAAA;AJ4UF;;AI1UA;EACE,gBAAA;AJ6UF;;AI3UA;EACE,gBAAA;AJ8UF;;AI5UA;EACE,gBAAA;AJ+UF;;AI7UA;;EAEE,gBAAA;AJgVF;;AI9UA;;;EAGE,gBAAA;AJiVF;;AI/UA;EACE,gBAAA;AJkVF;;AIhVA;EACE,gBAAA;AJmVF;;AIjVA;EACE,gBAAA;AJoVF;;AIlVA;;EAEE,gBAAA;AJqVF;;AInVA;EACE,gBAAA;AJsVF;;AIpVA;EACE,gBAAA;AJuVF;;AIrVA;EACE,gBAAA;AJwVF;;AItVA;EACE,gBAAA;AJyVF;;AIvVA;EACE,gBAAA;AJ0VF;;AIxVA;EACE,gBAAA;AJ2VF;;AIzVA;EACE,gBAAA;AJ4VF;;AI1VA;EACE,gBAAA;AJ6VF;;AI3VA;EACE,gBAAA;AJ8VF;;AI5VA;EACE,gBAAA;AJ+VF;;AI7VA;EACE,gBAAA;AJgWF;;AI9VA;EACE,gBAAA;AJiWF;;AI/VA;EACE,gBAAA;AJkWF;;AIhWA;EACE,gBAAA;AJmWF;;AIjWA;EACE,gBAAA;AJoWF;;AIlWA;EACE,gBAAA;AJqWF;;AInWA;EACE,gBAAA;AJsWF;;AIpWA;EACE,gBAAA;AJuWF;;AIrWA;EACE,gBAAA;AJwWF;;AItWA;EACE,gBAAA;AJyWF;;AIvWA;EACE,gBAAA;AJ0WF;;AIxWA;EACE,gBAAA;AJ2WF;;AIzWA;EACE,gBAAA;AJ4WF;;AI1WA;EACE,gBAAA;AJ6WF;;AI3WA;EACE,gBAAA;AJ8WF;;AI5WA;EACE,gBAAA;AJ+WF;;AI7WA;EACE,gBAAA;AJgXF;;AI9WA;EACE,gBAAA;AJiXF;;AI/WA;EACE,gBAAA;AJkXF;;AIhXA;EACE,gBAAA;AJmXF;;AIjXA;EACE,gBAAA;AJoXF;;AIlXA;EACE,gBAAA;AJqXF;;AInXA;EACE,gBAAA;AJsXF;;AIpXA;EACE,gBAAA;AJuXF;;AIrXA;EACE,gBAAA;AJwXF;;AItXA;EACE,gBAAA;AJyXF;;AIvXA;EACE,gBAAA;AJ0XF;;AIxXA;;EAEE,gBAAA;AJ2XF;;AIzXA;;EAEE,gBAAA;AJ4XF;;AI1XA;;EAEE,gBAAA;AJ6XF;;AI3XA;;EAEE,gBAAA;AJ8XF;;AI5XA;EACE,gBAAA;AJ+XF;;AI7XA;;EAEE,gBAAA;AJgYF;;AI9XA;;EAEE,gBAAA;AJiYF;;AI/XA;;;;EAIE,gBAAA;AJkYF;;AIhYA;;;EAGE,gBAAA;AJmYF;;AIjYA;;EAEE,gBAAA;AJoYF;;AIlYA;;EAEE,gBAAA;AJqYF;;AInYA;EACE,gBAAA;AJsYF;;AIpYA;EACE,gBAAA;AJuYF;;AIrYA;EACE,gBAAA;AJwYF;;AItYA;EACE,gBAAA;AJyYF;;AIvYA;EACE,gBAAA;AJ0YF;;AIxYA;EACE,gBAAA;AJ2YF;;AIzYA;EACE,gBAAA;AJ4YF;;AI1YA;EACE,gBAAA;AJ6YF;;AI3YA;EACE,gBAAA;AJ8YF;;AI5YA;EACE,gBAAA;AJ+YF;;AI7YA;EACE,gBAAA;AJgZF;;AI9YA;EACE,gBAAA;AJiZF;;AI/YA;EACE,gBAAA;AJkZF;;AIhZA;EACE,gBAAA;AJmZF;;AIjZA;EACE,gBAAA;AJoZF;;AIlZA;EACE,gBAAA;AJqZF;;AInZA;EACE,gBAAA;AJsZF;;AIpZA;EACE,gBAAA;AJuZF;;AIrZA;EACE,gBAAA;AJwZF;;AItZA;EACE,gBAAA;AJyZF;;AIvZA;EACE,gBAAA;AJ0ZF;;AIxZA;EACE,gBAAA;AJ2ZF;;AIzZA;EACE,gBAAA;AJ4ZF;;AI1ZA;EACE,gBAAA;AJ6ZF;;AI3ZA;EACE,gBAAA;AJ8ZF;;AI5ZA;EACE,gBAAA;AJ+ZF;;AI7ZA;EACE,gBAAA;AJgaF;;AI9ZA;EACE,gBAAA;AJiaF;;AI/ZA;EACE,gBAAA;AJkaF;;AIhaA;EACE,gBAAA;AJmaF;;AIjaA;EACE,gBAAA;AJoaF;;AIlaA;EACE,gBAAA;AJqaF;;AInaA;EACE,gBAAA;AJsaF;;AIpaA;EACE,gBAAA;AJuaF;;AIraA;EACE,gBAAA;AJwaF;;AItaA;EACE,gBAAA;AJyaF;;AIvaA;EACE,gBAAA;AJ0aF;;AIxaA;EACE,gBAAA;AJ2aF;;AIzaA;;EAEE,gBAAA;AJ4aF;;AI1aA;EACE,gBAAA;AJ6aF;;AI3aA;EACE,gBAAA;AJ8aF;;AI5aA;EACE,gBAAA;AJ+aF;;AI7aA;EACE,gBAAA;AJgbF;;AI9aA;EACE,gBAAA;AJibF;;AI/aA;EACE,gBAAA;AJkbF;;AIhbA;EACE,gBAAA;AJmbF;;AIjbA;EACE,gBAAA;AJobF;;AIlbA;EACE,gBAAA;AJqbF;;AInbA;EACE,gBAAA;AJsbF;;AIpbA;EACE,gBAAA;AJubF;;AIrbA;;EAEE,gBAAA;AJwbF;;AItbA;EACE,gBAAA;AJybF;;AIvbA;EACE,gBAAA;AJ0bF;;AIxbA;EACE,gBAAA;AJ2bF;;AIzbA;;EAEE,gBAAA;AJ4bF;;AI1bA;EACE,gBAAA;AJ6bF;;AI3bA;EACE,gBAAA;AJ8bF;;AI5bA;EACE,gBAAA;AJ+bF;;AI7bA;EACE,gBAAA;AJgcF;;AI9bA;EACE,gBAAA;AJicF;;AI/bA;EACE,gBAAA;AJkcF;;AIhcA;;;EAGE,gBAAA;AJmcF;;AIjcA;;EAEE,gBAAA;AJocF;;AIlcA;EACE,gBAAA;AJqcF;;AIncA;EACE,gBAAA;AJscF;;AIpcA;EACE,gBAAA;AJucF;;AIrcA;EACE,gBAAA;AJwcF;;AItcA;EACE,gBAAA;AJycF;;AIvcA;EACE,gBAAA;AJ0cF;;AIxcA;EACE,gBAAA;AJ2cF;;AIzcA;EACE,gBAAA;AJ4cF;;AI1cA;EACE,gBAAA;AJ6cF;;AI3cA;EACE,gBAAA;AJ8cF;;AI5cA;EACE,gBAAA;AJ+cF;;AI7cA;EACE,gBAAA;AJgdF;;AI9cA;EACE,gBAAA;AJidF;;AI/cA;EACE,gBAAA;AJkdF;;AIhdA;EACE,gBAAA;AJmdF;;AIjdA;EACE,gBAAA;AJodF;;AIldA;EACE,gBAAA;AJqdF;;AIndA;EACE,gBAAA;AJsdF;;AIpdA;EACE,gBAAA;AJudF;;AIrdA;EACE,gBAAA;AJwdF;;AItdA;EACE,gBAAA;AJydF;;AIvdA;EACE,gBAAA;AJ0dF;;AIxdA;EACE,gBAAA;AJ2dF;;AIzdA;EACE,gBAAA;AJ4dF;;AI1dA;EACE,gBAAA;AJ6dF;;AI3dA;;EAEE,gBAAA;AJ8dF;;AI5dA;;EAEE,gBAAA;AJ+dF;;AI7dA;EACE,gBAAA;AJgeF;;AI9dA;EACE,gBAAA;AJieF;;AI/dA;EACE,gBAAA;AJkeF;;AIheA;EACE,gBAAA;AJmeF;;AIjeA;EACE,gBAAA;AJoeF;;AIleA;EACE,gBAAA;AJqeF;;AIneA;EACE,gBAAA;AJseF;;AIpeA;EACE,gBAAA;AJueF;;AIreA;EACE,gBAAA;AJweF;;AIteA;;;EAGE,gBAAA;AJyeF;;AIveA;;EAEE,gBAAA;AJ0eF;;AIxeA;;EAEE,gBAAA;AJ2eF;;AIzeA;;EAEE,gBAAA;AJ4eF;;AI1eA;EACE,gBAAA;AJ6eF;;AI3eA;EACE,gBAAA;AJ8eF;;AI5eA;EACE,gBAAA;AJ+eF;;AI7eA;EACE,gBAAA;AJgfF;;AI9eA;;;;;EAKE,gBAAA;AJifF;;AI/eA;EACE,gBAAA;AJkfF;;AIhfA;;;EAGE,gBAAA;AJmfF;;AIjfA;;EAEE,gBAAA;AJofF;;AIlfA;EACE,gBAAA;AJqfF;;AInfA;EACE,gBAAA;AJsfF;;AIpfA;;;EAGE,gBAAA;AJufF;;AIrfA;EACE,gBAAA;AJwfF;;AItfA;EACE,gBAAA;AJyfF;;AIvfA;;EAEE,gBAAA;AJ0fF;;AIxfA;;EAEE,gBAAA;AJ2fF;;AIzfA;;EAEE,gBAAA;AJ4fF;;AI1fA;EACE,gBAAA;AJ6fF;;AI3fA;EACE,gBAAA;AJ8fF;;AI5fA;EACE,gBAAA;AJ+fF;;AI7fA;EACE,gBAAA;AJggBF;;AI9fA;EACE,gBAAA;AJigBF;;AI/fA;EACE,gBAAA;AJkgBF;;AIhgBA;EACE,gBAAA;AJmgBF;;AIjgBA;EACE,gBAAA;AJogBF;;AIlgBA;;EAEE,gBAAA;AJqgBF;;AIngBA;EACE,gBAAA;AJsgBF;;AIpgBA;EACE,gBAAA;AJugBF;;AIrgBA;EACE,gBAAA;AJwgBF;;AItgBA;EACE,gBAAA;AJygBF;;AIvgBA;EACE,gBAAA;AJ0gBF;;AIxgBA;EACE,gBAAA;AJ2gBF;;AIzgBA;EACE,gBAAA;AJ4gBF;;AI1gBA;EACE,gBAAA;AJ6gBF;;AI3gBA;EACE,gBAAA;AJ8gBF;;AI5gBA;EACE,gBAAA;AJ+gBF;;AI7gBA;EACE,gBAAA;AJghBF;;AI9gBA;EACE,gBAAA;AJihBF;;AI/gBA;EACE,gBAAA;AJkhBF;;AIhhBA;EACE,gBAAA;AJmhBF;;AIjhBA;EACE,gBAAA;AJohBF;;AIlhBA;EACE,gBAAA;AJqhBF;;AInhBA;EACE,gBAAA;AJshBF;;AIphBA;EACE,gBAAA;AJuhBF;;AIrhBA;EACE,gBAAA;AJwhBF;;AIthBA;EACE,gBAAA;AJyhBF;;AIvhBA;EACE,gBAAA;AJ0hBF;;AIxhBA;EACE,gBAAA;AJ2hBF;;AIzhBA;EACE,gBAAA;AJ4hBF;;AI1hBA;EACE,gBAAA;AJ6hBF;;AI3hBA;EACE,gBAAA;AJ8hBF;;AI5hBA;EACE,gBAAA;AJ+hBF;;AI7hBA;EACE,gBAAA;AJgiBF;;AI9hBA;EACE,gBAAA;AJiiBF;;AI/hBA;EACE,gBAAA;AJkiBF;;AIhiBA;EACE,gBAAA;AJmiBF;;AIjiBA;EACE,gBAAA;AJoiBF;;AIliBA;EACE,gBAAA;AJqiBF;;AIniBA;EACE,gBAAA;AJsiBF;;AIpiBA;EACE,gBAAA;AJuiBF;;AIriBA;EACE,gBAAA;AJwiBF;;AItiBA;EACE,gBAAA;AJyiBF;;AIviBA;EACE,gBAAA;AJ0iBF;;AIxiBA;;;EAGE,gBAAA;AJ2iBF;;AIziBA;EACE,gBAAA;AJ4iBF;;AI1iBA;EACE,gBAAA;AJ6iBF;;AI3iBA;EACE,gBAAA;AJ8iBF;;AI5iBA;EACE,gBAAA;AJ+iBF;;AI7iBA;EACE,gBAAA;AJgjBF;;AI9iBA;EACE,gBAAA;AJijBF;;AI/iBA;EACE,gBAAA;AJkjBF;;AIhjBA;EACE,gBAAA;AJmjBF;;AIjjBA;EACE,gBAAA;AJojBF;;AIljBA;EACE,gBAAA;AJqjBF;;AInjBA;EACE,gBAAA;AJsjBF;;AIpjBA;EACE,gBAAA;AJujBF;;AIrjBA;EACE,gBAAA;AJwjBF;;AItjBA;EACE,gBAAA;AJyjBF;;AIvjBA;EACE,gBAAA;AJ0jBF;;AIxjBA;EACE,gBAAA;AJ2jBF;;AIzjBA;EACE,gBAAA;AJ4jBF;;AI1jBA;EACE,gBAAA;AJ6jBF;;AI3jBA;EACE,gBAAA;AJ8jBF;;AI5jBA;EACE,gBAAA;AJ+jBF;;AI7jBA;EACE,gBAAA;AJgkBF;;AI9jBA;;EAEE,gBAAA;AJikBF;;AI/jBA;EACE,gBAAA;AJkkBF;;AIhkBA;EACE,gBAAA;AJmkBF;;AIjkBA;EACE,gBAAA;AJokBF;;AIlkBA;EACE,gBAAA;AJqkBF;;AInkBA;EACE,gBAAA;AJskBF;;AIpkBA;EACE,gBAAA;AJukBF;;AIrkBA;EACE,gBAAA;AJwkBF;;AItkBA;EACE,gBAAA;AJykBF;;AIvkBA;EACE,gBAAA;AJ0kBF;;AIxkBA;EACE,gBAAA;AJ2kBF;;AIzkBA;EACE,gBAAA;AJ4kBF;;AI1kBA;EACE,gBAAA;AJ6kBF;;AI3kBA;EACE,gBAAA;AJ8kBF;;AI5kBA;EACE,gBAAA;AJ+kBF;;AI7kBA;EACE,gBAAA;AJglBF;;AI9kBA;;EAEE,gBAAA;AJilBF;;AI/kBA;EACE,gBAAA;AJklBF;;AIhlBA;EACE,gBAAA;AJmlBF;;AIjlBA;EACE,gBAAA;AJolBF;;AIllBA;EACE,gBAAA;AJqlBF;;AInlBA;;EAEE,gBAAA;AJslBF;;AIplBA;EACE,gBAAA;AJulBF;;AIrlBA;EACE,gBAAA;AJwlBF;;AItlBA;EACE,gBAAA;AJylBF;;AIvlBA;;;EAGE,gBAAA;AJ0lBF;;AIxlBA;;EAEE,gBAAA;AJ2lBF;;AIzlBA;;EAEE,gBAAA;AJ4lBF;;AI1lBA;;EAEE,gBAAA;AJ6lBF;;AI3lBA;;EAEE,gBAAA;AJ8lBF;;AI5lBA;EACE,gBAAA;AJ+lBF;;AI7lBA;EACE,gBAAA;AJgmBF;;AI9lBA;EACE,gBAAA;AJimBF;;AI/lBA;EACE,gBAAA;AJkmBF;;AIhmBA;EACE,gBAAA;AJmmBF;;AIjmBA;EACE,gBAAA;AJomBF;;AIlmBA;EACE,gBAAA;AJqmBF;;AInmBA;EACE,gBAAA;AJsmBF;;AIpmBA;EACE,gBAAA;AJumBF;;AIrmBA;EACE,gBAAA;AJwmBF;;AItmBA;EACE,gBAAA;AJymBF;;AIvmBA;;EAEE,gBAAA;AJ0mBF;;AIxmBA;;EAEE,gBAAA;AJ2mBF;;AIzmBA;;EAEE,gBAAA;AJ4mBF;;AI1mBA;EACE,gBAAA;AJ6mBF;;AI3mBA;;EAEE,gBAAA;AJ8mBF;;AI5mBA;;EAEE,gBAAA;AJ+mBF;;AI7mBA;EACE,gBAAA;AJgnBF;;AI9mBA;EACE,gBAAA;AJinBF;;AI/mBA;EACE,gBAAA;AJknBF;;AIhnBA;EACE,gBAAA;AJmnBF;;AIjnBA;EACE,gBAAA;AJonBF;;AIlnBA;EACE,gBAAA;AJqnBF;;AInnBA;EACE,gBAAA;AJsnBF;;AIpnBA;EACE,gBAAA;AJunBF;;AIrnBA;EACE,gBAAA;AJwnBF;;AItnBA;EACE,gBAAA;AJynBF;;AIvnBA;EACE,gBAAA;AJ0nBF;;AIxnBA;EACE,gBAAA;AJ2nBF;;AIznBA;EACE,gBAAA;AJ4nBF;;AI1nBA;EACE,gBAAA;AJ6nBF;;AI3nBA;EACE,gBAAA;AJ8nBF;;AI5nBA;EACE,gBAAA;AJ+nBF;;AI7nBA;EACE,gBAAA;AJgoBF;;AI9nBA;EACE,gBAAA;AJioBF;;AI/nBA;EACE,gBAAA;AJkoBF;;AIhoBA;EACE,gBAAA;AJmoBF;;AIjoBA;;EAEE,gBAAA;AJooBF;;AIloBA;EACE,gBAAA;AJqoBF;;AInoBA;EACE,gBAAA;AJsoBF;;AIpoBA;EACE,gBAAA;AJuoBF;;AIroBA;EACE,gBAAA;AJwoBF;;AItoBA;EACE,gBAAA;AJyoBF;;AIvoBA;EACE,gBAAA;AJ0oBF;;AIxoBA;EACE,gBAAA;AJ2oBF;;AIzoBA;EACE,gBAAA;AJ4oBF;;AI1oBA;EACE,gBAAA;AJ6oBF;;AI3oBA;EACE,gBAAA;AJ8oBF;;AI5oBA;EACE,gBAAA;AJ+oBF;;AI7oBA;EACE,gBAAA;AJgpBF;;AI9oBA;EACE,gBAAA;AJipBF;;AI/oBA;EACE,gBAAA;AJkpBF;;AIhpBA;EACE,gBAAA;AJmpBF;;AIjpBA;EACE,gBAAA;AJopBF;;AIlpBA;EACE,gBAAA;AJqpBF;;AInpBA;EACE,gBAAA;AJspBF;;AIppBA;EACE,gBAAA;AJupBF;;AIrpBA;EACE,gBAAA;AJwpBF;;AItpBA;EACE,gBAAA;AJypBF;;AIvpBA;EACE,gBAAA;AJ0pBF;;AIxpBA;EACE,gBAAA;AJ2pBF;;AIzpBA;EACE,gBAAA;AJ4pBF;;AI1pBA;EACE,gBAAA;AJ6pBF;;AI3pBA;EACE,gBAAA;AJ8pBF;;AI5pBA;EACE,gBAAA;AJ+pBF;;AI7pBA;EACE,gBAAA;AJgqBF;;AI9pBA;EACE,gBAAA;AJiqBF;;AI/pBA;EACE,gBAAA;AJkqBF;;AIhqBA;EACE,gBAAA;AJmqBF;;AIjqBA;EACE,gBAAA;AJoqBF;;AIlqBA;EACE,gBAAA;AJqqBF;;AInqBA;EACE,gBAAA;AJsqBF;;AIpqBA;EACE,gBAAA;AJuqBF;;AIrqBA;EACE,gBAAA;AJwqBF;;AItqBA;EACE,gBAAA;AJyqBF;;AIvqBA;EACE,gBAAA;AJ0qBF;;AIxqBA;EACE,gBAAA;AJ2qBF;;AIzqBA;EACE,gBAAA;AJ4qBF;;AI1qBA;EACE,gBAAA;AJ6qBF;;AI3qBA;EACE,gBAAA;AJ8qBF;;AI5qBA;EACE,gBAAA;AJ+qBF;;AI7qBA;EACE,gBAAA;AJgrBF;;AI9qBA;EACE,gBAAA;AJirBF;;AI/qBA;EACE,gBAAA;AJkrBF;;AIhrBA;EACE,gBAAA;AJmrBF;;AIjrBA;EACE,gBAAA;AJorBF;;AIlrBA;EACE,gBAAA;AJqrBF;;AInrBA;EACE,gBAAA;AJsrBF;;AIprBA;;EAEE,gBAAA;AJurBF;;AIrrBA;;;EAGE,gBAAA;AJwrBF;;AItrBA;EACE,gBAAA;AJyrBF;;AIvrBA;EACE,gBAAA;AJ0rBF;;AIxrBA;;EAEE,gBAAA;AJ2rBF;;AIzrBA;EACE,gBAAA;AJ4rBF;;AI1rBA;EACE,gBAAA;AJ6rBF;;AI3rBA;EACE,gBAAA;AJ8rBF;;AI5rBA;EACE,gBAAA;AJ+rBF;;AI7rBA;EACE,gBAAA;AJgsBF;;AI9rBA;EACE,gBAAA;AJisBF;;AI/rBA;EACE,gBAAA;AJksBF;;AIhsBA;EACE,gBAAA;AJmsBF;;AIjsBA;EACE,gBAAA;AJosBF;;AIlsBA;EACE,gBAAA;AJqsBF;;AInsBA;;EAEE,gBAAA;AJssBF;;AIpsBA;;EAEE,gBAAA;AJusBF;;AIrsBA;EACE,gBAAA;AJwsBF;;AItsBA;EACE,gBAAA;AJysBF;;AIvsBA;EACE,gBAAA;AJ0sBF;;AIxsBA;EACE,gBAAA;AJ2sBF;;AIzsBA;EACE,gBAAA;AJ4sBF;;AI1sBA;EACE,gBAAA;AJ6sBF;;AI3sBA;;EAEE,gBAAA;AJ8sBF;;AI5sBA;;EAEE,gBAAA;AJ+sBF;;AI7sBA;EACE,gBAAA;AJgtBF;;AI9sBA;EACE,gBAAA;AJitBF;;AI/sBA;EACE,gBAAA;AJktBF;;AIhtBA;EACE,gBAAA;AJmtBF;;AIjtBA;;EAEE,gBAAA;AJotBF;;AIltBA;;EAEE,gBAAA;AJqtBF;;AIntBA;EACE,gBAAA;AJstBF;;AIptBA;EACE,gBAAA;AJutBF;;AIrtBA;EACE,gBAAA;AJwtBF;;AIttBA;;;EAGE,gBAAA;AJytBF;;AIvtBA;;EAEE,gBAAA;AJ0tBF;;AIxtBA;;EAEE,gBAAA;AJ2tBF;;AIztBA;;EAEE,gBAAA;AJ4tBF;;AI1tBA;;EAEE,gBAAA;AJ6tBF;;AI3tBA;EACE,gBAAA;AJ8tBF;;AI5tBA;;;EAGE,gBAAA;AJ+tBF;;AI7tBA;EACE,gBAAA;AJguBF;;AI9tBA;EACE,gBAAA;AJiuBF;;AI/tBA;EACE,gBAAA;AJkuBF;;AIhuBA;EACE,gBAAA;AJmuBF;;AIjuBA;;EAEE,gBAAA;AJouBF;;AIluBA;;EAEE,gBAAA;AJquBF;;AInuBA;EACE,gBAAA;AJsuBF;;AIpuBA;EACE,gBAAA;AJuuBF;;AIruBA;EACE,gBAAA;AJwuBF;;AItuBA;EACE,gBAAA;AJyuBF;;AIvuBA;EACE,gBAAA;AJ0uBF;;AIxuBA;EACE,gBAAA;AJ2uBF;;AIzuBA;EACE,gBAAA;AJ4uBF;;AI1uBA;EACE,gBAAA;AJ6uBF;;AI3uBA;EACE,gBAAA;AJ8uBF;;AI5uBA;EACE,gBAAA;AJ+uBF;;AI7uBA;EACE,gBAAA;AJgvBF;;AI9uBA;EACE,kBAAA;EACA,UAAA;EACA,WAAA;EACA,UAAA;EACA,YAAA;EACA,gBAAA;EACA,sBAAA;EACA,SAAA;AJivBF;;AI/uBA;;EAEE,gBAAA;EACA,WAAA;EACA,YAAA;EACA,SAAA;EACA,iBAAA;EACA,UAAA;AJkvBF;;AD1gGA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;AC6gGJ;AD3gGI;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;AC6gGR;ADzgGQ;EACI,yBEfK;AD0hGjB;ADxgGQ;EACI,4BErBK;AD+hGjB;ADtgGI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;ACwgGR;;AKhiGQ;EACI,aAAA;EACA,sBAAA;ALmiGZ;AKjiGY;EACI,aAAA;EACA,mBAAA;EACA,yBAAA;EACA,eAAA;ALmiGhB;AKhiGoB;EACI,oBAAA;ALkiGxB;AK7hGY;EACI,YAAA;EACA,cAAA;AL+hGhB;AKzhGwB;EAAc,WAAA;AL4hGtC;AK3hGwB;EAAe,WAAA;AL8hGvC;AK7hGwB;EAAe,WAAA;ALgiGvC;AK/hGwB;EAAe,WAAA;ALkiGvC;AK9hGwB;EAAc,iBAAA;ALiiGtC;AKhiGwB;EAAe,kBAAA;ALmiGvC;AKliGwB;EAAe,kBAAA;ALqiGvC;AKniGwB;EACI,kBAAA;ALqiG5B;AKniG4B;EACI,gBAAA;ALqiGhC;;ADjlGA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;AColGJ;ADllGI;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;AColGR;ADhlGQ;EACI,yBEfK;ADimGjB;AD/kGQ;EACI,4BErBK;ADsmGjB;AD7kGI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;AC+kGR;;AMvmGQ;EACI,aAAA;EACA,sBAAA;AN0mGZ;AMxmGY;EACI,aAAA;EACA,mBAAA;EACA,yBAAA;EACA,eAAA;AN0mGhB;AMvmGoB;EACI,oBAAA;ANymGxB;AMpmGY;EACI,YAAA;EACA,cAAA;ANsmGhB;AMhmGwB;EAAW,UAAA;ANmmGnC;AMlmGwB;EAAW,WAAA;ANqmGnC;AMpmGwB;EAAc,YAAA;ANumGtC;AMtmGwB;EAAgB,YAAA;ANymGxC;AMxmGwB;EAAiB,WAAA;AN2mGzC;AM1mGwB;EAAc,WAAA;AN6mGtC;AM5mGwB;EAAa,WAAA;AN+mGrC;AM3mGwB;EAAW,kBAAA;AN8mGnC;AM7mGwB;EAAiB,kBAAA;ANgnGzC;AM/mGwB;EAAc,kBAAA;ANknGtC;AMhnGwB;EACI,kBAAA;ANknG5B;AMhnG4B;EACI,aAAA;EACA,8BAAA;EACA,mBAAA;ANknGhC;AMhnGgC;EACI,gBAAA;ANknGpC;;ADtqGA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;ACyqGJ;ADvqGI;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;ACyqGR;ADrqGQ;EACI,yBEfK;ADsrGjB;ADpqGQ;EACI,4BErBK;AD2rGjB;ADlqGI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;ACoqGR;;AO5rGQ;EACI,aAAA;EACA,sBAAA;AP+rGZ;AO7rGY;EACI,aAAA;EACA,mBAAA;EACA,eAAA;EACA,gCAAA;AP+rGhB;AO7rGgB;EACI,aAAA;AP+rGpB;AO7rGoB;EACI,oBAAA;EACA,sBAAA;EACA,qBAAA;AP+rGxB;AO7rGwB;EACI,iBNxBX;EMyBW,yBNnBX;ADktGjB;AO1rGgB;EACI,YAAA;AP4rGpB;AOzrGgB;EACI,aAAA;EACA,mBAAA;AP2rGpB;AOxrGwB;EACI,oBAAA;AP0rG5B;AOprGY;EACI,YAAA;EACA,cAAA;APsrGhB;AO/qG4B;EAAa,YAAA;APkrGzC;AOjrG4B;EAAW,WAAA;APorGvC;AOnrG4B;EAAkB,YAAA;APsrG9C;AOlrG4B;EAAW,iBAAA;APqrGvC;AOprG4B;EAAkB,kBAAA;APurG9C;AOprGwB;EACI,YAAA;EACA,cN7DX;ADmvGjB;AOjrGgB;EACI,eAAA;EACA,aAAA;EACA,gBAAA;EACA,+BAAA;EACA,iCAAA;EACA,oEAAA;APmrGpB;AO/qGoB;EACI,0BAAA;APirGxB;AO9qGoB;EACI,eAAA;EACA,YAAA;APgrGxB;AO7qGoB;EACI,mBAAA;AP+qGxB;;AD1wGA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;AC6wGJ;AD3wGI;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;AC6wGR;ADzwGQ;EACI,yBEfK;AD0xGjB;ADxwGQ;EACI,4BErBK;AD+xGjB;ADtwGI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;ACwwGR;;AQhyGQ;EACI,aAAA;EACA,sBAAA;ARmyGZ;AQjyGY;EACI,eAAA;EACA,aAAA;EACA,mDAAA;EACA,gBAAA;EACA,mBAAA;ARmyGhB;AQjyGgB;EACI,iBAAA;ARmyGpB;AQ/xGY;EACI,YAAA;EACA,cAAA;ARiyGhB;AQ3xGwB;EAAW,UAAA;AR8xGnC;AQ7xGwB;EAAkB,YAAA;ARgyG1C;AQ/xGwB;EAAY,WAAA;ARkyGpC;AQjyGwB;EAAY,WAAA;ARoyGpC;AQnyGwB;EAAc,WAAA;ARsyGtC;AQryGwB;EAAa,WAAA;ARwyGrC;AQvyGwB;EAAa,WAAA;AR0yGrC;AQzyGwB;EAAe,aAAA;AR4yGvC;AQxyGwB;EAAqC,iBAAA;AR2yG7D;AQ1yGwB;EAAa,kBAAA;AR6yGrC;AQ5yGwB;EAAe,kBAAA;AR+yGvC;AQ7yGwB;EACI,yBAAA;EACA,iBAAA;AR+yG5B;AQzyG4B;EACI,yBPlDf;AD61GjB;AQryGgB;EACI,eAAA;EACA,aAAA;EACA,yBAAA;ARuyGpB;AQlyGgB;EACI,gBAAA;ARoyGpB;AQlyGoB;EACI,aAAA;EACA,eAAA;EACA,gCAAA;ARoyGxB;AQlyGwB;EACI,oBAAA;EACA,sBAAA;EACA,qBAAA;ARoyG5B;AQlyG4B;EACI,iBPjFf;EOkFe,yBP5Ef;ADg3GjB;AQ/xGoB;EACI,cAAA;EACA,aAAA;EACA,oBAAA;ARiyGxB;AQ/xGwB;EACI,YAAA;EACA,aAAA;EACA,mDAAA;EACA,gBAAA;EACA,mBAAA;ARiyG5B;AQ/xG4B;EACI,iBAAA;ARiyGhC;AQ7xGwB;EACI,mBAAA;AR+xG5B;AQxxGgB;EACI,aAAA;EACA,oBAAA;EACA,eAAA;EACA,6BAAA;EACA,kBAAA;AR0xGpB;AQxxGoB;EACI,YAAA;EACA,aAAA;EACA,oDAAA;EACA,gBAAA;EACA,mBAAA;AR0xGxB;AQxxGwB;EACI,iBAAA;AR0xG5B;AQtxGoB;EACI,mBAAA;ARwxGxB;;AD35GA;EACI,WAAA;EACA,iBAAA;EACA,mBAAA;EACA,kBAAA;AC85GJ;AD55GI;EACI,eAAA;EACA,mBEHS;EFIT,iBEVS;EFWT,gBAAA;EACA,MAAA;AC85GR;AD15GQ;EACI,yBEfK;AD26GjB;ADz5GQ;EACI,4BErBK;ADg7GjB;ADv5GI;EACI,eAAA;EACA,gBAAA;EACA,uBAAA;EACA,mBAAA;ACy5GR;;ASj7GQ;EACI,aAAA;EACA,sBAAA;ATo7GZ;ASl7GY;EACI,eAAA;EACA,aAAA;EACA,mDAAA;EACA,gBAAA;EACA,mBAAA;ATo7GhB;ASl7GgB;EACI,iBAAA;ATo7GpB;ASh7GY;EACI,6BAAA;EACA,eAAA;EACA,aAAA;EACA,oBAAA;ATk7GhB;ASh7GgB;EACI,YAAA;EACA,aAAA;EACA,mDAAA;EACA,gBAAA;EACA,mBAAA;ATk7GpB;ASh7GoB;EACI,iBAAA;ATk7GxB;AS96GgB;EACI,mBAAA;ATg7GpB;AS56GY;EACI,YAAA;EACA,cAAA;AT86GhB;ASx6GwB;EAAkB,YAAA;AT26G1C;AS16GwB;EAAY,YAAA;AT66GpC;AS56GwB;EAAY,WAAA;AT+6GpC;AS96GwB;EAAe,aAAA;ATi7GvC;AS76GwB;EAAe,kBAAA;ATg7GvC;AS96GwB;EACI,yBAAA;EACA,iBAAA;ATg7G5B;AS16G4B;EACI,yBRnEf;AD++GjB;ASt6GgB;EACI,eAAA;EACA,aAAA;EACA,yBAAA;ATw6GpB;ASp6GY;EACI,eAAA;EACA,aAAA;EACA,oDAAA;EACA,gBAAA;EACA,mBAAA;EACA,6BAAA;EACA,kBAAA;ATs6GhB;ASp6GgB;EACI,iBAAA;ATs6GpB;;AAp/GA;EACI,oBAAA;EACA,eAAA;AAu/GJ;;AAp/GA;EACI,SAAA;EACA,iBAAA;EACA,YAAA;EACA,gBAAA;AAu/GJ;;AAp/GA;EACI,YAAA;EACA,yBC5Ba;ED6Bb,aAAA;EACA,6CAAA;EACA,wCAAA;EACA,4IAAA;AAu/GJ;AAl/GI;EACI,eAAA;EACA,yBClCS;EDmCT,iBCzCS;AD6hHjB;AAj/GY;EACI,oBAAA;AAm/GhB;AA/+GQ;EACI,uBAAA;AAi/GZ;AA9+GQ;EACI,uBAAA;AAg/GZ;AA7+GgB;EACI,kBAAA;AA++GpB;AAz+GI;EACI,kBAAA;EACA,4BClES;EDmET,aAAA;EACA,sBAAA;EACA,gBAAA;EACA,gBAAA;AA2+GR;AAz+GQ;EACI,YAAA;EACA,cAAA;AA2+GZ;AAz+GY;EACI,eAAA;EACA,gBAAA;EACA,mBAAA;EACA,uBAAA;AA2+GhB;AAz+GgB;EACI,yBCjFH;AD4jHjB;AAx+GgB;EACI,oBAAA;AA0+GpB;AAx+GoB;EACI,UAAA;AA0+GxB;AAv+GoB;EACI,aAAA;AAy+GxB;AAt+GoB;EACI,YAAA;AAw+GxB;AAp+GgB;EACI,cClGH;ADwkHjB;AAj+GQ;EACI,cAAA;AAm+GZ;AA/9GI;EUtGF,+GAAA;EVwGM,yBAAA;EACA,iBAAA;EACA,yBClHS;ADmlHjB;AA99GI;EACI,eAAA;EACA,cAAA;AAg+GR;AA79GI;EUlHF,+GAAA;EVoHM,qBAAA;EACA,iBAAA;EACA,yBC9HS;ED+HT,UAAA;AA+9GR;AA59GI;EUpHF,gHAAA;EVsHM,cAAA;EACA,6BAAA;EACA,4BCzIS;ED0IT,cAAA;EACA,aA/HG;AA6lHX;AAz9GY;EACI,YAAA;EACA,kBAAA;AA29GhB;AAx9GY;EACI,gBAAA;AA09GhB;AAv9GY;EACI,yBAAA;EACA,iBAAA;AAy9GhB;AAp9GI;EACI,kBAAA;EACA,WAAA;EACA,YAAA;EACA,oCAAA;EACA,aAAA;EACA,uBAAA;EACA,mBAAA;EACA,WAAA;AAs9GR;AAp9GQ;EUzJN,gHAAA;EV2JU,yBAAA;EACA,qBAAA;EACA,4BC9KK;ED+KL,eAAA;AAs9GZ;AAp9GY;EACI,aAAA;EACA,sBAAA;AAs9GhB;AAp9GgB;EACI,eAAA;EACA,eAAA;EACA,aAAA;EACA,mBAAA;AAs9GpB;AAp9GoB;EACI,WAAA;EACA,oBAAA;AAs9GxB;AAn9GoB;EACI,yBC/LP;ADopHjB;AAh9GY;EACI,aAAA;EACA,gCAAA;EACA,gBAAA;EACA,mBAAA;AAk9GhB;AAh9GgB;EACI,mBAAA;EACA,mBCxMH;EDyMG,iBC/MH;EDgNG,eAAA;EACA,gBAAA;AAk9GpB;AA/8GgB;EACI,iBAAA;AAi9GpB;AA98GgB;EACI,mBAAA;EACA,cCpNH;EDqNG,iBAAA;AAg9GpB","sourcesContent":["@import 'color';\n\n\n%table {\n    width: 100%;\n    border-spacing: 0;\n    table-layout: fixed;\n    position: relative;\n\n    th {\n        padding: 0.2rem;\n        background: $color-grey-700;\n        color: $color-grey-100;\n        position: sticky;\n        top: 0;\n    }\n\n    tr {\n        &:nth-child(even) {\n            background-color: $color-grey-300;\n        }\n\n        &:nth-child(odd) {\n            background-color: $color-grey-100;\n        }\n    }\n\n    td {\n        padding: 0.2rem;\n        overflow: hidden;\n        text-overflow: ellipsis;\n        white-space: nowrap;\n    }\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n@import 'fonts/inter';\n@import 'fonts/robotomono';\n@import 'fonts/fontawesome';\n\n@import 'orchestrator';\n@import 'monitor';\n@import 'event';\n@import 'iec104';\n@import 'modbus';\n\n\n$log-size: 10rem;\n$resizer-size: 0.4rem;\n\nhtml, body, button, input, select {\n    font-family: 'Inter';\n    font-size: 12pt;\n}\n\nhtml, body {\n    margin: 0;\n    user-select: none;\n    height: 100%;\n    overflow: hidden;\n}\n\n#main {\n    height: 100%;\n    background-color: $color-grey-200;\n    display: grid;\n    grid-template-columns: max-content $resizer-size 1fr;\n    grid-template-rows: auto 1fr $resizer-size auto;\n    grid-template-areas: 'globalHeader deviceHeader deviceHeader'\n                         'sidebar sidebarResizer page'\n                         'logResizer logResizer logResizer'\n                         'log log log';\n\n    & > .header {\n        padding: 0.2rem;\n        background-color: $color-grey-700;\n        color: $color-grey-100;\n\n        & > button {\n            &:not(:last-child) {\n                margin-right: 0.2rem;\n            }\n        }\n\n        &.global {\n            grid-area: globalHeader;\n        }\n\n        &.device {\n            grid-area: deviceHeader;\n\n            & > * {\n                &:not(button) {\n                    margin-right: 1rem;\n                }\n            }\n        }\n    }\n\n    & > .sidebar {\n        grid-area: sidebar;\n        background-color: $color-grey-100;\n        display: flex;\n        flex-direction: column;\n        overflow: hidden;\n        min-width: 10rem;\n\n        & > .devices {\n            flex-grow: 1;\n            overflow: auto;\n\n            & > .device {\n                padding: 0.5rem;\n                overflow: hidden;\n                white-space: nowrap;\n                text-overflow: ellipsis;\n\n                &:hover, &.selected {\n                    background-color: $color-grey-300;\n                }\n\n                & > .status {\n                    margin-right: 0.5rem;\n\n                    &.stopped {\n                        color: red;\n                    }\n\n                    &.starting, &.stopping {\n                        color: yellow;\n                    }\n\n                    &.started {\n                        color: green;\n                    }\n                }\n\n                & > .type {\n                    color: $color-grey-600;\n                }\n            }\n        }\n\n        & > .add {\n            margin: 0.5rem;\n        }\n    }\n\n    & > .sidebar-resizer {\n        @include shadow-2dp;\n        grid-area: sidebarResizer;\n        cursor: ew-resize;\n        background-color: $color-grey-400;\n    }\n\n    & > .page {\n        grid-area: page;\n        overflow: auto;\n    }\n\n    & > .log-resizer {\n        @include shadow-2dp;\n        grid-area: logResizer;\n        cursor: ns-resize;\n        background-color: $color-grey-400;\n        z-index: 1;\n    }\n\n    & > .log {\n        @include shadow-4dp;\n        grid-area: log;\n        border-top: 1px solid $color-grey-400;\n        background-color: $color-grey-100;\n        overflow: auto;\n        height: $log-size;\n\n        & > table {\n            @extend %table;\n\n            .col-time {\n                width: 15rem;\n                text-align: center;\n            }\n\n            .col-message {\n                text-align: left;\n            }\n\n            td.col-time {\n                font-family: 'RobotoMono';\n                font-size: 0.9rem;\n            }\n        }\n    }\n\n    & > .overlay {\n        position: absolute;\n        width: 100%;\n        height: 100%;\n        background-color: rgba(black, 0.2);\n        display: flex;\n        justify-content: center;\n        align-items: center;\n        z-index: 10;\n\n        & > .dialog {\n            @include shadow-4dp;\n            border: 1px solid $color-grey-500;\n            border-radius: 0.2rem;\n            background-color: $color-grey-100;\n            padding: 0.5rem;\n\n            &.add {\n                display: flex;\n                flex-direction: column;\n\n                & > .device {\n                    padding: 0.5rem;\n                    cursor: pointer;\n                    display: flex;\n                    align-items: center;\n\n                    .icon {\n                        width: 32px;\n                        margin-right: 0.5rem;\n                    }\n\n                    &:hover {\n                        background-color: $color-grey-300;\n                    }\n                }\n            }\n\n            &.settings {\n                display: grid;\n                grid-template-columns: auto auto;\n                grid-gap: 0.5rem;\n                align-items: center;\n\n                & > .title {\n                    grid-column: span 2;\n                    background: $color-grey-700;\n                    color: $color-grey-100;\n                    padding: 0.2rem;\n                    font-weight: 600;\n                }\n\n                & > .label {\n                    justify-self: end;\n                }\n\n                & > .note {\n                    grid-column: span 2;\n                    color: $color-grey-700;\n                    justify-self: end;\n                }\n            }\n        }\n    }\n}\n\n\n","\n$color-grey-50: rgb(250, 250, 250);\n$color-grey-100: rgb(245, 245, 245);\n$color-grey-200: rgb(238, 238, 238);\n$color-grey-300: rgb(224, 224, 224);\n$color-grey-400: rgb(189, 189, 189);\n$color-grey-500: rgb(158, 158, 158);\n$color-grey-600: rgb(117, 117, 117);\n$color-grey-700: rgb(97, 97, 97);\n$color-grey-800: rgb(66, 66, 66);\n$color-grey-900: rgb(33, 33, 33);\n","\n@font-face {\n    font-family: 'Inter';\n    src: url('fonts/Inter/Inter.woff2') format('woff2');\n    font-weight: 100 900;\n    font-style: oblique 0deg 10deg;\n    font-display: swap;\n}\n","\n@font-face {\n    font-family: 'RobotoMono';\n    src: url('fonts/RobotoMono/RobotoMono.woff2') format('woff2');\n    font-weight: 100 900;\n    font-style: normal;\n    font-display: swap;\n}\n\n@font-face {\n    font-family: 'RobotoMono';\n    src: url('fonts/RobotoMono/RobotoMono-Italic.woff2') format('woff2');\n    font-weight: 100 900;\n    font-style: italic;\n    font-display: swap;\n}\n","\n@font-face {\n    font-family: 'FontAwesome';\n    src: url('fonts/FontAwesome/FontAwesome.woff2') format('woff2');\n    font-weight: normal;\n    font-style: normal;\n}\n\n\n.fa {\n  display: inline-block;\n  font: normal normal normal 14px/1 FontAwesome;\n  font-size: inherit;\n  text-rendering: auto;\n  -webkit-font-smoothing: antialiased;\n  -moz-osx-font-smoothing: grayscale;\n}\n/* makes the font 33% larger relative to the icon container */\n.fa-lg {\n  font-size: 1.33333333em;\n  line-height: 0.75em;\n  vertical-align: -15%;\n}\n.fa-2x {\n  font-size: 2em;\n}\n.fa-3x {\n  font-size: 3em;\n}\n.fa-4x {\n  font-size: 4em;\n}\n.fa-5x {\n  font-size: 5em;\n}\n.fa-fw {\n  width: 1.28571429em;\n  text-align: center;\n}\n.fa-ul {\n  padding-left: 0;\n  margin-left: 2.14285714em;\n  list-style-type: none;\n}\n.fa-ul > li {\n  position: relative;\n}\n.fa-li {\n  position: absolute;\n  left: -2.14285714em;\n  width: 2.14285714em;\n  top: 0.14285714em;\n  text-align: center;\n}\n.fa-li.fa-lg {\n  left: -1.85714286em;\n}\n.fa-border {\n  padding: .2em .25em .15em;\n  border: solid 0.08em #eeeeee;\n  border-radius: .1em;\n}\n.fa-pull-left {\n  float: left;\n}\n.fa-pull-right {\n  float: right;\n}\n.fa.fa-pull-left {\n  margin-right: .3em;\n}\n.fa.fa-pull-right {\n  margin-left: .3em;\n}\n/* Deprecated as of 4.4.0 */\n.pull-right {\n  float: right;\n}\n.pull-left {\n  float: left;\n}\n.fa.pull-left {\n  margin-right: .3em;\n}\n.fa.pull-right {\n  margin-left: .3em;\n}\n.fa-spin {\n  -webkit-animation: fa-spin 2s infinite linear;\n  animation: fa-spin 2s infinite linear;\n}\n.fa-pulse {\n  -webkit-animation: fa-spin 1s infinite steps(8);\n  animation: fa-spin 1s infinite steps(8);\n}\n@-webkit-keyframes fa-spin {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(359deg);\n    transform: rotate(359deg);\n  }\n}\n@keyframes fa-spin {\n  0% {\n    -webkit-transform: rotate(0deg);\n    transform: rotate(0deg);\n  }\n  100% {\n    -webkit-transform: rotate(359deg);\n    transform: rotate(359deg);\n  }\n}\n.fa-rotate-90 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=1)\";\n  -webkit-transform: rotate(90deg);\n  -ms-transform: rotate(90deg);\n  transform: rotate(90deg);\n}\n.fa-rotate-180 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=2)\";\n  -webkit-transform: rotate(180deg);\n  -ms-transform: rotate(180deg);\n  transform: rotate(180deg);\n}\n.fa-rotate-270 {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=3)\";\n  -webkit-transform: rotate(270deg);\n  -ms-transform: rotate(270deg);\n  transform: rotate(270deg);\n}\n.fa-flip-horizontal {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=0, mirror=1)\";\n  -webkit-transform: scale(-1, 1);\n  -ms-transform: scale(-1, 1);\n  transform: scale(-1, 1);\n}\n.fa-flip-vertical {\n  -ms-filter: \"progid:DXImageTransform.Microsoft.BasicImage(rotation=2, mirror=1)\";\n  -webkit-transform: scale(1, -1);\n  -ms-transform: scale(1, -1);\n  transform: scale(1, -1);\n}\n:root .fa-rotate-90,\n:root .fa-rotate-180,\n:root .fa-rotate-270,\n:root .fa-flip-horizontal,\n:root .fa-flip-vertical {\n  filter: none;\n}\n.fa-stack {\n  position: relative;\n  display: inline-block;\n  width: 2em;\n  height: 2em;\n  line-height: 2em;\n  vertical-align: middle;\n}\n.fa-stack-1x,\n.fa-stack-2x {\n  position: absolute;\n  left: 0;\n  width: 100%;\n  text-align: center;\n}\n.fa-stack-1x {\n  line-height: inherit;\n}\n.fa-stack-2x {\n  font-size: 2em;\n}\n.fa-inverse {\n  color: #ffffff;\n}\n/* Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen\n   readers do not read off random characters that represent icons */\n.fa-glass:before {\n  content: \"\\f000\";\n}\n.fa-music:before {\n  content: \"\\f001\";\n}\n.fa-search:before {\n  content: \"\\f002\";\n}\n.fa-envelope-o:before {\n  content: \"\\f003\";\n}\n.fa-heart:before {\n  content: \"\\f004\";\n}\n.fa-star:before {\n  content: \"\\f005\";\n}\n.fa-star-o:before {\n  content: \"\\f006\";\n}\n.fa-user:before {\n  content: \"\\f007\";\n}\n.fa-film:before {\n  content: \"\\f008\";\n}\n.fa-th-large:before {\n  content: \"\\f009\";\n}\n.fa-th:before {\n  content: \"\\f00a\";\n}\n.fa-th-list:before {\n  content: \"\\f00b\";\n}\n.fa-check:before {\n  content: \"\\f00c\";\n}\n.fa-remove:before,\n.fa-close:before,\n.fa-times:before {\n  content: \"\\f00d\";\n}\n.fa-search-plus:before {\n  content: \"\\f00e\";\n}\n.fa-search-minus:before {\n  content: \"\\f010\";\n}\n.fa-power-off:before {\n  content: \"\\f011\";\n}\n.fa-signal:before {\n  content: \"\\f012\";\n}\n.fa-gear:before,\n.fa-cog:before {\n  content: \"\\f013\";\n}\n.fa-trash-o:before {\n  content: \"\\f014\";\n}\n.fa-home:before {\n  content: \"\\f015\";\n}\n.fa-file-o:before {\n  content: \"\\f016\";\n}\n.fa-clock-o:before {\n  content: \"\\f017\";\n}\n.fa-road:before {\n  content: \"\\f018\";\n}\n.fa-download:before {\n  content: \"\\f019\";\n}\n.fa-arrow-circle-o-down:before {\n  content: \"\\f01a\";\n}\n.fa-arrow-circle-o-up:before {\n  content: \"\\f01b\";\n}\n.fa-inbox:before {\n  content: \"\\f01c\";\n}\n.fa-play-circle-o:before {\n  content: \"\\f01d\";\n}\n.fa-rotate-right:before,\n.fa-repeat:before {\n  content: \"\\f01e\";\n}\n.fa-refresh:before {\n  content: \"\\f021\";\n}\n.fa-list-alt:before {\n  content: \"\\f022\";\n}\n.fa-lock:before {\n  content: \"\\f023\";\n}\n.fa-flag:before {\n  content: \"\\f024\";\n}\n.fa-headphones:before {\n  content: \"\\f025\";\n}\n.fa-volume-off:before {\n  content: \"\\f026\";\n}\n.fa-volume-down:before {\n  content: \"\\f027\";\n}\n.fa-volume-up:before {\n  content: \"\\f028\";\n}\n.fa-qrcode:before {\n  content: \"\\f029\";\n}\n.fa-barcode:before {\n  content: \"\\f02a\";\n}\n.fa-tag:before {\n  content: \"\\f02b\";\n}\n.fa-tags:before {\n  content: \"\\f02c\";\n}\n.fa-book:before {\n  content: \"\\f02d\";\n}\n.fa-bookmark:before {\n  content: \"\\f02e\";\n}\n.fa-print:before {\n  content: \"\\f02f\";\n}\n.fa-camera:before {\n  content: \"\\f030\";\n}\n.fa-font:before {\n  content: \"\\f031\";\n}\n.fa-bold:before {\n  content: \"\\f032\";\n}\n.fa-italic:before {\n  content: \"\\f033\";\n}\n.fa-text-height:before {\n  content: \"\\f034\";\n}\n.fa-text-width:before {\n  content: \"\\f035\";\n}\n.fa-align-left:before {\n  content: \"\\f036\";\n}\n.fa-align-center:before {\n  content: \"\\f037\";\n}\n.fa-align-right:before {\n  content: \"\\f038\";\n}\n.fa-align-justify:before {\n  content: \"\\f039\";\n}\n.fa-list:before {\n  content: \"\\f03a\";\n}\n.fa-dedent:before,\n.fa-outdent:before {\n  content: \"\\f03b\";\n}\n.fa-indent:before {\n  content: \"\\f03c\";\n}\n.fa-video-camera:before {\n  content: \"\\f03d\";\n}\n.fa-photo:before,\n.fa-image:before,\n.fa-picture-o:before {\n  content: \"\\f03e\";\n}\n.fa-pencil:before {\n  content: \"\\f040\";\n}\n.fa-map-marker:before {\n  content: \"\\f041\";\n}\n.fa-adjust:before {\n  content: \"\\f042\";\n}\n.fa-tint:before {\n  content: \"\\f043\";\n}\n.fa-edit:before,\n.fa-pencil-square-o:before {\n  content: \"\\f044\";\n}\n.fa-share-square-o:before {\n  content: \"\\f045\";\n}\n.fa-check-square-o:before {\n  content: \"\\f046\";\n}\n.fa-arrows:before {\n  content: \"\\f047\";\n}\n.fa-step-backward:before {\n  content: \"\\f048\";\n}\n.fa-fast-backward:before {\n  content: \"\\f049\";\n}\n.fa-backward:before {\n  content: \"\\f04a\";\n}\n.fa-play:before {\n  content: \"\\f04b\";\n}\n.fa-pause:before {\n  content: \"\\f04c\";\n}\n.fa-stop:before {\n  content: \"\\f04d\";\n}\n.fa-forward:before {\n  content: \"\\f04e\";\n}\n.fa-fast-forward:before {\n  content: \"\\f050\";\n}\n.fa-step-forward:before {\n  content: \"\\f051\";\n}\n.fa-eject:before {\n  content: \"\\f052\";\n}\n.fa-chevron-left:before {\n  content: \"\\f053\";\n}\n.fa-chevron-right:before {\n  content: \"\\f054\";\n}\n.fa-plus-circle:before {\n  content: \"\\f055\";\n}\n.fa-minus-circle:before {\n  content: \"\\f056\";\n}\n.fa-times-circle:before {\n  content: \"\\f057\";\n}\n.fa-check-circle:before {\n  content: \"\\f058\";\n}\n.fa-question-circle:before {\n  content: \"\\f059\";\n}\n.fa-info-circle:before {\n  content: \"\\f05a\";\n}\n.fa-crosshairs:before {\n  content: \"\\f05b\";\n}\n.fa-times-circle-o:before {\n  content: \"\\f05c\";\n}\n.fa-check-circle-o:before {\n  content: \"\\f05d\";\n}\n.fa-ban:before {\n  content: \"\\f05e\";\n}\n.fa-arrow-left:before {\n  content: \"\\f060\";\n}\n.fa-arrow-right:before {\n  content: \"\\f061\";\n}\n.fa-arrow-up:before {\n  content: \"\\f062\";\n}\n.fa-arrow-down:before {\n  content: \"\\f063\";\n}\n.fa-mail-forward:before,\n.fa-share:before {\n  content: \"\\f064\";\n}\n.fa-expand:before {\n  content: \"\\f065\";\n}\n.fa-compress:before {\n  content: \"\\f066\";\n}\n.fa-plus:before {\n  content: \"\\f067\";\n}\n.fa-minus:before {\n  content: \"\\f068\";\n}\n.fa-asterisk:before {\n  content: \"\\f069\";\n}\n.fa-exclamation-circle:before {\n  content: \"\\f06a\";\n}\n.fa-gift:before {\n  content: \"\\f06b\";\n}\n.fa-leaf:before {\n  content: \"\\f06c\";\n}\n.fa-fire:before {\n  content: \"\\f06d\";\n}\n.fa-eye:before {\n  content: \"\\f06e\";\n}\n.fa-eye-slash:before {\n  content: \"\\f070\";\n}\n.fa-warning:before,\n.fa-exclamation-triangle:before {\n  content: \"\\f071\";\n}\n.fa-plane:before {\n  content: \"\\f072\";\n}\n.fa-calendar:before {\n  content: \"\\f073\";\n}\n.fa-random:before {\n  content: \"\\f074\";\n}\n.fa-comment:before {\n  content: \"\\f075\";\n}\n.fa-magnet:before {\n  content: \"\\f076\";\n}\n.fa-chevron-up:before {\n  content: \"\\f077\";\n}\n.fa-chevron-down:before {\n  content: \"\\f078\";\n}\n.fa-retweet:before {\n  content: \"\\f079\";\n}\n.fa-shopping-cart:before {\n  content: \"\\f07a\";\n}\n.fa-folder:before {\n  content: \"\\f07b\";\n}\n.fa-folder-open:before {\n  content: \"\\f07c\";\n}\n.fa-arrows-v:before {\n  content: \"\\f07d\";\n}\n.fa-arrows-h:before {\n  content: \"\\f07e\";\n}\n.fa-bar-chart-o:before,\n.fa-bar-chart:before {\n  content: \"\\f080\";\n}\n.fa-twitter-square:before {\n  content: \"\\f081\";\n}\n.fa-facebook-square:before {\n  content: \"\\f082\";\n}\n.fa-camera-retro:before {\n  content: \"\\f083\";\n}\n.fa-key:before {\n  content: \"\\f084\";\n}\n.fa-gears:before,\n.fa-cogs:before {\n  content: \"\\f085\";\n}\n.fa-comments:before {\n  content: \"\\f086\";\n}\n.fa-thumbs-o-up:before {\n  content: \"\\f087\";\n}\n.fa-thumbs-o-down:before {\n  content: \"\\f088\";\n}\n.fa-star-half:before {\n  content: \"\\f089\";\n}\n.fa-heart-o:before {\n  content: \"\\f08a\";\n}\n.fa-sign-out:before {\n  content: \"\\f08b\";\n}\n.fa-linkedin-square:before {\n  content: \"\\f08c\";\n}\n.fa-thumb-tack:before {\n  content: \"\\f08d\";\n}\n.fa-external-link:before {\n  content: \"\\f08e\";\n}\n.fa-sign-in:before {\n  content: \"\\f090\";\n}\n.fa-trophy:before {\n  content: \"\\f091\";\n}\n.fa-github-square:before {\n  content: \"\\f092\";\n}\n.fa-upload:before {\n  content: \"\\f093\";\n}\n.fa-lemon-o:before {\n  content: \"\\f094\";\n}\n.fa-phone:before {\n  content: \"\\f095\";\n}\n.fa-square-o:before {\n  content: \"\\f096\";\n}\n.fa-bookmark-o:before {\n  content: \"\\f097\";\n}\n.fa-phone-square:before {\n  content: \"\\f098\";\n}\n.fa-twitter:before {\n  content: \"\\f099\";\n}\n.fa-facebook-f:before,\n.fa-facebook:before {\n  content: \"\\f09a\";\n}\n.fa-github:before {\n  content: \"\\f09b\";\n}\n.fa-unlock:before {\n  content: \"\\f09c\";\n}\n.fa-credit-card:before {\n  content: \"\\f09d\";\n}\n.fa-feed:before,\n.fa-rss:before {\n  content: \"\\f09e\";\n}\n.fa-hdd-o:before {\n  content: \"\\f0a0\";\n}\n.fa-bullhorn:before {\n  content: \"\\f0a1\";\n}\n.fa-bell:before {\n  content: \"\\f0f3\";\n}\n.fa-certificate:before {\n  content: \"\\f0a3\";\n}\n.fa-hand-o-right:before {\n  content: \"\\f0a4\";\n}\n.fa-hand-o-left:before {\n  content: \"\\f0a5\";\n}\n.fa-hand-o-up:before {\n  content: \"\\f0a6\";\n}\n.fa-hand-o-down:before {\n  content: \"\\f0a7\";\n}\n.fa-arrow-circle-left:before {\n  content: \"\\f0a8\";\n}\n.fa-arrow-circle-right:before {\n  content: \"\\f0a9\";\n}\n.fa-arrow-circle-up:before {\n  content: \"\\f0aa\";\n}\n.fa-arrow-circle-down:before {\n  content: \"\\f0ab\";\n}\n.fa-globe:before {\n  content: \"\\f0ac\";\n}\n.fa-wrench:before {\n  content: \"\\f0ad\";\n}\n.fa-tasks:before {\n  content: \"\\f0ae\";\n}\n.fa-filter:before {\n  content: \"\\f0b0\";\n}\n.fa-briefcase:before {\n  content: \"\\f0b1\";\n}\n.fa-arrows-alt:before {\n  content: \"\\f0b2\";\n}\n.fa-group:before,\n.fa-users:before {\n  content: \"\\f0c0\";\n}\n.fa-chain:before,\n.fa-link:before {\n  content: \"\\f0c1\";\n}\n.fa-cloud:before {\n  content: \"\\f0c2\";\n}\n.fa-flask:before {\n  content: \"\\f0c3\";\n}\n.fa-cut:before,\n.fa-scissors:before {\n  content: \"\\f0c4\";\n}\n.fa-copy:before,\n.fa-files-o:before {\n  content: \"\\f0c5\";\n}\n.fa-paperclip:before {\n  content: \"\\f0c6\";\n}\n.fa-save:before,\n.fa-floppy-o:before {\n  content: \"\\f0c7\";\n}\n.fa-square:before {\n  content: \"\\f0c8\";\n}\n.fa-navicon:before,\n.fa-reorder:before,\n.fa-bars:before {\n  content: \"\\f0c9\";\n}\n.fa-list-ul:before {\n  content: \"\\f0ca\";\n}\n.fa-list-ol:before {\n  content: \"\\f0cb\";\n}\n.fa-strikethrough:before {\n  content: \"\\f0cc\";\n}\n.fa-underline:before {\n  content: \"\\f0cd\";\n}\n.fa-table:before {\n  content: \"\\f0ce\";\n}\n.fa-magic:before {\n  content: \"\\f0d0\";\n}\n.fa-truck:before {\n  content: \"\\f0d1\";\n}\n.fa-pinterest:before {\n  content: \"\\f0d2\";\n}\n.fa-pinterest-square:before {\n  content: \"\\f0d3\";\n}\n.fa-google-plus-square:before {\n  content: \"\\f0d4\";\n}\n.fa-google-plus:before {\n  content: \"\\f0d5\";\n}\n.fa-money:before {\n  content: \"\\f0d6\";\n}\n.fa-caret-down:before {\n  content: \"\\f0d7\";\n}\n.fa-caret-up:before {\n  content: \"\\f0d8\";\n}\n.fa-caret-left:before {\n  content: \"\\f0d9\";\n}\n.fa-caret-right:before {\n  content: \"\\f0da\";\n}\n.fa-columns:before {\n  content: \"\\f0db\";\n}\n.fa-unsorted:before,\n.fa-sort:before {\n  content: \"\\f0dc\";\n}\n.fa-sort-down:before,\n.fa-sort-desc:before {\n  content: \"\\f0dd\";\n}\n.fa-sort-up:before,\n.fa-sort-asc:before {\n  content: \"\\f0de\";\n}\n.fa-envelope:before {\n  content: \"\\f0e0\";\n}\n.fa-linkedin:before {\n  content: \"\\f0e1\";\n}\n.fa-rotate-left:before,\n.fa-undo:before {\n  content: \"\\f0e2\";\n}\n.fa-legal:before,\n.fa-gavel:before {\n  content: \"\\f0e3\";\n}\n.fa-dashboard:before,\n.fa-tachometer:before {\n  content: \"\\f0e4\";\n}\n.fa-comment-o:before {\n  content: \"\\f0e5\";\n}\n.fa-comments-o:before {\n  content: \"\\f0e6\";\n}\n.fa-flash:before,\n.fa-bolt:before {\n  content: \"\\f0e7\";\n}\n.fa-sitemap:before {\n  content: \"\\f0e8\";\n}\n.fa-umbrella:before {\n  content: \"\\f0e9\";\n}\n.fa-paste:before,\n.fa-clipboard:before {\n  content: \"\\f0ea\";\n}\n.fa-lightbulb-o:before {\n  content: \"\\f0eb\";\n}\n.fa-exchange:before {\n  content: \"\\f0ec\";\n}\n.fa-cloud-download:before {\n  content: \"\\f0ed\";\n}\n.fa-cloud-upload:before {\n  content: \"\\f0ee\";\n}\n.fa-user-md:before {\n  content: \"\\f0f0\";\n}\n.fa-stethoscope:before {\n  content: \"\\f0f1\";\n}\n.fa-suitcase:before {\n  content: \"\\f0f2\";\n}\n.fa-bell-o:before {\n  content: \"\\f0a2\";\n}\n.fa-coffee:before {\n  content: \"\\f0f4\";\n}\n.fa-cutlery:before {\n  content: \"\\f0f5\";\n}\n.fa-file-text-o:before {\n  content: \"\\f0f6\";\n}\n.fa-building-o:before {\n  content: \"\\f0f7\";\n}\n.fa-hospital-o:before {\n  content: \"\\f0f8\";\n}\n.fa-ambulance:before {\n  content: \"\\f0f9\";\n}\n.fa-medkit:before {\n  content: \"\\f0fa\";\n}\n.fa-fighter-jet:before {\n  content: \"\\f0fb\";\n}\n.fa-beer:before {\n  content: \"\\f0fc\";\n}\n.fa-h-square:before {\n  content: \"\\f0fd\";\n}\n.fa-plus-square:before {\n  content: \"\\f0fe\";\n}\n.fa-angle-double-left:before {\n  content: \"\\f100\";\n}\n.fa-angle-double-right:before {\n  content: \"\\f101\";\n}\n.fa-angle-double-up:before {\n  content: \"\\f102\";\n}\n.fa-angle-double-down:before {\n  content: \"\\f103\";\n}\n.fa-angle-left:before {\n  content: \"\\f104\";\n}\n.fa-angle-right:before {\n  content: \"\\f105\";\n}\n.fa-angle-up:before {\n  content: \"\\f106\";\n}\n.fa-angle-down:before {\n  content: \"\\f107\";\n}\n.fa-desktop:before {\n  content: \"\\f108\";\n}\n.fa-laptop:before {\n  content: \"\\f109\";\n}\n.fa-tablet:before {\n  content: \"\\f10a\";\n}\n.fa-mobile-phone:before,\n.fa-mobile:before {\n  content: \"\\f10b\";\n}\n.fa-circle-o:before {\n  content: \"\\f10c\";\n}\n.fa-quote-left:before {\n  content: \"\\f10d\";\n}\n.fa-quote-right:before {\n  content: \"\\f10e\";\n}\n.fa-spinner:before {\n  content: \"\\f110\";\n}\n.fa-circle:before {\n  content: \"\\f111\";\n}\n.fa-mail-reply:before,\n.fa-reply:before {\n  content: \"\\f112\";\n}\n.fa-github-alt:before {\n  content: \"\\f113\";\n}\n.fa-folder-o:before {\n  content: \"\\f114\";\n}\n.fa-folder-open-o:before {\n  content: \"\\f115\";\n}\n.fa-smile-o:before {\n  content: \"\\f118\";\n}\n.fa-frown-o:before {\n  content: \"\\f119\";\n}\n.fa-meh-o:before {\n  content: \"\\f11a\";\n}\n.fa-gamepad:before {\n  content: \"\\f11b\";\n}\n.fa-keyboard-o:before {\n  content: \"\\f11c\";\n}\n.fa-flag-o:before {\n  content: \"\\f11d\";\n}\n.fa-flag-checkered:before {\n  content: \"\\f11e\";\n}\n.fa-terminal:before {\n  content: \"\\f120\";\n}\n.fa-code:before {\n  content: \"\\f121\";\n}\n.fa-mail-reply-all:before,\n.fa-reply-all:before {\n  content: \"\\f122\";\n}\n.fa-star-half-empty:before,\n.fa-star-half-full:before,\n.fa-star-half-o:before {\n  content: \"\\f123\";\n}\n.fa-location-arrow:before {\n  content: \"\\f124\";\n}\n.fa-crop:before {\n  content: \"\\f125\";\n}\n.fa-code-fork:before {\n  content: \"\\f126\";\n}\n.fa-unlink:before,\n.fa-chain-broken:before {\n  content: \"\\f127\";\n}\n.fa-question:before {\n  content: \"\\f128\";\n}\n.fa-info:before {\n  content: \"\\f129\";\n}\n.fa-exclamation:before {\n  content: \"\\f12a\";\n}\n.fa-superscript:before {\n  content: \"\\f12b\";\n}\n.fa-subscript:before {\n  content: \"\\f12c\";\n}\n.fa-eraser:before {\n  content: \"\\f12d\";\n}\n.fa-puzzle-piece:before {\n  content: \"\\f12e\";\n}\n.fa-microphone:before {\n  content: \"\\f130\";\n}\n.fa-microphone-slash:before {\n  content: \"\\f131\";\n}\n.fa-shield:before {\n  content: \"\\f132\";\n}\n.fa-calendar-o:before {\n  content: \"\\f133\";\n}\n.fa-fire-extinguisher:before {\n  content: \"\\f134\";\n}\n.fa-rocket:before {\n  content: \"\\f135\";\n}\n.fa-maxcdn:before {\n  content: \"\\f136\";\n}\n.fa-chevron-circle-left:before {\n  content: \"\\f137\";\n}\n.fa-chevron-circle-right:before {\n  content: \"\\f138\";\n}\n.fa-chevron-circle-up:before {\n  content: \"\\f139\";\n}\n.fa-chevron-circle-down:before {\n  content: \"\\f13a\";\n}\n.fa-html5:before {\n  content: \"\\f13b\";\n}\n.fa-css3:before {\n  content: \"\\f13c\";\n}\n.fa-anchor:before {\n  content: \"\\f13d\";\n}\n.fa-unlock-alt:before {\n  content: \"\\f13e\";\n}\n.fa-bullseye:before {\n  content: \"\\f140\";\n}\n.fa-ellipsis-h:before {\n  content: \"\\f141\";\n}\n.fa-ellipsis-v:before {\n  content: \"\\f142\";\n}\n.fa-rss-square:before {\n  content: \"\\f143\";\n}\n.fa-play-circle:before {\n  content: \"\\f144\";\n}\n.fa-ticket:before {\n  content: \"\\f145\";\n}\n.fa-minus-square:before {\n  content: \"\\f146\";\n}\n.fa-minus-square-o:before {\n  content: \"\\f147\";\n}\n.fa-level-up:before {\n  content: \"\\f148\";\n}\n.fa-level-down:before {\n  content: \"\\f149\";\n}\n.fa-check-square:before {\n  content: \"\\f14a\";\n}\n.fa-pencil-square:before {\n  content: \"\\f14b\";\n}\n.fa-external-link-square:before {\n  content: \"\\f14c\";\n}\n.fa-share-square:before {\n  content: \"\\f14d\";\n}\n.fa-compass:before {\n  content: \"\\f14e\";\n}\n.fa-toggle-down:before,\n.fa-caret-square-o-down:before {\n  content: \"\\f150\";\n}\n.fa-toggle-up:before,\n.fa-caret-square-o-up:before {\n  content: \"\\f151\";\n}\n.fa-toggle-right:before,\n.fa-caret-square-o-right:before {\n  content: \"\\f152\";\n}\n.fa-euro:before,\n.fa-eur:before {\n  content: \"\\f153\";\n}\n.fa-gbp:before {\n  content: \"\\f154\";\n}\n.fa-dollar:before,\n.fa-usd:before {\n  content: \"\\f155\";\n}\n.fa-rupee:before,\n.fa-inr:before {\n  content: \"\\f156\";\n}\n.fa-cny:before,\n.fa-rmb:before,\n.fa-yen:before,\n.fa-jpy:before {\n  content: \"\\f157\";\n}\n.fa-ruble:before,\n.fa-rouble:before,\n.fa-rub:before {\n  content: \"\\f158\";\n}\n.fa-won:before,\n.fa-krw:before {\n  content: \"\\f159\";\n}\n.fa-bitcoin:before,\n.fa-btc:before {\n  content: \"\\f15a\";\n}\n.fa-file:before {\n  content: \"\\f15b\";\n}\n.fa-file-text:before {\n  content: \"\\f15c\";\n}\n.fa-sort-alpha-asc:before {\n  content: \"\\f15d\";\n}\n.fa-sort-alpha-desc:before {\n  content: \"\\f15e\";\n}\n.fa-sort-amount-asc:before {\n  content: \"\\f160\";\n}\n.fa-sort-amount-desc:before {\n  content: \"\\f161\";\n}\n.fa-sort-numeric-asc:before {\n  content: \"\\f162\";\n}\n.fa-sort-numeric-desc:before {\n  content: \"\\f163\";\n}\n.fa-thumbs-up:before {\n  content: \"\\f164\";\n}\n.fa-thumbs-down:before {\n  content: \"\\f165\";\n}\n.fa-youtube-square:before {\n  content: \"\\f166\";\n}\n.fa-youtube:before {\n  content: \"\\f167\";\n}\n.fa-xing:before {\n  content: \"\\f168\";\n}\n.fa-xing-square:before {\n  content: \"\\f169\";\n}\n.fa-youtube-play:before {\n  content: \"\\f16a\";\n}\n.fa-dropbox:before {\n  content: \"\\f16b\";\n}\n.fa-stack-overflow:before {\n  content: \"\\f16c\";\n}\n.fa-instagram:before {\n  content: \"\\f16d\";\n}\n.fa-flickr:before {\n  content: \"\\f16e\";\n}\n.fa-adn:before {\n  content: \"\\f170\";\n}\n.fa-bitbucket:before {\n  content: \"\\f171\";\n}\n.fa-bitbucket-square:before {\n  content: \"\\f172\";\n}\n.fa-tumblr:before {\n  content: \"\\f173\";\n}\n.fa-tumblr-square:before {\n  content: \"\\f174\";\n}\n.fa-long-arrow-down:before {\n  content: \"\\f175\";\n}\n.fa-long-arrow-up:before {\n  content: \"\\f176\";\n}\n.fa-long-arrow-left:before {\n  content: \"\\f177\";\n}\n.fa-long-arrow-right:before {\n  content: \"\\f178\";\n}\n.fa-apple:before {\n  content: \"\\f179\";\n}\n.fa-windows:before {\n  content: \"\\f17a\";\n}\n.fa-android:before {\n  content: \"\\f17b\";\n}\n.fa-linux:before {\n  content: \"\\f17c\";\n}\n.fa-dribbble:before {\n  content: \"\\f17d\";\n}\n.fa-skype:before {\n  content: \"\\f17e\";\n}\n.fa-foursquare:before {\n  content: \"\\f180\";\n}\n.fa-trello:before {\n  content: \"\\f181\";\n}\n.fa-female:before {\n  content: \"\\f182\";\n}\n.fa-male:before {\n  content: \"\\f183\";\n}\n.fa-gittip:before,\n.fa-gratipay:before {\n  content: \"\\f184\";\n}\n.fa-sun-o:before {\n  content: \"\\f185\";\n}\n.fa-moon-o:before {\n  content: \"\\f186\";\n}\n.fa-archive:before {\n  content: \"\\f187\";\n}\n.fa-bug:before {\n  content: \"\\f188\";\n}\n.fa-vk:before {\n  content: \"\\f189\";\n}\n.fa-weibo:before {\n  content: \"\\f18a\";\n}\n.fa-renren:before {\n  content: \"\\f18b\";\n}\n.fa-pagelines:before {\n  content: \"\\f18c\";\n}\n.fa-stack-exchange:before {\n  content: \"\\f18d\";\n}\n.fa-arrow-circle-o-right:before {\n  content: \"\\f18e\";\n}\n.fa-arrow-circle-o-left:before {\n  content: \"\\f190\";\n}\n.fa-toggle-left:before,\n.fa-caret-square-o-left:before {\n  content: \"\\f191\";\n}\n.fa-dot-circle-o:before {\n  content: \"\\f192\";\n}\n.fa-wheelchair:before {\n  content: \"\\f193\";\n}\n.fa-vimeo-square:before {\n  content: \"\\f194\";\n}\n.fa-turkish-lira:before,\n.fa-try:before {\n  content: \"\\f195\";\n}\n.fa-plus-square-o:before {\n  content: \"\\f196\";\n}\n.fa-space-shuttle:before {\n  content: \"\\f197\";\n}\n.fa-slack:before {\n  content: \"\\f198\";\n}\n.fa-envelope-square:before {\n  content: \"\\f199\";\n}\n.fa-wordpress:before {\n  content: \"\\f19a\";\n}\n.fa-openid:before {\n  content: \"\\f19b\";\n}\n.fa-institution:before,\n.fa-bank:before,\n.fa-university:before {\n  content: \"\\f19c\";\n}\n.fa-mortar-board:before,\n.fa-graduation-cap:before {\n  content: \"\\f19d\";\n}\n.fa-yahoo:before {\n  content: \"\\f19e\";\n}\n.fa-google:before {\n  content: \"\\f1a0\";\n}\n.fa-reddit:before {\n  content: \"\\f1a1\";\n}\n.fa-reddit-square:before {\n  content: \"\\f1a2\";\n}\n.fa-stumbleupon-circle:before {\n  content: \"\\f1a3\";\n}\n.fa-stumbleupon:before {\n  content: \"\\f1a4\";\n}\n.fa-delicious:before {\n  content: \"\\f1a5\";\n}\n.fa-digg:before {\n  content: \"\\f1a6\";\n}\n.fa-pied-piper-pp:before {\n  content: \"\\f1a7\";\n}\n.fa-pied-piper-alt:before {\n  content: \"\\f1a8\";\n}\n.fa-drupal:before {\n  content: \"\\f1a9\";\n}\n.fa-joomla:before {\n  content: \"\\f1aa\";\n}\n.fa-language:before {\n  content: \"\\f1ab\";\n}\n.fa-fax:before {\n  content: \"\\f1ac\";\n}\n.fa-building:before {\n  content: \"\\f1ad\";\n}\n.fa-child:before {\n  content: \"\\f1ae\";\n}\n.fa-paw:before {\n  content: \"\\f1b0\";\n}\n.fa-spoon:before {\n  content: \"\\f1b1\";\n}\n.fa-cube:before {\n  content: \"\\f1b2\";\n}\n.fa-cubes:before {\n  content: \"\\f1b3\";\n}\n.fa-behance:before {\n  content: \"\\f1b4\";\n}\n.fa-behance-square:before {\n  content: \"\\f1b5\";\n}\n.fa-steam:before {\n  content: \"\\f1b6\";\n}\n.fa-steam-square:before {\n  content: \"\\f1b7\";\n}\n.fa-recycle:before {\n  content: \"\\f1b8\";\n}\n.fa-automobile:before,\n.fa-car:before {\n  content: \"\\f1b9\";\n}\n.fa-cab:before,\n.fa-taxi:before {\n  content: \"\\f1ba\";\n}\n.fa-tree:before {\n  content: \"\\f1bb\";\n}\n.fa-spotify:before {\n  content: \"\\f1bc\";\n}\n.fa-deviantart:before {\n  content: \"\\f1bd\";\n}\n.fa-soundcloud:before {\n  content: \"\\f1be\";\n}\n.fa-database:before {\n  content: \"\\f1c0\";\n}\n.fa-file-pdf-o:before {\n  content: \"\\f1c1\";\n}\n.fa-file-word-o:before {\n  content: \"\\f1c2\";\n}\n.fa-file-excel-o:before {\n  content: \"\\f1c3\";\n}\n.fa-file-powerpoint-o:before {\n  content: \"\\f1c4\";\n}\n.fa-file-photo-o:before,\n.fa-file-picture-o:before,\n.fa-file-image-o:before {\n  content: \"\\f1c5\";\n}\n.fa-file-zip-o:before,\n.fa-file-archive-o:before {\n  content: \"\\f1c6\";\n}\n.fa-file-sound-o:before,\n.fa-file-audio-o:before {\n  content: \"\\f1c7\";\n}\n.fa-file-movie-o:before,\n.fa-file-video-o:before {\n  content: \"\\f1c8\";\n}\n.fa-file-code-o:before {\n  content: \"\\f1c9\";\n}\n.fa-vine:before {\n  content: \"\\f1ca\";\n}\n.fa-codepen:before {\n  content: \"\\f1cb\";\n}\n.fa-jsfiddle:before {\n  content: \"\\f1cc\";\n}\n.fa-life-bouy:before,\n.fa-life-buoy:before,\n.fa-life-saver:before,\n.fa-support:before,\n.fa-life-ring:before {\n  content: \"\\f1cd\";\n}\n.fa-circle-o-notch:before {\n  content: \"\\f1ce\";\n}\n.fa-ra:before,\n.fa-resistance:before,\n.fa-rebel:before {\n  content: \"\\f1d0\";\n}\n.fa-ge:before,\n.fa-empire:before {\n  content: \"\\f1d1\";\n}\n.fa-git-square:before {\n  content: \"\\f1d2\";\n}\n.fa-git:before {\n  content: \"\\f1d3\";\n}\n.fa-y-combinator-square:before,\n.fa-yc-square:before,\n.fa-hacker-news:before {\n  content: \"\\f1d4\";\n}\n.fa-tencent-weibo:before {\n  content: \"\\f1d5\";\n}\n.fa-qq:before {\n  content: \"\\f1d6\";\n}\n.fa-wechat:before,\n.fa-weixin:before {\n  content: \"\\f1d7\";\n}\n.fa-send:before,\n.fa-paper-plane:before {\n  content: \"\\f1d8\";\n}\n.fa-send-o:before,\n.fa-paper-plane-o:before {\n  content: \"\\f1d9\";\n}\n.fa-history:before {\n  content: \"\\f1da\";\n}\n.fa-circle-thin:before {\n  content: \"\\f1db\";\n}\n.fa-header:before {\n  content: \"\\f1dc\";\n}\n.fa-paragraph:before {\n  content: \"\\f1dd\";\n}\n.fa-sliders:before {\n  content: \"\\f1de\";\n}\n.fa-share-alt:before {\n  content: \"\\f1e0\";\n}\n.fa-share-alt-square:before {\n  content: \"\\f1e1\";\n}\n.fa-bomb:before {\n  content: \"\\f1e2\";\n}\n.fa-soccer-ball-o:before,\n.fa-futbol-o:before {\n  content: \"\\f1e3\";\n}\n.fa-tty:before {\n  content: \"\\f1e4\";\n}\n.fa-binoculars:before {\n  content: \"\\f1e5\";\n}\n.fa-plug:before {\n  content: \"\\f1e6\";\n}\n.fa-slideshare:before {\n  content: \"\\f1e7\";\n}\n.fa-twitch:before {\n  content: \"\\f1e8\";\n}\n.fa-yelp:before {\n  content: \"\\f1e9\";\n}\n.fa-newspaper-o:before {\n  content: \"\\f1ea\";\n}\n.fa-wifi:before {\n  content: \"\\f1eb\";\n}\n.fa-calculator:before {\n  content: \"\\f1ec\";\n}\n.fa-paypal:before {\n  content: \"\\f1ed\";\n}\n.fa-google-wallet:before {\n  content: \"\\f1ee\";\n}\n.fa-cc-visa:before {\n  content: \"\\f1f0\";\n}\n.fa-cc-mastercard:before {\n  content: \"\\f1f1\";\n}\n.fa-cc-discover:before {\n  content: \"\\f1f2\";\n}\n.fa-cc-amex:before {\n  content: \"\\f1f3\";\n}\n.fa-cc-paypal:before {\n  content: \"\\f1f4\";\n}\n.fa-cc-stripe:before {\n  content: \"\\f1f5\";\n}\n.fa-bell-slash:before {\n  content: \"\\f1f6\";\n}\n.fa-bell-slash-o:before {\n  content: \"\\f1f7\";\n}\n.fa-trash:before {\n  content: \"\\f1f8\";\n}\n.fa-copyright:before {\n  content: \"\\f1f9\";\n}\n.fa-at:before {\n  content: \"\\f1fa\";\n}\n.fa-eyedropper:before {\n  content: \"\\f1fb\";\n}\n.fa-paint-brush:before {\n  content: \"\\f1fc\";\n}\n.fa-birthday-cake:before {\n  content: \"\\f1fd\";\n}\n.fa-area-chart:before {\n  content: \"\\f1fe\";\n}\n.fa-pie-chart:before {\n  content: \"\\f200\";\n}\n.fa-line-chart:before {\n  content: \"\\f201\";\n}\n.fa-lastfm:before {\n  content: \"\\f202\";\n}\n.fa-lastfm-square:before {\n  content: \"\\f203\";\n}\n.fa-toggle-off:before {\n  content: \"\\f204\";\n}\n.fa-toggle-on:before {\n  content: \"\\f205\";\n}\n.fa-bicycle:before {\n  content: \"\\f206\";\n}\n.fa-bus:before {\n  content: \"\\f207\";\n}\n.fa-ioxhost:before {\n  content: \"\\f208\";\n}\n.fa-angellist:before {\n  content: \"\\f209\";\n}\n.fa-cc:before {\n  content: \"\\f20a\";\n}\n.fa-shekel:before,\n.fa-sheqel:before,\n.fa-ils:before {\n  content: \"\\f20b\";\n}\n.fa-meanpath:before {\n  content: \"\\f20c\";\n}\n.fa-buysellads:before {\n  content: \"\\f20d\";\n}\n.fa-connectdevelop:before {\n  content: \"\\f20e\";\n}\n.fa-dashcube:before {\n  content: \"\\f210\";\n}\n.fa-forumbee:before {\n  content: \"\\f211\";\n}\n.fa-leanpub:before {\n  content: \"\\f212\";\n}\n.fa-sellsy:before {\n  content: \"\\f213\";\n}\n.fa-shirtsinbulk:before {\n  content: \"\\f214\";\n}\n.fa-simplybuilt:before {\n  content: \"\\f215\";\n}\n.fa-skyatlas:before {\n  content: \"\\f216\";\n}\n.fa-cart-plus:before {\n  content: \"\\f217\";\n}\n.fa-cart-arrow-down:before {\n  content: \"\\f218\";\n}\n.fa-diamond:before {\n  content: \"\\f219\";\n}\n.fa-ship:before {\n  content: \"\\f21a\";\n}\n.fa-user-secret:before {\n  content: \"\\f21b\";\n}\n.fa-motorcycle:before {\n  content: \"\\f21c\";\n}\n.fa-street-view:before {\n  content: \"\\f21d\";\n}\n.fa-heartbeat:before {\n  content: \"\\f21e\";\n}\n.fa-venus:before {\n  content: \"\\f221\";\n}\n.fa-mars:before {\n  content: \"\\f222\";\n}\n.fa-mercury:before {\n  content: \"\\f223\";\n}\n.fa-intersex:before,\n.fa-transgender:before {\n  content: \"\\f224\";\n}\n.fa-transgender-alt:before {\n  content: \"\\f225\";\n}\n.fa-venus-double:before {\n  content: \"\\f226\";\n}\n.fa-mars-double:before {\n  content: \"\\f227\";\n}\n.fa-venus-mars:before {\n  content: \"\\f228\";\n}\n.fa-mars-stroke:before {\n  content: \"\\f229\";\n}\n.fa-mars-stroke-v:before {\n  content: \"\\f22a\";\n}\n.fa-mars-stroke-h:before {\n  content: \"\\f22b\";\n}\n.fa-neuter:before {\n  content: \"\\f22c\";\n}\n.fa-genderless:before {\n  content: \"\\f22d\";\n}\n.fa-facebook-official:before {\n  content: \"\\f230\";\n}\n.fa-pinterest-p:before {\n  content: \"\\f231\";\n}\n.fa-whatsapp:before {\n  content: \"\\f232\";\n}\n.fa-server:before {\n  content: \"\\f233\";\n}\n.fa-user-plus:before {\n  content: \"\\f234\";\n}\n.fa-user-times:before {\n  content: \"\\f235\";\n}\n.fa-hotel:before,\n.fa-bed:before {\n  content: \"\\f236\";\n}\n.fa-viacoin:before {\n  content: \"\\f237\";\n}\n.fa-train:before {\n  content: \"\\f238\";\n}\n.fa-subway:before {\n  content: \"\\f239\";\n}\n.fa-medium:before {\n  content: \"\\f23a\";\n}\n.fa-yc:before,\n.fa-y-combinator:before {\n  content: \"\\f23b\";\n}\n.fa-optin-monster:before {\n  content: \"\\f23c\";\n}\n.fa-opencart:before {\n  content: \"\\f23d\";\n}\n.fa-expeditedssl:before {\n  content: \"\\f23e\";\n}\n.fa-battery-4:before,\n.fa-battery:before,\n.fa-battery-full:before {\n  content: \"\\f240\";\n}\n.fa-battery-3:before,\n.fa-battery-three-quarters:before {\n  content: \"\\f241\";\n}\n.fa-battery-2:before,\n.fa-battery-half:before {\n  content: \"\\f242\";\n}\n.fa-battery-1:before,\n.fa-battery-quarter:before {\n  content: \"\\f243\";\n}\n.fa-battery-0:before,\n.fa-battery-empty:before {\n  content: \"\\f244\";\n}\n.fa-mouse-pointer:before {\n  content: \"\\f245\";\n}\n.fa-i-cursor:before {\n  content: \"\\f246\";\n}\n.fa-object-group:before {\n  content: \"\\f247\";\n}\n.fa-object-ungroup:before {\n  content: \"\\f248\";\n}\n.fa-sticky-note:before {\n  content: \"\\f249\";\n}\n.fa-sticky-note-o:before {\n  content: \"\\f24a\";\n}\n.fa-cc-jcb:before {\n  content: \"\\f24b\";\n}\n.fa-cc-diners-club:before {\n  content: \"\\f24c\";\n}\n.fa-clone:before {\n  content: \"\\f24d\";\n}\n.fa-balance-scale:before {\n  content: \"\\f24e\";\n}\n.fa-hourglass-o:before {\n  content: \"\\f250\";\n}\n.fa-hourglass-1:before,\n.fa-hourglass-start:before {\n  content: \"\\f251\";\n}\n.fa-hourglass-2:before,\n.fa-hourglass-half:before {\n  content: \"\\f252\";\n}\n.fa-hourglass-3:before,\n.fa-hourglass-end:before {\n  content: \"\\f253\";\n}\n.fa-hourglass:before {\n  content: \"\\f254\";\n}\n.fa-hand-grab-o:before,\n.fa-hand-rock-o:before {\n  content: \"\\f255\";\n}\n.fa-hand-stop-o:before,\n.fa-hand-paper-o:before {\n  content: \"\\f256\";\n}\n.fa-hand-scissors-o:before {\n  content: \"\\f257\";\n}\n.fa-hand-lizard-o:before {\n  content: \"\\f258\";\n}\n.fa-hand-spock-o:before {\n  content: \"\\f259\";\n}\n.fa-hand-pointer-o:before {\n  content: \"\\f25a\";\n}\n.fa-hand-peace-o:before {\n  content: \"\\f25b\";\n}\n.fa-trademark:before {\n  content: \"\\f25c\";\n}\n.fa-registered:before {\n  content: \"\\f25d\";\n}\n.fa-creative-commons:before {\n  content: \"\\f25e\";\n}\n.fa-gg:before {\n  content: \"\\f260\";\n}\n.fa-gg-circle:before {\n  content: \"\\f261\";\n}\n.fa-tripadvisor:before {\n  content: \"\\f262\";\n}\n.fa-odnoklassniki:before {\n  content: \"\\f263\";\n}\n.fa-odnoklassniki-square:before {\n  content: \"\\f264\";\n}\n.fa-get-pocket:before {\n  content: \"\\f265\";\n}\n.fa-wikipedia-w:before {\n  content: \"\\f266\";\n}\n.fa-safari:before {\n  content: \"\\f267\";\n}\n.fa-chrome:before {\n  content: \"\\f268\";\n}\n.fa-firefox:before {\n  content: \"\\f269\";\n}\n.fa-opera:before {\n  content: \"\\f26a\";\n}\n.fa-internet-explorer:before {\n  content: \"\\f26b\";\n}\n.fa-tv:before,\n.fa-television:before {\n  content: \"\\f26c\";\n}\n.fa-contao:before {\n  content: \"\\f26d\";\n}\n.fa-500px:before {\n  content: \"\\f26e\";\n}\n.fa-amazon:before {\n  content: \"\\f270\";\n}\n.fa-calendar-plus-o:before {\n  content: \"\\f271\";\n}\n.fa-calendar-minus-o:before {\n  content: \"\\f272\";\n}\n.fa-calendar-times-o:before {\n  content: \"\\f273\";\n}\n.fa-calendar-check-o:before {\n  content: \"\\f274\";\n}\n.fa-industry:before {\n  content: \"\\f275\";\n}\n.fa-map-pin:before {\n  content: \"\\f276\";\n}\n.fa-map-signs:before {\n  content: \"\\f277\";\n}\n.fa-map-o:before {\n  content: \"\\f278\";\n}\n.fa-map:before {\n  content: \"\\f279\";\n}\n.fa-commenting:before {\n  content: \"\\f27a\";\n}\n.fa-commenting-o:before {\n  content: \"\\f27b\";\n}\n.fa-houzz:before {\n  content: \"\\f27c\";\n}\n.fa-vimeo:before {\n  content: \"\\f27d\";\n}\n.fa-black-tie:before {\n  content: \"\\f27e\";\n}\n.fa-fonticons:before {\n  content: \"\\f280\";\n}\n.fa-reddit-alien:before {\n  content: \"\\f281\";\n}\n.fa-edge:before {\n  content: \"\\f282\";\n}\n.fa-credit-card-alt:before {\n  content: \"\\f283\";\n}\n.fa-codiepie:before {\n  content: \"\\f284\";\n}\n.fa-modx:before {\n  content: \"\\f285\";\n}\n.fa-fort-awesome:before {\n  content: \"\\f286\";\n}\n.fa-usb:before {\n  content: \"\\f287\";\n}\n.fa-product-hunt:before {\n  content: \"\\f288\";\n}\n.fa-mixcloud:before {\n  content: \"\\f289\";\n}\n.fa-scribd:before {\n  content: \"\\f28a\";\n}\n.fa-pause-circle:before {\n  content: \"\\f28b\";\n}\n.fa-pause-circle-o:before {\n  content: \"\\f28c\";\n}\n.fa-stop-circle:before {\n  content: \"\\f28d\";\n}\n.fa-stop-circle-o:before {\n  content: \"\\f28e\";\n}\n.fa-shopping-bag:before {\n  content: \"\\f290\";\n}\n.fa-shopping-basket:before {\n  content: \"\\f291\";\n}\n.fa-hashtag:before {\n  content: \"\\f292\";\n}\n.fa-bluetooth:before {\n  content: \"\\f293\";\n}\n.fa-bluetooth-b:before {\n  content: \"\\f294\";\n}\n.fa-percent:before {\n  content: \"\\f295\";\n}\n.fa-gitlab:before {\n  content: \"\\f296\";\n}\n.fa-wpbeginner:before {\n  content: \"\\f297\";\n}\n.fa-wpforms:before {\n  content: \"\\f298\";\n}\n.fa-envira:before {\n  content: \"\\f299\";\n}\n.fa-universal-access:before {\n  content: \"\\f29a\";\n}\n.fa-wheelchair-alt:before {\n  content: \"\\f29b\";\n}\n.fa-question-circle-o:before {\n  content: \"\\f29c\";\n}\n.fa-blind:before {\n  content: \"\\f29d\";\n}\n.fa-audio-description:before {\n  content: \"\\f29e\";\n}\n.fa-volume-control-phone:before {\n  content: \"\\f2a0\";\n}\n.fa-braille:before {\n  content: \"\\f2a1\";\n}\n.fa-assistive-listening-systems:before {\n  content: \"\\f2a2\";\n}\n.fa-asl-interpreting:before,\n.fa-american-sign-language-interpreting:before {\n  content: \"\\f2a3\";\n}\n.fa-deafness:before,\n.fa-hard-of-hearing:before,\n.fa-deaf:before {\n  content: \"\\f2a4\";\n}\n.fa-glide:before {\n  content: \"\\f2a5\";\n}\n.fa-glide-g:before {\n  content: \"\\f2a6\";\n}\n.fa-signing:before,\n.fa-sign-language:before {\n  content: \"\\f2a7\";\n}\n.fa-low-vision:before {\n  content: \"\\f2a8\";\n}\n.fa-viadeo:before {\n  content: \"\\f2a9\";\n}\n.fa-viadeo-square:before {\n  content: \"\\f2aa\";\n}\n.fa-snapchat:before {\n  content: \"\\f2ab\";\n}\n.fa-snapchat-ghost:before {\n  content: \"\\f2ac\";\n}\n.fa-snapchat-square:before {\n  content: \"\\f2ad\";\n}\n.fa-pied-piper:before {\n  content: \"\\f2ae\";\n}\n.fa-first-order:before {\n  content: \"\\f2b0\";\n}\n.fa-yoast:before {\n  content: \"\\f2b1\";\n}\n.fa-themeisle:before {\n  content: \"\\f2b2\";\n}\n.fa-google-plus-circle:before,\n.fa-google-plus-official:before {\n  content: \"\\f2b3\";\n}\n.fa-fa:before,\n.fa-font-awesome:before {\n  content: \"\\f2b4\";\n}\n.fa-handshake-o:before {\n  content: \"\\f2b5\";\n}\n.fa-envelope-open:before {\n  content: \"\\f2b6\";\n}\n.fa-envelope-open-o:before {\n  content: \"\\f2b7\";\n}\n.fa-linode:before {\n  content: \"\\f2b8\";\n}\n.fa-address-book:before {\n  content: \"\\f2b9\";\n}\n.fa-address-book-o:before {\n  content: \"\\f2ba\";\n}\n.fa-vcard:before,\n.fa-address-card:before {\n  content: \"\\f2bb\";\n}\n.fa-vcard-o:before,\n.fa-address-card-o:before {\n  content: \"\\f2bc\";\n}\n.fa-user-circle:before {\n  content: \"\\f2bd\";\n}\n.fa-user-circle-o:before {\n  content: \"\\f2be\";\n}\n.fa-user-o:before {\n  content: \"\\f2c0\";\n}\n.fa-id-badge:before {\n  content: \"\\f2c1\";\n}\n.fa-drivers-license:before,\n.fa-id-card:before {\n  content: \"\\f2c2\";\n}\n.fa-drivers-license-o:before,\n.fa-id-card-o:before {\n  content: \"\\f2c3\";\n}\n.fa-quora:before {\n  content: \"\\f2c4\";\n}\n.fa-free-code-camp:before {\n  content: \"\\f2c5\";\n}\n.fa-telegram:before {\n  content: \"\\f2c6\";\n}\n.fa-thermometer-4:before,\n.fa-thermometer:before,\n.fa-thermometer-full:before {\n  content: \"\\f2c7\";\n}\n.fa-thermometer-3:before,\n.fa-thermometer-three-quarters:before {\n  content: \"\\f2c8\";\n}\n.fa-thermometer-2:before,\n.fa-thermometer-half:before {\n  content: \"\\f2c9\";\n}\n.fa-thermometer-1:before,\n.fa-thermometer-quarter:before {\n  content: \"\\f2ca\";\n}\n.fa-thermometer-0:before,\n.fa-thermometer-empty:before {\n  content: \"\\f2cb\";\n}\n.fa-shower:before {\n  content: \"\\f2cc\";\n}\n.fa-bathtub:before,\n.fa-s15:before,\n.fa-bath:before {\n  content: \"\\f2cd\";\n}\n.fa-podcast:before {\n  content: \"\\f2ce\";\n}\n.fa-window-maximize:before {\n  content: \"\\f2d0\";\n}\n.fa-window-minimize:before {\n  content: \"\\f2d1\";\n}\n.fa-window-restore:before {\n  content: \"\\f2d2\";\n}\n.fa-times-rectangle:before,\n.fa-window-close:before {\n  content: \"\\f2d3\";\n}\n.fa-times-rectangle-o:before,\n.fa-window-close-o:before {\n  content: \"\\f2d4\";\n}\n.fa-bandcamp:before {\n  content: \"\\f2d5\";\n}\n.fa-grav:before {\n  content: \"\\f2d6\";\n}\n.fa-etsy:before {\n  content: \"\\f2d7\";\n}\n.fa-imdb:before {\n  content: \"\\f2d8\";\n}\n.fa-ravelry:before {\n  content: \"\\f2d9\";\n}\n.fa-eercast:before {\n  content: \"\\f2da\";\n}\n.fa-microchip:before {\n  content: \"\\f2db\";\n}\n.fa-snowflake-o:before {\n  content: \"\\f2dc\";\n}\n.fa-superpowers:before {\n  content: \"\\f2dd\";\n}\n.fa-wpexplorer:before {\n  content: \"\\f2de\";\n}\n.fa-meetup:before {\n  content: \"\\f2e0\";\n}\n.sr-only {\n  position: absolute;\n  width: 1px;\n  height: 1px;\n  padding: 0;\n  margin: -1px;\n  overflow: hidden;\n  clip: rect(0, 0, 0, 0);\n  border: 0;\n}\n.sr-only-focusable:active,\n.sr-only-focusable:focus {\n  position: static;\n  width: auto;\n  height: auto;\n  margin: 0;\n  overflow: visible;\n  clip: auto;\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n\n\n#main {\n    & > .page {\n        &.orchestrator {\n            display: flex;\n            flex-direction: column;\n\n            & > .header {\n                display: flex;\n                align-items: center;\n                justify-content: flex-end;\n                padding: 0.2rem;\n\n                & > * {\n                    &:not(:last-child) {\n                        margin-right: 0.2rem;\n                    }\n                }\n            }\n\n            & > .components {\n                flex-grow: 1;\n                overflow: auto;\n\n                table {\n                    @extend %table;\n\n                    th {\n                        &.col-delay { width: 5rem; }\n                        &.col-revive { width: 5rem; }\n                        &.col-status { width: 8rem; }\n                        &.col-action { width: 5rem; }\n                    }\n\n                    td {\n                        &.col-delay { text-align: right; }\n                        &.col-revive { text-align: center; }\n                        &.col-status { text-align: center; }\n\n                        &.col-action {\n                            text-align: center;\n\n                            button {\n                                margin: 0 0.1rem;\n                            }\n                        }\n                    }\n                }\n            }\n        }\n    }\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n\n\n#main {\n    & > .page {\n        &.monitor {\n            display: flex;\n            flex-direction: column;\n\n            & > .header {\n                display: flex;\n                align-items: center;\n                justify-content: flex-end;\n                padding: 0.2rem;\n\n                & > * {\n                    &:not(:last-child) {\n                        margin-right: 0.2rem;\n                    }\n                }\n            }\n\n            & > .components {\n                flex-grow: 1;\n                overflow: auto;\n\n                table {\n                    @extend %table;\n\n                    th {\n                        &.hidden { padding: 0; }\n                        &.col-id { width: 5rem; }\n                        &.col-group { width: 10rem; }\n                        &.col-address { width: 15rem; }\n                        &.col-blessing { width: 5rem; }\n                        &.col-ready { width: 5rem; }\n                        &.col-rank { width: 8rem; }\n                    }\n\n                    td {\n                        &.col-id { text-align: center; }\n                        &.col-blessing { text-align: center; }\n                        &.col-ready { text-align: center; }\n\n                        &.col-rank {\n                            text-align: center;\n\n                            & > div {\n                                display: flex;\n                                justify-content: space-between;\n                                align-items: center;\n\n                                button {\n                                    margin: 0 0.1rem;\n                                }\n                            }\n                        }\n                    }\n                }\n            }\n        }\n    }\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n\n\n#main {\n    & > .page {\n        &.event {\n            display: flex;\n            flex-direction: column;\n\n            & > .header {\n                display: flex;\n                align-items: center;\n                padding: 0.2rem;\n                border-bottom: 1px solid $color-grey-700;\n\n                & > .menu {\n                    display: flex;\n\n                    & > * {\n                        margin-right: 0.2rem;\n                        padding: 0.2rem 0.5rem;\n                        border-radius: 0.2rem;\n\n                        &.selected, &:hover {\n                            color: $color-grey-100;\n                            background-color: $color-grey-700;\n                        }\n                    }\n                }\n\n                & > .spacer {\n                    flex-grow: 1;\n                }\n\n                & > .form {\n                    display: flex;\n                    align-items: center;\n\n                    & > * {\n                        &:not(:last-child) {\n                            margin-right: 0.2rem;\n                        }\n                    }\n                }\n            }\n\n            & > .subpage {\n                flex-grow: 1;\n                overflow: auto;\n\n                &.latest, &.changes {\n                    & > table {\n                        @extend %table;\n\n                        th {\n                            &.col-type { width: 25rem; }\n                            &.col-id { width: 5rem; }\n                            &.col-timestamp { width: 13rem; }\n                        }\n\n                        td {\n                            &.col-id { text-align: right; }\n                            &.col-timestamp { text-align: center; }\n                        }\n\n                        .expand-icon {\n                            padding: 5px;\n                            color: $color-grey-700;\n                        }\n                    }\n                }\n\n                &.register {\n                    padding: 0.5rem;\n                    display: grid;\n                    grid-gap: 0.5rem;\n                    grid-template-columns: 1fr auto;\n                    grid-template-rows: auto auto 1fr;\n                    grid-template-areas: 'text sourceTimestamp'\n                                         'text register'\n                                         'text .';\n\n                    & > .source-timestamp {\n                        grid-area: sourceTimestamp;\n                    }\n\n                    & > .text {\n                        grid-area: text;\n                        resize: none;\n                    }\n\n                    & > .register {\n                        grid-area: register;\n                    }\n                }\n            }\n        }\n    }\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n\n\n#main {\n    & > .page {\n        &.iec104 {\n            display: flex;\n            flex-direction: column;\n\n            & > .properties {\n                padding: 0.2rem;\n                display: grid;\n                grid-template-columns: repeat(auto-fit, 10rem 8rem);\n                grid-gap: 0.2rem;\n                align-items: center;\n\n                & > .label {\n                    justify-self: end;\n                }\n            }\n\n            & > .data, & > .commands {\n                flex-grow: 1;\n                overflow: auto;\n\n                & > table {\n                    @extend %table;\n\n                    th {\n                        &.hidden { padding: 0; }\n                        &.col-timestamp { width: 13rem; }\n                        &.col-str { width: 8rem; }\n                        &.col-int { width: 4rem; }\n                        &.col-short { width: 3rem; }\n                        &.col-long { width: 5rem; }\n                        &.col-bool { width: 3rem; }\n                        &.col-remove { width: 4.5rem; }\n                    }\n\n                    td {\n                        &.col-int, &.col-short, &.col-long { text-align: right; }\n                        &.col-bool { text-align: center; }\n                        &.col-remove { text-align: center; }\n\n                        &.col-timestamp {\n                            font-family: 'RobotoMono';\n                            font-size: 0.9rem;\n                        }\n                    }\n\n                    & > tbody {\n                        & > tr {\n                            &.selected, &:hover {\n                                background-color: $color-grey-400;\n                            }\n                        }\n                    }\n                }\n\n                & > .control {\n                    padding: 0.2rem;\n                    display: flex;\n                    justify-content: flex-end;\n                }\n            }\n\n            &.master {\n                & > .control {\n                    margin: 0.2rem 0;\n\n                    & > .header {\n                        display: flex;\n                        padding: 0.2rem;\n                        border-bottom: 1px solid $color-grey-700;\n\n                        & > * {\n                            margin-right: 0.2rem;\n                            padding: 0.2rem 0.5rem;\n                            border-radius: 0.2rem;\n\n                            &.selected, &:hover {\n                                color: $color-grey-100;\n                                background-color: $color-grey-700;\n                            }\n                        }\n                    }\n\n                    & > .content {\n                        margin: 0.2rem;\n                        display: flex;\n                        align-items: stretch;\n\n                        & > .form {\n                            flex-grow: 1;\n                            display: grid;\n                            grid-template-columns: repeat(auto-fit, 8rem 12rem);\n                            grid-gap: 0.2rem;\n                            align-items: center;\n\n                            & > label {\n                                justify-self: end;\n                            }\n                        }\n\n                        & > button {\n                            margin-left: 0.2rem;\n                        }\n                    }\n                }\n            }\n\n            &.slave {\n                & > .panel {\n                    display: flex;\n                    align-items: stretch;\n                    padding: 0.2rem;\n                    border-top: 1px solid $color-grey-700;\n                    margin-top: 0.2rem;\n\n                    & > .form {\n                        flex-grow: 1;\n                        display: grid;\n                        grid-template-columns: repeat(auto-fit, 12rem 12rem);\n                        grid-gap: 0.2rem;\n                        align-items: center;\n\n                        & > .label {\n                            justify-self: end;\n                        }\n                    }\n\n                    & > button {\n                        margin-left: 0.2rem;\n                    }\n                }\n            }\n        }\n    }\n}\n","@import 'color';\n@import 'shadow';\n@import 'table';\n\n\n#main {\n    & > .page {\n        &.modbus {\n            display: flex;\n            flex-direction: column;\n\n            & > .properties {\n                padding: 0.2rem;\n                display: grid;\n                grid-template-columns: repeat(auto-fit, 10rem 8rem);\n                grid-gap: 0.2rem;\n                align-items: center;\n\n                & > .label {\n                    justify-self: end;\n                }\n            }\n\n            & > .action {\n                border-top: 1px solid $color-grey-700;\n                padding: 0.2rem;\n                display: flex;\n                align-items: stretch;\n\n                & > .form {\n                    flex-grow: 1;\n                    display: grid;\n                    grid-template-columns: repeat(auto-fit, 8rem 12rem);\n                    grid-gap: 0.2rem;\n                    align-items: center;\n\n                    & > label {\n                        justify-self: end;\n                    }\n                }\n\n                & > button {\n                    margin-left: 0.2rem;\n                }\n            }\n\n            & > .data {\n                flex-grow: 1;\n                overflow: auto;\n\n                & > table {\n                    @extend %table;\n\n                    th {\n                        &.col-timestamp { width: 13rem; }\n                        &.col-str { width: 10rem; }\n                        &.col-int { width: 5rem; }\n                        &.col-remove { width: 4.5rem; }\n                    }\n\n                    td {\n                        &.col-remove { text-align: center; }\n\n                        &.col-timestamp {\n                            font-family: 'RobotoMono';\n                            font-size: 0.9rem;\n                        }\n                    }\n\n                    & > tbody {\n                        & > tr {\n                            &.selected, &:hover {\n                                background-color: $color-grey-400;\n                            }\n                        }\n                    }\n                }\n\n                & > .control {\n                    padding: 0.2rem;\n                    display: flex;\n                    justify-content: flex-end;\n                }\n            }\n\n            & > .panel {\n                padding: 0.2rem;\n                display: grid;\n                grid-template-columns: repeat(auto-fit, 12rem 12rem);\n                grid-gap: 0.2rem;\n                align-items: center;\n                border-top: 1px solid $color-grey-700;\n                margin-top: 0.2rem;\n\n                & > .label {\n                    justify-self: end;\n                }\n            }\n        }\n    }\n}\n","\n$shadow-key-umbra-opacity: 0.2 !default;\n$shadow-key-penumbra-opacity: 0.14 !default;\n$shadow-ambient-shadow-opacity: 0.12 !default;\n\n\n@mixin shadow-1dp() {\n  box-shadow: 0 1px 1px 0 rgba(0, 0, 0, $shadow-key-penumbra-opacity),\n              0 2px 1px -1px rgba(0, 0, 0, $shadow-key-umbra-opacity),\n              0 1px 2px 0 rgba(0, 0, 0, $shadow-ambient-shadow-opacity);\n}\n\n@mixin shadow-2dp() {\n  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, $shadow-key-penumbra-opacity),\n              0 3px 1px -2px rgba(0, 0, 0, $shadow-key-umbra-opacity),\n              0 1px 5px 0 rgba(0, 0, 0, $shadow-ambient-shadow-opacity);\n}\n\n@mixin shadow-4dp() {\n  box-shadow: 0 4px 5px 0 rgba(0, 0, 0, $shadow-key-penumbra-opacity),\n              0 1px 10px 0 rgba(0, 0, 0, $shadow-ambient-shadow-opacity),\n              0 2px 4px -1px rgba(0, 0, 0, $shadow-key-umbra-opacity);\n}\n"],"sourceRoot":""}]);
// Exports
/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (___CSS_LOADER_EXPORT___);


/***/ }),
/* 54 */
/***/ ((module) => {

"use strict";


module.exports = function (item) {
  var content = item[1];
  var cssMapping = item[3];

  if (!cssMapping) {
    return content;
  }

  if (typeof btoa === "function") {
    var base64 = btoa(unescape(encodeURIComponent(JSON.stringify(cssMapping))));
    var data = "sourceMappingURL=data:application/json;charset=utf-8;base64,".concat(base64);
    var sourceMapping = "/*# ".concat(data, " */");
    var sourceURLs = cssMapping.sources.map(function (source) {
      return "/*# sourceURL=".concat(cssMapping.sourceRoot || "").concat(source, " */");
    });
    return [content].concat(sourceURLs).concat([sourceMapping]).join("\n");
  }

  return [content].join("\n");
};

/***/ }),
/* 55 */
/***/ ((module) => {

"use strict";


/*
  MIT License http://www.opensource.org/licenses/mit-license.php
  Author Tobias Koppers @sokra
*/
module.exports = function (cssWithMappingToString) {
  var list = []; // return the list of modules as css string

  list.toString = function toString() {
    return this.map(function (item) {
      var content = "";
      var needLayer = typeof item[5] !== "undefined";

      if (item[4]) {
        content += "@supports (".concat(item[4], ") {");
      }

      if (item[2]) {
        content += "@media ".concat(item[2], " {");
      }

      if (needLayer) {
        content += "@layer".concat(item[5].length > 0 ? " ".concat(item[5]) : "", " {");
      }

      content += cssWithMappingToString(item);

      if (needLayer) {
        content += "}";
      }

      if (item[2]) {
        content += "}";
      }

      if (item[4]) {
        content += "}";
      }

      return content;
    }).join("");
  }; // import a list of modules into the list


  list.i = function i(modules, media, dedupe, supports, layer) {
    if (typeof modules === "string") {
      modules = [[null, modules, undefined]];
    }

    var alreadyImportedModules = {};

    if (dedupe) {
      for (var k = 0; k < this.length; k++) {
        var id = this[k][0];

        if (id != null) {
          alreadyImportedModules[id] = true;
        }
      }
    }

    for (var _k = 0; _k < modules.length; _k++) {
      var item = [].concat(modules[_k]);

      if (dedupe && alreadyImportedModules[item[0]]) {
        continue;
      }

      if (typeof layer !== "undefined") {
        if (typeof item[5] === "undefined") {
          item[5] = layer;
        } else {
          item[1] = "@layer".concat(item[5].length > 0 ? " ".concat(item[5]) : "", " {").concat(item[1], "}");
          item[5] = layer;
        }
      }

      if (media) {
        if (!item[2]) {
          item[2] = media;
        } else {
          item[1] = "@media ".concat(item[2], " {").concat(item[1], "}");
          item[2] = media;
        }
      }

      if (supports) {
        if (!item[4]) {
          item[4] = "".concat(supports);
        } else {
          item[1] = "@supports (".concat(item[4], ") {").concat(item[1], "}");
          item[4] = supports;
        }
      }

      list.push(item);
    }
  };

  return list;
};

/***/ })
/******/ 	]);
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			id: moduleId,
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
/******/ 	/* webpack/runtime/compat get default export */
/******/ 	(() => {
/******/ 		// getDefaultExport function for compatibility with non-harmony modules
/******/ 		__webpack_require__.n = (module) => {
/******/ 			var getter = module && module.__esModule ?
/******/ 				() => (module['default']) :
/******/ 				() => (module);
/******/ 			__webpack_require__.d(getter, { a: getter });
/******/ 			return getter;
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/define property getters */
/******/ 	(() => {
/******/ 		// define getter functions for harmony exports
/******/ 		__webpack_require__.d = (exports, definition) => {
/******/ 			for(var key in definition) {
/******/ 				if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 					Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 				}
/******/ 			}
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/hasOwnProperty shorthand */
/******/ 	(() => {
/******/ 		__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/make namespace object */
/******/ 	(() => {
/******/ 		// define __esModule on exports
/******/ 		__webpack_require__.r = (exports) => {
/******/ 			if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 				Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 			}
/******/ 			Object.defineProperty(exports, '__esModule', { value: true });
/******/ 		};
/******/ 	})();
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry need to be wrapped in an IIFE because it need to be in strict mode.
(() => {
"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony import */ var _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1);
/* harmony import */ var _hat_open_util__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(2);
/* harmony import */ var _common__WEBPACK_IMPORTED_MODULE_2__ = __webpack_require__(12);
/* harmony import */ var _vt__WEBPACK_IMPORTED_MODULE_3__ = __webpack_require__(29);
/* harmony import */ var _src_scss_main_scss__WEBPACK_IMPORTED_MODULE_4__ = __webpack_require__(46);









function main() {
    const root = document.body.appendChild(document.createElement('div'));
    _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"].init(root, _common__WEBPACK_IMPORTED_MODULE_2__.defaultState, _vt__WEBPACK_IMPORTED_MODULE_3__.main);
    _common__WEBPACK_IMPORTED_MODULE_2__.init();
}


window.addEventListener('load', main);
window.r = _hat_open_renderer__WEBPACK_IMPORTED_MODULE_0__["default"];
window.u = _hat_open_util__WEBPACK_IMPORTED_MODULE_1__;

})();

/******/ })()
;
//# sourceMappingURL=main.js.map